@@ -1,8 +1,4 @@
-{-# OPTIONS_GHC -fno-warn-orphans #-}
-{-# LANGUAGE CPP, ImpredicativeTypes #-}
-#if __GLASGOW_HASKELL__ >= 708
-{-# LANGUAGE AllowAmbiguousTypes #-}
-#endif
+{-# LANGUAGE AllowAmbiguousTypes  #-}
 --  Copyright (C) 2002-2005,2007 David Roundy
 --
 --  This program is free software; you can redistribute it and/or modify
@@ -35,19 +31,21 @@ import Darcs.Test.Patch.Utils ( testConditional )
 
 import Darcs.Patch.Witnesses.Ordered
 import Darcs.Patch.Witnesses.Sealed
-import Darcs.Patch.Witnesses.Eq ( MyEq, unsafeCompare )
+import Darcs.Patch.Witnesses.Eq ( Eq2, unsafeCompare )
 import Darcs.Patch.Witnesses.Show
-import Darcs.Patch.Prim( PrimPatch, coalesce, FromPrim, PrimOf, PrimPatchBase )
-import qualified Darcs.Patch.Prim.V1 as V1 ( Prim )
+import Darcs.Patch.Prim( PrimPatch, coalesce, FromPrim, PrimOf )
 import qualified Darcs.Patch.Prim.FileUUID as FileUUID ( Prim )
 import Darcs.Patch.RepoPatch ( RepoPatch )
 import Darcs.Patch.Type ( PatchType(..) )
 import Darcs.Patch.V1 as V1 ( RepoPatchV1 )
+import qualified Darcs.Patch.V1.Prim as V1 ( Prim )
+import qualified Darcs.Patch.V2.Prim as V2 ( Prim )
 import Darcs.Patch.V2.RepoPatch ( isConsistent, isForward, RepoPatchV2 )
-import Darcs.Patch.Patchy ( Commute(..), Patchy )
+import Darcs.Patch.Commute ( Commute(..) )
+import Darcs.Patch.Invert ( Invert )
 import Darcs.Patch.Merge( Merge )
 import Darcs.Patch.Show ( ShowPatchBasic )
-import Darcs.Patch.Apply( ApplyState )
+import Darcs.Patch.Apply( Apply, ApplyState )
 
 import Darcs.Test.Patch.Arbitrary.Generic
 import qualified Darcs.Test.Patch.Arbitrary.PrimV1 as P1
@@ -76,15 +74,16 @@ import qualified Darcs.Test.Patch.Rebase as Rebase
 import qualified Darcs.Test.Patch.WSub as WSub
 
 
-type instance ModelOf (FL prim) = ModelOf prim
+type Prim1 = V1.Prim
+type Prim2 = V2.Prim
 
-type TestGenerator thing gen = (forall t ctx . ((forall wXx wYy . thing wXx wYy -> t) -> (gen ctx -> t)))
-type TestCondition thing = (forall wYy wZz . thing wYy wZz -> Bool)
-type TestCheck thing t = (forall wYy wZz . thing wYy wZz -> t)
+newtype TestGenerator thing gen = TestGenerator (forall t ctx . ((forall wXx wYy . thing wXx wYy -> t) -> (gen ctx -> t)))
+newtype TestCondition thing = TestCondition (forall wYy wZz . thing wYy wZz -> Bool)
+newtype TestCheck thing t = TestCheck (forall wYy wZz . thing wYy wZz -> t)
 
 -- arbitraryThing :: (forall wXx wYy . thing wXx wYy -> t) -> (thing wA wB -> t)
 arbitraryThing :: x -> TestGenerator thing (thing x)
-arbitraryThing _ f p = f p
+arbitraryThing _ = TestGenerator (\f p -> f p)
 
 -- | Run a test function on a set of data, using HUnit. The test function should
 --   return @Nothing@ upon success and a @Just x@ upon failure.
@@ -123,7 +122,7 @@ unit_V2P1 =
   , testCases "FL prim recommute" (PropU.recommute WSub.commute) ExU.commutablesFL
   , testCases "FL prim patch and inverse commute" (PropU.patchAndInverseCommute WSub.commute) ExU.commutablesFL
   , testCases "FL prim inverses commute" (PropU.commuteInverses WSub.commute) $ ExU.commutablesFL
-  , testCases "fails" (PropU.commuteFails WSub.commute) ([] :: [(V1.Prim WSub.:> V1.Prim) wX wY])
+  , testCases "fails" (PropU.commuteFails WSub.commute) ([] :: [(Prim2 WSub.:> Prim2) wX wY])
   , testCases "read and show work on Prim" PropU.show_read ExU.primPatches
   , testCases "read and show work on RepoPatchV2" PropU.show_read ExU.repov2Patches
   , testCases "example flattenings work" PropU.consistentTreeFlattenings ExU.repov2PatchLoopExamples
@@ -140,27 +139,16 @@ unit_V2P1 =
   , testCases "V2 partial permutivity" (PropU.partialPermutivity WSub.commute) ExU.repov2NonduplicateTriples
   ]
 
-instance PrimPatch prim => Check (RepoPatchV2 prim) where
-  checkPatch p = return $ isNothing $ isConsistent p
-
-instance Check FileUUID.Prim where
-  checkPatch _ = return True -- XXX
-
-commuteRepoPatchV2s :: PrimPatch prim => (RepoPatchV2 prim :> RepoPatchV2 prim) wX wY -> Maybe ((RepoPatchV2 prim :> RepoPatchV2 prim) wX wY)
-commuteRepoPatchV2s = commute
-
 qc_prim :: forall prim wX wY wA model.
            (PrimPatch prim, ArbitraryPrim prim, Show2 prim
            , model ~ ModelOf prim, RepoModel model
            , RepoState model ~ ApplyState (PrimOf prim)
            , Show1 (ModelOf prim)
-           , Check prim, PrimPatchBase prim, PrimOf prim ~ prim
+           , Check prim, PrimOf prim ~ prim
            , FromPrim prim
            , MightBeEmptyHunk prim
            , MightHaveDuplicate prim
            , Show1 (prim wA)
-           , Show1 ((prim :> prim) wA)
-           , Show1 (WithState model prim wA)
            , Arbitrary (Sealed ((prim :> prim) wA))
            , Arbitrary (Sealed ((prim :> prim :> prim) wA))
            , Arbitrary (Sealed (prim wA))
@@ -195,12 +183,18 @@ qc_prim p =
   ]
       where arbitraryThing' = arbitraryThing (undefined :: wA) -- bind the witness for generator
 
+consistentV2 :: RepoPatchV2 Prim2 wX wY -> TestResult
+consistentV2 = fromMaybe . isConsistent
+
+commuteRepoPatchV2s :: (RepoPatchV2 Prim2 :> RepoPatchV2 Prim2) wX wY -> Maybe ((RepoPatchV2 Prim2 :> RepoPatchV2 Prim2) wX wY)
+commuteRepoPatchV2s = commute
+
 qc_V2P1 :: [Test]
 qc_V2P1 =
   [ testProperty "tree flattenings are consistent... "
-    (PropR.propConsistentTreeFlattenings :: Sealed (WithStartState (ModelOf V1.Prim) (Tree V1.Prim)) -> Bool)
+    (PropR.propConsistentTreeFlattenings :: Sealed (WithStartState (ModelOf Prim2) (Tree Prim2)) -> Bool)
   , testProperty "with quickcheck that RepoPatchV2 patches are consistent... "
-    (unseal $ P1.patchFromTree $ fromMaybe . isConsistent)
+    (unseal $ P1.patchFromTree $ consistentV2)
   -- permutivity ----------------------------------------------------------------------------
   , testConditional "permutivity"
     (unseal $ P1.commuteTripleFromTree notDuplicatestriple)
@@ -214,7 +208,7 @@ qc_V2P1 =
   ]
 
 qc_V2 :: forall prim wXx wYy . (PrimPatch prim, Show1 (ModelOf prim), RepoModel (ModelOf prim),
-                                  Check (RepoPatchV2 prim), ArbitraryPrim prim, Show2 prim,
+                                  ArbitraryPrim prim, Show2 prim,
                                   RepoState (ModelOf prim) ~ ApplyState prim)
       => prim wXx wYy -> [Test]
 qc_V2 _ =
@@ -228,11 +222,11 @@ qc_V2 _ =
                      -> Maybe (Tree (RepoPatchV2 prim) wX)))
   ]
   ++ concat
-  [ merge_properties   (undefined :: RepoPatchV2 prim wX wY) "tree" mergePairFromTree
-  , merge_properties   (undefined :: RepoPatchV2 prim wX wY) "twfp" mergePairFromTWFP
-  , pair_properties    (undefined :: RepoPatchV2 prim wX wY) "tree" commutePairFromTree
-  , pair_properties    (undefined :: RepoPatchV2 prim wX wY) "twfp" commutePairFromTWFP
-  , patch_properties   (undefined :: RepoPatchV2 prim wX wY) "tree" patchFromTree
+  [ merge_properties   (undefined :: RepoPatchV2 prim wX wY) "tree" (TestGenerator mergePairFromTree)
+  , merge_properties   (undefined :: RepoPatchV2 prim wX wY) "twfp" (TestGenerator mergePairFromTWFP)
+  , pair_properties    (undefined :: RepoPatchV2 prim wX wY) "tree" (TestGenerator commutePairFromTree)
+  , pair_properties    (undefined :: RepoPatchV2 prim wX wY) "twfp" (TestGenerator commutePairFromTWFP)
+  , patch_properties   (undefined :: RepoPatchV2 prim wX wY) "tree" (TestGenerator patchFromTree)
   ]
 
 properties :: forall thing gen. (Show1 gen, Arbitrary (Sealed gen)) =>
@@ -241,86 +235,98 @@ properties :: forall thing gen. (Show1 gen, Arbitrary (Sealed gen)) =>
            -> String -> String
            -> forall t. Testable t => [(String, TestCondition thing, TestCheck thing t)]
            -> [Test]
-properties gen prefix genname tests =
+properties (TestGenerator gen) prefix genname tests =
   [ cond name condition check | (name, condition, check) <- tests ]
   where cond :: forall testable. Testable testable
              => String -> TestCondition thing -> TestCheck thing testable -> Test
-        cond t c p =
+        cond t (TestCondition c) (TestCheck p) =
           testConditional (prefix ++ " (" ++ genname ++ "): " ++ t) (unseal $ gen c) (unseal $ gen p)
 
 type PropList what gen = String -> TestGenerator what gen -> [Test]
 
 pair_properties :: forall p gen x y
-                 . ( Show1 gen, Arbitrary (Sealed gen), Patchy p, MightHaveDuplicate p
-                   , ShowPatchBasic p, MyEq p
+                 . ( Show1 gen, Arbitrary (Sealed gen), MightHaveDuplicate p
+                   , Commute p, Invert p, ShowPatchBasic p, Eq2 p
                    )
                 => p x y -> PropList (p :> p) gen
 pair_properties _ genname gen =
   properties gen "commute" genname
-  [ ("recommute"              , const True       , PropG.recommute commute             )
-  , ("nontrivial recommute"   , nontrivialCommute, PropG.recommute commute             )
-  , ("inverses commute"       , const True       , PropG.commuteInverses commute       )
-  , ("nontrivial inverses"    , nontrivialCommute, PropG.commuteInverses commute       )
-  , ("inverse composition"    , const True       , PropG.inverseComposition            )
+  [ ("recommute"              , TestCondition (const True)     , TestCheck (PropG.recommute commute)           )
+  , ("nontrivial recommute"   , TestCondition nontrivialCommute, TestCheck (PropG.recommute commute)           )
+  , ("inverses commute"       , TestCondition (const True)     , TestCheck (PropG.commuteInverses commute)     )
+  , ("nontrivial inverses"    , TestCondition nontrivialCommute, TestCheck (PropG.commuteInverses commute)     )
+  , ("inverse composition"    , TestCondition (const True)     , TestCheck PropG.inverseComposition            )
   ]
 
 coalesce_properties :: forall p gen x y
-                     . ( Show1 gen, Arbitrary (Sealed gen), Patchy p, PrimPatch p
+                     . ( Show1 gen, Arbitrary (Sealed gen), PrimPatch p
                        , ArbitraryPrim p, MightBeEmptyHunk p
                        )
                     => p x y -> PropList (p :> p :> p) gen
 coalesce_properties p genname gen =
   properties gen "commute" genname
-   (if runCoalesceTests p then [ ("coalesce commutes with commute", const True, PropG.coalesceCommute coalesce) ] else [])
+   (if runCoalesceTests p then [ ("coalesce commutes with commute", TestCondition (const True), TestCheck (PropG.coalesceCommute coalesce)) ] else [])
 
 -- The following properties do not hold for "RepoPatchV2" patches (conflictors and
 -- duplicates, specifically) .
 nonrpv2_commute_properties :: forall p gen x y
-                            . (Show1 gen, Arbitrary (Sealed gen), Patchy p, ShowPatchBasic p, MyEq p)
+                            . (Show1 gen, Arbitrary (Sealed gen), Commute p, Invert p, ShowPatchBasic p, Eq2 p)
                            => p x y -> PropList (p :> p) gen
 nonrpv2_commute_properties _ genname gen =
   properties gen "commute" genname
-  [ ("patch & inverse commute", const True       , PropG.patchAndInverseCommute commute)
-  , ("patch & inverse commute", nontrivialCommute, PropG.patchAndInverseCommute commute)
+  [ ("patch & inverse commute", TestCondition (const True)     , TestCheck (PropG.patchAndInverseCommute commute))
+  , ("patch & inverse commute", TestCondition nontrivialCommute, TestCheck (PropG.patchAndInverseCommute commute))
   ]
 
-patch_properties :: forall p gen x y. (Show1 gen, Arbitrary (Sealed gen), Patchy p, MyEq p)
+patch_properties :: forall p gen x y. (Show1 gen, Arbitrary (Sealed gen), Invert p, Apply p, Eq2 p)
                  => p x y -> PropList p gen
 patch_properties _ genname gen =
   properties gen "patch" genname
-  [ ("inverse . inverse is id"  , const True       , PropG.invertSymmetry)
+  [ ("inverse . inverse is id"  , TestCondition (const True)     , TestCheck PropG.invertSymmetry)
   ]
 
 patch_repo_properties :: forall p gen x y. (Show1 gen, Arbitrary (Sealed gen),
-                                            Patchy p, ShowPatchBasic p,
+                                            Invert p, Apply p, ShowPatchBasic p,
                                             RepoModel (ModelOf (PrimOf p)),
                                             RepoState (ModelOf (PrimOf p)) ~ ApplyState p)
                       => p x y -> PropList (WithState (ModelOf (PrimOf p)) p) gen
 patch_repo_properties _ genname gen =
   properties gen "patch/repo" genname
-  [ ("invert rollback"          , const True       , PropG.invertRollback)
+  [ ("invert rollback"          , TestCondition (const True)     , TestCheck PropG.invertRollback)
   ]
 
-pair_repo_properties :: forall p gen x y. (Show1 gen, Arbitrary (Sealed gen), Patchy p,
-                                            MightBeEmptyHunk p,
-                                            RepoModel (ModelOf p),
-                                            RepoState (ModelOf p) ~ ApplyState p)
-                      => p x y -> PropList (WithState (ModelOf p) (p :> p)) gen
+pair_repo_properties
+  :: forall p gen x y.
+     ( Show1 gen
+     , Arbitrary (Sealed gen)
+     , Commute p
+     , Apply p
+     , ShowPatchBasic p
+     , MightBeEmptyHunk p
+     , RepoModel (ModelOf p)
+     , RepoState (ModelOf p) ~ ApplyState p
+     )
+  => p x y -> PropList (WithState (ModelOf p) (p :> p)) gen
 pair_repo_properties _ genname gen =
-  properties gen "patch/repo" genname
-  [ ("commute is effect preserving" , const True       , PropG.effectPreserving commute )
-  ]
+  properties
+    gen
+    "patch/repo"
+    genname
+    [ ( "commute is effect preserving"
+      , TestCondition (const True)
+      , TestCheck (PropG.effectPreserving commute))
+    ]
 
 merge_properties :: forall p gen x y. (Show1 gen, Arbitrary (Sealed gen)
-                                      , Patchy p, MyEq p, Merge p, ShowPatchBasic p
+                                      , Invert p, Eq2 p, Merge p, ShowPatchBasic p
                                       , MightHaveDuplicate p, Show2 p, Check p)
                  => p x y -> PropList (p :\/: p) gen
 merge_properties _ genname gen =
   properties gen "merge" genname
-  [ ("merge either way"           , const True     , PropG.mergeEitherWay      )
-  , ("merge either way valid"     , const True     , Prop1.tMergeEitherWayValid)
-  , ("nontrivial merge either way", nontrivialMerge, PropG.mergeEitherWay      )
-  , ("merge commute"              , const True     , PropG.mergeCommute        )
+  [ ("merge either way"           , TestCondition (const True)   , TestCheck PropG.mergeEitherWay      )
+  , ("merge either way valid"     , TestCondition (const True)   , TestCheck Prop1.tMergeEitherWayValid)
+  , ("nontrivial merge either way", TestCondition nontrivialMerge, TestCheck PropG.mergeEitherWay      )
+  , ("merge commute"              , TestCondition (const True)   , TestCheck PropG.mergeCommute        )
   ]
 
 qc_V1P1 :: [Test]
@@ -357,10 +363,10 @@ qc_V1P1 =
   ] -- the following properties are disabled, because they routinely lead to
     -- exponential cases, making the tests run for ever and ever; nevertheless,
     -- we would expect them to hold
- {- ++ merge_properties (undefined :: V1.RepoPatchV1 Prim wX wY) "tree" mergePairFromTree
-    ++ merge_properties (undefined :: V1.RepoPatchV1 Prim wX wY) "twfp" mergePairFromTWFP
-    ++ commute_properties (undefined :: V1.RepoPatchV1 Prim wX wY) "tree" commutePairFromTree
-    ++ commute_properties (undefined :: V1.RepoPatchV1 Prim wX wY) "twfp" commutePairFromTWFP -}
+ {- ++ merge_properties (undefined :: V1.RepoPatchV1 Prim1 wX wY) "tree" mergePairFromTree
+    ++ merge_properties (undefined :: V1.RepoPatchV1 Prim1 wX wY) "twfp" mergePairFromTWFP
+    ++ commute_properties (undefined :: V1.RepoPatchV1 Prim1 wX wY) "tree" commutePairFromTree
+    ++ commute_properties (undefined :: V1.RepoPatchV1 Prim1 wX wY) "twfp" commutePairFromTWFP -}
 
 -- tests (either QuickCheck or Unit) that should be run on any type of patch
 general_patchTests :: (RepoPatch p, ArbitraryPrim (PrimOf p), Show2 (PrimOf p)) => PatchType rt p -> [Test]
@@ -370,16 +376,22 @@ general_patchTests pt =
 
 -- | This is the big list of tests that will be run using testrunner.
 testSuite :: [Test]
-testSuite = [ testGroup "Darcs.Patch.Prim.V1" $ qc_prim (undefined :: V1.Prim wX wY)
-            , testGroup "Darcs.Patch.V1 (using Prim.V1)" $
-                unit_V1P1 ++ qc_V1P1 ++ general_patchTests (PatchType :: PatchType rt (V1.RepoPatchV1 V1.Prim))
-            , testGroup "Darcs.Patch.V2 (using Prim.V1)" $
-                unit_V2P1 ++ qc_V2 (undefined :: V1.Prim wX wY) ++ qc_V2P1 ++
-                general_patchTests (PatchType :: PatchType rt (RepoPatchV2 V1.Prim))
-            -- , testGroup "Darcs.Patch.Prim.FileUUID" $ qc_prim (undefined :: FileUUID.Prim wX wY)
-            , testGroup "Darcs.Patch.V2 (using Prim.FileUUID)" $
-                qc_V2 (undefined :: FileUUID.Prim wX wY) ++
-                general_patchTests (PatchType :: PatchType rt (RepoPatchV2 FileUUID.Prim))
-            , Darcs.Test.Patch.Info.testSuite
-            , Darcs.Test.Patch.Selection.testSuite
-            ]
+testSuite =
+  [ testGroup "Darcs.Patch.Prim.V1 for V1" $ qc_prim (undefined :: Prim1 wX wY)
+    -- testing both Prim1 and Prim2 here is redundant, since they differ
+    -- only in their read/show behavior, which is not tested in qc_prim;
+    -- we still include them because such tests might be added in the future
+  , testGroup "Darcs.Patch.Prim.V1 for V2" $ qc_prim (undefined :: Prim2 wX wY)
+  , testGroup "Darcs.Patch.Prim.FileUUID" $ qc_prim (undefined :: FileUUID.Prim wX wY)
+  , testGroup "Darcs.Patch.V1 (using Prim.V1)" $
+      unit_V1P1 ++ qc_V1P1 ++
+      general_patchTests (PatchType :: PatchType rt (V1.RepoPatchV1 Prim1))
+  , testGroup "Darcs.Patch.V2 (using Prim.V1)" $
+      unit_V2P1 ++ qc_V2 (undefined :: Prim2 wX wY) ++ qc_V2P1 ++
+      general_patchTests (PatchType :: PatchType rt (RepoPatchV2 Prim2))
+  , testGroup "Darcs.Patch.V2 (using Prim.FileUUID)" $
+      qc_V2 (undefined :: FileUUID.Prim wX wY) ++
+      general_patchTests (PatchType :: PatchType rt (RepoPatchV2 FileUUID.Prim))
+  , Darcs.Test.Patch.Info.testSuite
+  , Darcs.Test.Patch.Selection.testSuite
+  ]
