@@ -27,9 +27,8 @@ import Darcs.Test.Util.QuickCheck ( notIn, maybeOf )
 
 import Darcs.Patch.Prim
 
-import Control.Applicative ( (<$>) )
 import qualified Data.ByteString as B
-import Data.Maybe ( isJust )
+import Data.Maybe ( fromJust, isJust )
 import qualified Data.Map as M
 import Darcs.Util.Hash( Hash(..) )
 
