@@ -36,7 +36,7 @@ import Darcs.Patch.Prim
 import Darcs.Patch.Apply ( ApplyState )
 
 import qualified Data.ByteString.Char8 as BC
-import Data.Maybe ( isJust )
+import Data.Maybe ( fromJust, isJust )
 
 type Prim1 = V1.Prim
 type Prim2 = V2.Prim
