@@ -33,10 +33,10 @@ import Darcs.Patch.Witnesses.Show
 import Darcs.Util.Path ( Name, unsafeMakeName )
 import Darcs.Util.Hash( Hash(..) )
 
-import Control.Applicative ( (<$>) )
 import qualified Data.ByteString as B
 import qualified Data.ByteString.Char8 as BC
 import qualified Data.Map as M
+import Data.Maybe ( fromJust )
 import Test.QuickCheck
   ( Arbitrary(..)
   , Gen, choose, vectorOf, frequency, oneof )
