@@ -1,5 +1,3 @@
-{-# LANGUAGE CPP #-}
-{-# OPTIONS_GHC -fno-warn-orphans #-}
 module Darcs.Test.Patch.Properties.V1Set1
        ( checkMerge, checkMergeEquiv, checkMergeSwap, checkCanon
        , checkCommute, checkCantCommute
@@ -7,16 +5,17 @@ module Darcs.Test.Patch.Properties.V1Set1
        , tMergeEitherWayValid, tTestCheck ) where
 
 import Darcs.Patch
-     ( Patchy, commute, invert, merge, effect
+     ( commute, invert, merge, effect
      , readPatch, showPatch
      , fromPrim, canonize, sortCoalesceFL )
-import Darcs.Patch.Prim.V1 ( Prim )
+import Darcs.Patch.Invert ( Invert )
 import Darcs.Patch.Merge ( Merge )
 import Darcs.Patch.Read ( ReadPatch )
-import Darcs.Patch.Show ( ShowPatchBasic )
+import Darcs.Patch.Show ( ShowPatchBasic, ShowPatchFor(..) )
 import qualified Darcs.Patch.V1 as V1 ( RepoPatchV1 )
+import qualified Darcs.Patch.V1.Prim as V1 ( Prim(..) )
 import Darcs.Test.Patch.Properties.Check ( checkAPatch, Check )
-import Darcs.Util.Printer ( renderPS, RenderMode(..) )
+import Darcs.Util.Printer ( renderPS )
 import Darcs.Patch.Witnesses.Eq
 import Darcs.Patch.Witnesses.Ordered
 import Darcs.Patch.Witnesses.Show
@@ -27,14 +26,14 @@ import qualified Darcs.Util.Diff as D (DiffAlgorithm(..))
 import Darcs.Util.Printer ( text )
 
 
-type Patch = V1.RepoPatchV1 Prim
+type Patch = V1.RepoPatchV1 V1.Prim
 
 
-quickmerge :: (Patchy p, Merge p) => (p :\/: p ) wX wY -> p wY wZ
+quickmerge :: (Merge p) => (p :\/: p ) wX wY -> p wY wZ
 quickmerge (p1:\/:p2) = case merge (p1:\/:p2) of
                         _ :/\: p1' -> unsafeCoercePEnd p1'
 
-instance MyEq p => Eq ((p :/\: p) wX wY) where
+instance Eq2 p => Eq ((p :/\: p) wX wY) where
    (x :/\: y) == (x' :/\: y') = isIsEq (x =\/= x') && isIsEq (y =\/= y')
 
 -- ----------------------------------------------------------------------------
@@ -130,15 +129,15 @@ checkCantCommute (p2 :> p1) =
 -- A few "test" properties, doing things with input patches and giving a OK/not
 -- OK type of answer.
 
-tShowRead :: (Show2 p, Patchy p, ReadPatch p, ShowPatchBasic p)
+tShowRead :: (Show2 p, ReadPatch p, ShowPatchBasic p)
           => (forall wX wY wW wZ . p wX wY -> p wW wZ -> Bool) -> forall wX wY . p wX wY -> TestResult
 tShowRead eq p =
-    case readPatch $ renderPS Standard $ showPatch p of
+    case readPatch $ renderPS $ showPatch ForStorage p of
     Just (Sealed p') -> if p' `eq` p then succeeded
                         else failed $ text $ "Failed to read shown:  "++(show2 p)++"\n"
     Nothing -> failed $ text $ "Failed to read at all:  "++(show2 p)++"\n"
 
-tMergeEitherWayValid :: forall wX wY p . (Check p, Show2 p, Merge p, Patchy p) => (p :\/: p) wX wY -> TestResult
+tMergeEitherWayValid :: forall wX wY p . (Check p, Show2 p, Merge p, Invert p) => (p :\/: p) wX wY -> TestResult
 tMergeEitherWayValid (p1 :\/: p2) =
   case p2 :>: quickmerge (p1:\/: p2) :>: NilFL of
   combo2 ->
