@@ -2,7 +2,6 @@ module Darcs.Test.Patch.RepoModel where
 import Darcs.Patch.Apply ( Apply, ApplyState )
 import Darcs.Patch.Witnesses.Ordered ( FL )
 
-import Control.Monad.Fail ( MonadFail(fail) )
 import Test.QuickCheck ( Gen )
 
 data Fail a = Failed String | OK a
@@ -23,9 +22,6 @@ instance Monad Fail where
   Failed s >>= _ = Failed s
   OK v >>= f = f v
 
-instance MonadFail Fail where
-  fail = Failed
-
 unFail :: Fail t -> t
 unFail (OK x) = x
 unFail (Failed err) = error $ "unFail failed: " ++ err
