@@ -3,26 +3,16 @@
 {-# OPTIONS_GHC -fno-warn-type-defaults #-}
 module Main ( main ) where
 
-#ifdef DISABLE_TESTING
-main :: IO ()
-main = fail $   "test infrastructure not built,"
-             ++ " please pass --enable-tests to configure, then rebuild"
-#else
-
 import qualified Darcs.Test.Misc
 import qualified Darcs.Test.Patch
 import qualified Darcs.Test.Email
-import qualified Storage.Hashed.Test
+import qualified Darcs.Test.Repository.Inventory
+import qualified Darcs.Test.HashedStorage
 
 import Control.Monad ( filterM )
 import Control.Exception ( SomeException )
-import qualified Control.Monad.Trans
 import Data.Text ( Text, pack, unpack )
-import Data.Char ( toLower )
 import Data.List ( isPrefixOf, isSuffixOf, sort )
-import Data.List.Split ( splitOn )
-import qualified Data.ByteString.Char8 as B
-import Data.Maybe ( fromMaybe )
 import GHC.IO.Encoding ( textEncodingName )
 import System.Console.CmdArgs hiding ( args )
 import System.Directory ( doesFileExist )
@@ -40,13 +30,14 @@ doUnit = return unitTests
 
 -- | TODO make runnable in parallel
 doHashed :: IO [Test]
-doHashed = return Storage.Hashed.Test.tests
+doHashed = return Darcs.Test.HashedStorage.tests
 
 -- | This is the big list of tests that will be run using testrunner.
 unitTests :: [Test]
 unitTests =
   [ Darcs.Test.Email.testSuite
   , Darcs.Test.Misc.testSuite
+  , Darcs.Test.Repository.Inventory.testSuite
   ] ++ Darcs.Test.Patch.testSuite
 
 -- ----------------------------------------------------------------------
@@ -72,7 +63,7 @@ data ShellTest = ShellTest { format :: Format
                            , testfile :: FilePath
                            , testdir  :: Maybe FilePath -- ^ only if you want to set it explicitly
                            , _darcspath :: FilePath
-                           , _diffalgorithm :: DiffAlgorithm
+                           , diffalgorithm :: DiffAlgorithm
                            }
                  deriving Typeable
 
@@ -104,17 +95,15 @@ runtest' (ShellTest fmt _ _ dp da) srcdir =
                        case code of
                         200 -> return Skipped
                         _   -> Failed <$> unpack <$> lastStderr
-  where defaults = pack $ unlines (["ALL " ++ fmtstr, "send no-edit-description", "ALL ignore-times"] ++ dcs)
+  where defaults = pack $ unlines
+          [ "ALL " ++ fmtstr
+          , "send no-edit-description"
+          , "ALL ignore-times"
+          , "ALL " ++ daf
+          ]
         fmtstr = case fmt of
                   Darcs2 -> "darcs-2"
                   Darcs1 -> "darcs-1"
-        dcs = [dc ++ " " ++ daf | dc <- ["revert","unrevert", "whatsnew",
-                                         "record", "unpull", "obliterate",
-                                         "amend-record", "mark-conflicts",
-                                         "rebase", "pull", "repair",
-                                         "rollback", "apply",
-                                         "rebase pull", "rebase suspend",
-                                         "rebase unsuspend", "rebase obliterate"] ]
         daf = case da of
                 PatienceDiff -> "patience"
                 MyersDiff -> "myers"
@@ -125,33 +114,19 @@ runtest' (ShellTest fmt _ _ dp da) srcdir =
         pathVarSeparator = ":"
 #endif
 
--- TODO: add a 'all' option (implement using an Enum instance)?
-readOptionList :: (String -> a) -> (String -> [a])
-readOptionList readElem str = map readElem (splitOn "," str)
-
-readDiffAlgorithm :: String -> DiffAlgorithm
-readDiffAlgorithm (map toLower -> "myers") = MyersDiff
-readDiffAlgorithm (map toLower -> "patience") = PatienceDiff
-readDiffAlgorithm _ = error "Valid diff algorithms: myers, patience"
-
-readRepoFormat :: String -> Format
-readRepoFormat (map toLower -> "darcs-1") = Darcs1
-readRepoFormat (map toLower -> "darcs-2") = Darcs2
-readRepoFormat _ = error "Valid repo formats: darcs-1, darcs-2"
-
 runtest :: ShellTest -> Sh Result
 runtest t =
  withTmp $ \dir -> do
   cp "tests/lib" dir
   cp "tests/network/sshlib" dir
-  cp ("tests" </> testfile t) (dir </> "test")
+  cp (fromText $ pack $ testfile t) (dir </> "test")
   srcdir <- pwd
   silently $ sub $ cd dir >> runtest' t (toTextIgnore srcdir)
  where
   withTmp =
    case testdir t of
      Just dir -> \job -> do
-       let d = (dir </> show (format t) </> takeBaseName (testfile t))
+       let d = (dir </> show (format t) </> show (diffalgorithm t) </> takeBaseName (testfile t))
        mkdir_p d
        job d
      Nothing  -> withTmpDir
@@ -162,53 +137,22 @@ instance Testlike Running Result ShellTest where
                                        liftIO (shelly $ runtest test)
 
 shellTest :: FilePath -> Format -> Maybe FilePath -> String -> DiffAlgorithm -> Test
-shellTest dp fmt tdir file da = Test (file ++ " (" ++ show fmt ++ ")" ++ " (" ++ show da ++ ")") $ ShellTest fmt file tdir dp da
-
-hasPrefix :: B.ByteString -> B.ByteString -> Maybe B.ByteString
-hasPrefix prefix =
-    let len = B.length prefix in
-    \str -> if B.take len str == prefix then Just (B.drop len str) else Nothing
+shellTest dp fmt tdir file da =
+  Test (takeBaseName file ++ " (" ++ show fmt ++ ")" ++ " (" ++ show da ++ ")") $
+  ShellTest fmt file tdir dp da
 
 toString :: Shelly.FilePath -> String
 toString = unpack . toTextIgnore
 
--- use of a pragma in a test script overrides the user's selection for that particular test,
--- based on the assumption that the test author knows best
-parsePragmas :: FilePath -> FilePath -> IO (FilePath, (Maybe [DiffAlgorithm], Maybe [Format]))
-parsePragmas path file = do
-  contents <- B.lines <$> B.readFile (toString $ path </> file)
-  let parseLine
-          (_diffAlgorithms, repoFormats)
-          (hasPrefix (B.pack "#pragma diff-algorithm ") -> Just (readOptionList readDiffAlgorithm . B.unpack -> newDiffAlgorithms))
-         = (Just newDiffAlgorithms, repoFormats)
-      parseLine
-          (diffAlgorithms, _repoFormats)
-          (hasPrefix (B.pack "#pragma repo-format ") -> Just (readOptionList readRepoFormat . B.unpack -> newRepoFormats))
-         = (diffAlgorithms, Just newRepoFormats)
-      parseLine _ (hasPrefix (B.pack "#pragma ") -> Just pragma) = error $ "Unknown pragma " ++ B.unpack pragma ++ " in " ++ (toString $ path </> file)
-      parseLine x _ = x
-      pragmas = foldl parseLine (Nothing, Nothing) contents
-  return (file, pragmas)
-
-findShell :: FilePath -> Maybe FilePath -> Bool -> [DiffAlgorithm] -> [Format] -> Sh [Test]
-findShell dp tdir isFailing diffAlgorithmsDefault repoFormatsDefault =
-  do allFiles <- map (drop (length ("tests/"::String)) . toString)  <$> ls (fromText "tests")
-     let files = sort $ filter relevant $ filter (".sh" `isSuffixOf`) allFiles
-     annotatedFiles <- Control.Monad.Trans.liftIO $ mapM (parsePragmas "tests") files
+findShell :: FilePath -> Text -> Maybe FilePath -> Bool -> [DiffAlgorithm] -> [Format] -> Sh [Test]
+findShell dp sdir tdir isFailing diffAlgorithms repoFormats =
+  do files <- ls (fromText sdir)
+     let test_files = sort $ filter relevant $ filter (hasExt "sh") files
      return [ shellTest dp fmt tdir file da
-            | (file, (diffAlgorithmsPragma, repoFormatsPragma)) <- annotatedFiles
-            , fmt <- fromMaybe repoFormatsDefault repoFormatsPragma
-            , da <- fromMaybe diffAlgorithmsDefault diffAlgorithmsPragma ]
-  where relevant = (if isFailing then id else not) . ("failing-" `isPrefixOf`)
-
-findNetwork :: FilePath -> Maybe FilePath -> [DiffAlgorithm] -> [Format] -> Sh [Test]
-findNetwork dp tdir diffAlgorithmsDefault repoFormatsDefault =
-  do files <- sort <$> filter (".sh" `isSuffixOf`) <$> map (drop (length ("tests/network/"::String)) . toString) <$> ls "tests/network"
-     annotatedFiles <- Control.Monad.Trans.liftIO $ mapM (parsePragmas "tests/network") files
-     return [ shellTest dp fmt tdir (toString $ "network" </> file) da
-            | (file, (diffAlgorithmsPragma, repoFormatsPragma)) <- annotatedFiles
-            , fmt <- fromMaybe repoFormatsDefault repoFormatsPragma
-            , da <- fromMaybe diffAlgorithmsDefault diffAlgorithmsPragma ]
+            | file <- map toString test_files
+            , fmt <- repoFormats
+            , da <- diffAlgorithms ]
+  where relevant = (if isFailing then id else not) . ("failing-" `isPrefixOf`) . takeBaseName . toString
 
 -- ----------------------------------------------------------------------
 -- harness
@@ -223,6 +167,7 @@ data Config = Config { hashed :: Bool
                      , patience :: Bool
                      , darcs1 :: Bool
                      , darcs2 :: Bool
+                     , full :: Bool
                      , darcs :: String
                      , tests :: [String]
                      , testDir :: Maybe FilePath
@@ -240,12 +185,15 @@ defaultConfig
      [ hashed        := False    += help "Run hashed-storage tests [no]"
      , failing       := False    += help "Run the failing (shell) tests [no]"
      , shell         := True     += help "Run the passing, non-network shell tests [yes]"
+-- RELEASE BRANCH ONLY: disable network tests (too fragile)
+--     , network       := True     += help "Run the network shell tests [yes]"
      , network       := False    += help "Run the network shell tests [no]"
      , unit          := True     += help "Run the unit tests [yes]"
      , myers         := False    += help "Use myers diff [no]"
      , patience      := True     += help "Use patience diff [yes]" += name "p"
      , darcs1        := False    += help "Use darcs-1 repo format [no]" += name "1"
      , darcs2        := True     += help "Use darcs-2 repo format [yes]" += name "2"
+     , full          := False    += help "Run all tests in all variants"
      , darcs         := ""       += help "Darcs binary path" += typ "PATH"
      , tests         := []       += help "Pattern to limit the tests to run" += typ "PATTERN" += name "t"
      , testDir       := Nothing  += help "Directory to run tests in" += typ "PATH" += name "d"
@@ -306,12 +254,14 @@ run conf = do
                       . (if patience conf then (PatienceDiff:) else id)
                       $ []
 
-    ftests <- shelly $ if failing conf then findShell darcsBin (testDir conf) True diffAlgorithm repoFormat else return []
-    stests <- shelly $ if shell conf then findShell darcsBin (testDir conf) False diffAlgorithm repoFormat else return []
+    stests <- shelly $
+      if shell conf
+        then findShell darcsBin "tests" (testDir conf) (failing conf) diffAlgorithm repoFormat
+        else return []
     utests <- if unit conf then doUnit else return []
-    ntests <- shelly $ if network conf then findNetwork darcsBin (testDir conf) diffAlgorithm repoFormat else return []
+    ntests <- shelly $ if network conf then findShell darcsBin "tests/network" (testDir conf) (failing conf) diffAlgorithm repoFormat else return []
     hstests <- if hashed conf then doHashed else return []
-    defaultMainWithArgs (ftests ++ stests ++ utests ++ ntests ++ hstests) args
+    defaultMainWithArgs (stests ++ utests ++ ntests ++ hstests) args
        where
           exeSuffix :: String
 #ifdef WIN32
@@ -326,6 +276,15 @@ main = do hSetBinaryMode stdout True
           hSetBinaryMode stderr True
           hSetBinaryMode stdin True
           clp  <- cmdArgs_ defaultConfig
-          run clp
-
-#endif
+          run $
+            if full clp then clp
+              { hashed   = True
+              , shell    = True
+              , network  = True
+              , unit     = True
+              , myers    = True
+              , patience = True
+              , darcs1   = True
+              , darcs2   = True
+              }
+            else clp
