@@ -1,4 +1,4 @@
-{-# LANGUAGE CPP, OverloadedStrings, TypeSynonymInstances, MultiParamTypeClasses #-}
+{-# LANGUAGE OverloadedStrings, TypeSynonymInstances, MultiParamTypeClasses #-}
 
 -- Copyright (C) 2010 Petr Rockai
 --
@@ -32,16 +32,19 @@
 
 module Darcs.Patch.Annotate
     (
-      annotate
+      annotateFile
     , annotateDirectory
     , format
     , machineFormat
     , AnnotateResult
+    , Annotate(..)
     ) where
 
 import Prelude ()
 import Darcs.Prelude
 
+import Control.Monad.State ( modify, modify', when, gets, State, execState )
+
 import qualified Data.ByteString as B
 import qualified Data.ByteString.Char8 as BC
 import qualified Data.Map as M
@@ -51,175 +54,171 @@ import Data.Function ( on )
 import Data.List( nub, groupBy )
 import Data.Maybe( isJust, mapMaybe )
 
-import Control.Monad.State ( modify, when, gets, State, execState )
+import qualified Darcs.Patch.Prim.FileUUID as FileUUID
 
-import Darcs.Patch.ApplyMonad( ApplyMonad(..), ApplyMonadTree(..) )
-import Darcs.Patch.Apply ( Apply, apply, ApplyState )
-import Darcs.Patch.Info ( PatchInfo(..), showPatchInfoUI, piAuthor, makePatchname )
-import Darcs.Patch.PatchInfoAnd( info, PatchInfoAnd )
+import Darcs.Patch.Info ( PatchInfo(..), displayPatchInfo, piAuthor, makePatchname )
+import Darcs.Patch.Named ( Named(..) )
+import Darcs.Patch.Named.Wrapped ( WrappedNamed(..) )
+import Darcs.Patch.PatchInfoAnd( info, PatchInfoAnd, hopefully )
+import Darcs.Patch.Prim.V1.Core ( Prim(..), DirPatchType(..), FilePatchType(..) )
+import Darcs.Patch.TokenReplace ( annotateReplace )
 import Darcs.Patch.Witnesses.Ordered
 
-import Darcs.Util.Tree( Tree )
 import Darcs.Util.Path ( FileName, movedirfilename, fn2ps, ps2fn )
-import Darcs.Util.Printer( renderString, RenderMode(..) )
-import Darcs.Util.ByteString ( linesPS, unlinesPS )
-import Darcs.Util.Diff ( getChanges )
-import qualified Darcs.Util.Diff as D ( DiffAlgorithm )
-
-#include "impossible.h"
-
+import Darcs.Util.Printer( renderString )
+import Darcs.Util.ByteString ( linesPS, decodeLocale )
 
 data FileOrDirectory = File
                      | Directory
                        deriving (Show, Eq)
 
+type AnnotateResult = V.Vector (Maybe PatchInfo, B.ByteString)
+
 data Annotated = Annotated
-    { annotated     :: V.Vector (Maybe PatchInfo, B.ByteString)
-    , current       :: [(Int, B.ByteString)]
-    , path          :: Maybe FileName
+    { annotated     :: !AnnotateResult
+    , current       :: ![(Int, B.ByteString)]
+    , path          :: (Maybe FileName)
     , what          :: FileOrDirectory
     , currentInfo   :: PatchInfo
-    , diffAlgorithm :: D.DiffAlgorithm
     } deriving Show
 
-type AnnotateResult = V.Vector (Maybe PatchInfo, B.ByteString)
-
 type AnnotatedM = State Annotated
 
-instance ApplyMonad Tree AnnotatedM where
-  type ApplyMonadBase AnnotatedM = AnnotatedM
+class Annotate p where
+  annotate :: p wX wY -> AnnotatedM ()
 
-  nestedApply _ _ = undefinedFun "nestedApply"
-  liftApply _ _   = undefinedFun "liftApply"
-  getApplyState   = undefinedFun "getApplyState"
+instance Annotate Prim where
+  annotate (FP fn fp) = case fp of
+    RmFile -> do
+      whenPathIs fn $ modify' (\s -> s { path = Nothing })
+      whenWhatIs Directory $ updateDirectory fn
+    AddFile -> return ()
+    Hunk off o n -> whenPathIs fn $ whenWhatIs File $ do
+      let remove = length o
+      let add = length n
+      i <- gets currentInfo
+      c <- gets current
+      a <- gets annotated
+      -- NOTE patches are inverted and in inverse order
+      modify' $ \s ->
+        -- NOTE subtract one from offset because darcs counts from one,
+        -- whereas vectors and lists count from zero.
+        let (to,from) = splitAt (off-1) c
+        in  s { current = map eval $ to ++ replicate add (-1, B.empty) ++ drop remove from
+              , annotated = merge i a $ map eval $ take remove $ from
+              }
+    TokReplace t o n -> whenPathIs fn $ whenWhatIs File $ do
+      let test = annotateReplace t (BC.pack o) (BC.pack n)
+      i <- gets currentInfo
+      c <- gets current
+      a <- gets annotated
+      modify' $ \s -> s
+        { current = map (\(ix,b)->if test b then (-1,B.empty) else (ix,b)) c
+        , annotated = merge i a $ map eval $ filter (test . snd) $ c
+        }
+    -- TODO what if the status of a file changed from text to binary?
+    Binary _ _ -> whenPathIs fn $ bug "annotate: can't handle binary changes"
+  annotate (DP _ AddDir) = return ()
+  annotate (DP fn RmDir) = whenWhatIs Directory $ do
+    whenPathIs fn $ modify' (\s -> s { path = Nothing })
+    updateDirectory fn
+  annotate (Move fn fn') = do
+    modify' (\s -> s { path = fmap (movedirfilename fn fn') (path s) })
+    whenWhatIs Directory $ do
+      let fix (i, x) = (i, fn2ps $ movedirfilename fn fn' (ps2fn x))
+      modify $ \s -> s { current = map fix $ current s }
+  annotate (ChangePref _ _ _) = return ()
 
-instance ApplyMonadTree AnnotatedM where
-  mReadFilePS     = undefinedFun "mReadFilePS"
+instance Annotate FileUUID.Prim where
+  annotate _ = bug "annotate not implemented for FileUUID patches"
 
-  mDoesFileExist _      = return True
-  mDoesDirectoryExist _ = return True
-  mCreateDirectory _    = return ()
-  mCreateFile _         = return ()
+instance Annotate p => Annotate (FL p) where
+  annotate = sequence_ . mapFL annotate
 
-  mRemoveFile f = do
-      p <- gets path
-      when (p == Just f) $ modify (\x -> x { path = Nothing })
-      updateDirectory f
+instance Annotate p => Annotate (Named p) where
+  annotate (NamedP _ _ p) = annotate p
 
-  mRemoveDirectory = mRemoveFile
+instance Annotate p => Annotate (WrappedNamed rt p) where
+  annotate (NormalP n) = annotate n
+  annotate (RebaseP _ _) = bug "annotate not implemented for Rebase patches"
 
-  mRename a b = do
-      p <- gets path
-      w <- gets what
-      when (isJust p) $
-          modify $ \st -> st { path = Just $ movedirfilename a b (fromJust p) }
-      when (w == Directory) $ do
-          let fix (i, x) = (i, fn2ps $ movedirfilename a b (ps2fn x))
-          modify $ \st -> st { current = map fix $ current st }
+instance Annotate p => Annotate (PatchInfoAnd rt p) where
+  annotate = annotate . hopefully
 
-  mModifyFilePS f job = do
-      p <- gets path
-      when (p == Just f) $ updateFile (fmap linesPS . job . unlinesPS)
+whenWhatIs :: FileOrDirectory -> AnnotatedM () -> AnnotatedM ()
+whenWhatIs w actions = do
+  w' <- gets what
+  when (w == w') actions
 
-  mModifyFilePSs f job = do
-      p <- gets path
-      when (p == Just f) $ updateFile job
+whenPathIs :: FileName -> AnnotatedM () -> AnnotatedM ()
+whenPathIs fn actions = do
+  p <- gets path
+  when (p == Just fn) actions
 
-undefinedFun :: Monad m
-             => String
-             -> m a
-undefinedFun name = fail $ name ++ " undefined for Annotated"
-
-
-updateFile :: ([B.ByteString]
-           -> AnnotatedM [B.ByteString])
-           -> AnnotatedM ()
-updateFile job = (==File) <$> gets what >>= flip when go
-  where
-    go = do
-        before <- map snd `fmap` gets current
-        after <- job before
-        da <- gets diffAlgorithm
-        reannotate $ getChanges da before after
-
-    reannotate [] = return ()
-    reannotate ((off, remove, add):rest) = do
-        i <- gets currentInfo
-        c <- gets current
-        a <- gets annotated
-        modify $ \s -> s { current = take off c ++ [ (-1, x) | x <- add ] ++
-                                     drop (off + length remove) c
-                         , annotated = merge i a $ take (length remove) $ drop off c
-                         }
-        reannotate rest
-
-    merge i a l = a V.// [ (line, (Just i, B.empty))
-                         | (line, _) <- l, line >= 0 && line < V.length a]
+eval :: (Int, B.ByteString) -> (Int, B.ByteString)
+eval (i,b) = seq i $ seq b $ (i,b)
 
+merge :: a
+      -> V.Vector (Maybe a, BC.ByteString)
+      -> [(Int, t)]
+      -> V.Vector (Maybe a, BC.ByteString)
+merge i a l = a V.// [ (line, (Just i, B.empty))
+                     | (line, _) <- l, line >= 0 && line < V.length a]
 
 updateDirectory :: FileName -> AnnotatedM ()
-updateDirectory p = (==Directory) <$> gets what >>= flip when go
+updateDirectory p = whenWhatIs Directory $ do
+    let line = fn2ps p
+    files <- gets current
+    case filter ((==line) . snd) files of
+      [match@(ident, _)] -> reannotate ident match line
+      _ -> return ()
   where
-    go = do let line = fn2ps p
-            files <- gets current
-            case filter ((==line) . snd) files of
-              [match@(ident, _)] -> reannotate ident match line
-              _ -> return ()
     reannotate ident match line =
       modify $ \x -> x { annotated = annotated x V.// [ (ident, update line $ currentInfo x) ]
                        , current = filter (/= match) $ current x }
     update line inf = (Just inf, BC.concat [ " -- created as: ", line ])
 
-
 complete :: Annotated -> Bool
 complete x = V.all (isJust . fst) $ annotated x
 
-
-annotate' :: (Apply p, ApplyState p ~ Tree)
+annotate' :: Annotate p
           => FL (PatchInfoAnd rt p) wX wY
           -> Annotated
           -> Annotated
 annotate' NilFL ann = ann
 annotate' (p :>: ps) ann
     | complete ann = ann
-    | otherwise = annotate' ps $ execState (apply p) (ann { currentInfo = info p })
+    | otherwise = annotate' ps $ execState (annotate p) (ann { currentInfo = info p })
 
-
-annotate :: (Apply p, ApplyState p ~ Tree)
-         => D.DiffAlgorithm
-         -> FL (PatchInfoAnd rt p) wX wY
-         -> FileName
-         -> B.ByteString
-         -> AnnotateResult
-annotate da patches inipath inicontent = annotated $ annotate' patches initial
+annotateFile :: Annotate p
+             => FL (PatchInfoAnd rt p) wX wY
+             -> FileName
+             -> B.ByteString
+             -> AnnotateResult
+annotateFile patches inipath inicontent = annotated $ annotate' patches initial
   where
     initial = Annotated { path = Just inipath
-                        , currentInfo = error "There is no currentInfo."
+                        , currentInfo = bug "There is no currentInfo."
                         , current = zip [0..] (linesPS inicontent)
                         , what = File
                         , annotated = V.replicate (length $ breakLines inicontent)
                                                       (Nothing, B.empty)
-                        , diffAlgorithm = da
                         }
 
-
-annotateDirectory :: (Apply p, ApplyState p ~ Tree)
-                  => D.DiffAlgorithm
-                  -> FL (PatchInfoAnd rt p) wX wY
+annotateDirectory :: Annotate p
+                  => FL (PatchInfoAnd rt p) wX wY
                   -> FileName
                   -> [FileName]
                   -> AnnotateResult
-annotateDirectory da patches inipath inicontent = annotated $ annotate' patches initial
+annotateDirectory patches inipath inicontent = annotated $ annotate' patches initial
   where
     initial = Annotated { path = Just inipath
-                        , currentInfo = error "There is no currentInfo."
+                        , currentInfo = bug "There is no currentInfo."
                         , current = zip [0..] (map fn2ps inicontent)
                         , what = Directory
                         , annotated = V.replicate (length inicontent) (Nothing, B.empty)
-                        , diffAlgorithm = da
                         }
 
-
 machineFormat :: B.ByteString -> AnnotateResult -> String
 machineFormat d a = unlines [ case i of
                                  Just inf -> show $ makePatchname inf
@@ -228,10 +227,7 @@ machineFormat d a = unlines [ case i of
                               ++ " | " ++ BC.unpack line ++ " " ++ BC.unpack add
                             | ((i, add), line) <- zip (V.toList a) (breakLines d) ]
 
-
-format :: B.ByteString
-       -> AnnotateResult
-       -> String
+format :: B.ByteString -> AnnotateResult -> String
 format d a = pi_list ++ "\n" ++ numbered
   where
     numberedLines = zip [(1 :: Int)..] . lines $ file
@@ -244,7 +240,7 @@ format d a = pi_list ++ "\n" ++ numbered
 
     numbered = unlines . map prependNum $ numberedLines
 
-    pi_list = unlines [ show n ++ ": " ++ renderString Encode (showPatchInfoUI i)
+    pi_list = unlines [ show n ++ ": " ++ renderString (displayPatchInfo i)
                       | (n :: Int, i) <- zip [1..] pis ]
 
     file = concat [ annotation (fst $ head chunk) ++ " | " ++ line (head chunk) ++
@@ -257,7 +253,7 @@ format d a = pi_list ++ "\n" ++ numbered
 
     file_ann = groupBy ((==) `on` fst) $ zip (V.toList a) (breakLines d)
 
-    line ((_, add), l) = BC.unpack $ BC.concat [l, " ", add]
+    line ((_, add), l) = decodeLocale $ BC.concat [l, " ", add]
 
     annotation (Just i, _) | Just n <- M.lookup i pi_map =
         pad 20 (piMail i) ++ " " ++ pad 4 ('#' : show n)
@@ -271,11 +267,8 @@ format d a = pi_list ++ "\n" ++ numbered
         | '<' `elem` piAuthor pi = takeWhile (/= '>') . drop 1 . dropWhile (/= '<') $ piAuthor pi
         | otherwise = piAuthor pi
 
-
-breakLines :: BC.ByteString
-           -> [BC.ByteString]
+breakLines :: BC.ByteString -> [BC.ByteString]
 breakLines s = case BC.split '\n' s of
     [] -> []
     split | BC.null (last split) -> init split
           | otherwise -> split
-
