@@ -16,7 +16,7 @@
 --  Boston, MA 02110-1301, USA.
 
 
-{-# LANGUAGE CPP, MultiParamTypeClasses #-}
+{-# LANGUAGE MultiParamTypeClasses #-}
 
 
 -- |
@@ -34,25 +34,20 @@ module Darcs.Patch.Apply
     , applyToTree
     , applyToState
     , maybeApplyToTree
-    , applyToFileMods
     , effectOnFilePaths
     ) where
 
 import Prelude ()
 import Darcs.Prelude
 
-import Data.Set ( Set )
-
 import Control.Exception ( catch, IOException )
 import Control.Arrow ( (***) )
 
 import Darcs.Util.Tree( Tree )
 
 import Darcs.Patch.ApplyMonad ( ApplyMonad(..), withFileNames, ApplyMonadTrans(..) )
-import Darcs.Util.Path( FileName, fn2fp, fp2fn )
+import Darcs.Util.Path( fn2fp, fp2fn )
 import Darcs.Patch.Witnesses.Ordered ( FL(..), RL(..) )
-import Darcs.Patch.Index.Monad ( withPatchMods )
-import Darcs.Patch.Index.Types ( PatchMod )
 
 class Apply p where
     type ApplyState p :: (* -> *) -> *
@@ -90,7 +85,7 @@ applyToFilePaths pa ofpos fs = toFPs $ withFileNames ofnos fns (apply pa) where
 
 
 -- | Apply a patch to a 'Tree', yielding a new 'Tree'.
-applyToTree :: (Apply p, Functor m, Monad m, ApplyState p ~ Tree)
+applyToTree :: (Apply p, Monad m, ApplyState p ~ Tree)
             => p wX wY
             -> Tree m
             -> m (Tree m)
@@ -109,8 +104,3 @@ maybeApplyToTree :: (Apply p, ApplyState p ~ Tree) => p wX wY -> Tree IO
                  -> IO (Maybe (Tree IO))
 maybeApplyToTree patch tree =
     (Just `fmap` applyToTree patch tree) `catch` (\(_ :: IOException) -> return Nothing)
-
---------------------------------------------------------------------------------
--- | Apply a patch to set of 'FileName's, yielding the new set of 'FileName's and 'PatchMod's
-applyToFileMods :: (Apply p, ApplyState p ~ Tree) => p wX wY -> Set FileName -> (Set FileName, [PatchMod FileName])
-applyToFileMods patch = withPatchMods (apply patch)
