@@ -3,6 +3,8 @@ module Darcs.Patch.Bracketed
     , BracketedFL, mapBracketedFLFL, unBracketedFL
     ) where
 
+import Prelude ()
+import Darcs.Prelude
 
 import Darcs.Patch.Format ( PatchListFormat )
 import Darcs.Patch.Witnesses.Ordered ( FL(..), mapFL_FL, concatFL )
