@@ -64,7 +64,7 @@ import Darcs.Util.ByteString
     ( mmapFilePS, linesPS, unlinesPS, dropSpace, substrPS, decodeLocale )
 import Darcs.Util.Hash ( sha1PS )
 import Darcs.Util.Printer ( Doc, renderPS, newline, text, ($$),
-                 (<>), vcat, vsep, renderString )
+                 vcat, vsep, renderString )
 
 -- |hashBundle creates a SHA1 string of a given a FL of named patches. This
 -- allows us to ensure that the patches in a received patchBundle have not been
