@@ -1,5 +1,5 @@
 --  Copyright (C) 2002-2003 David Roundy, 2010 Ganesh Sittampalam
-{-# LANGUAGE CPP, ViewPatterns #-}
+{-# LANGUAGE ViewPatterns #-}
 module Darcs.Patch.Conflict
     ( Conflict(..), CommuteNoConflicts(..), listConflictedFiles
     , IsConflictedPrim(..), ConflictState(..)
@@ -31,8 +31,6 @@ import Darcs.Patch.Witnesses.Show ( Show2, showsPrec2 )
 import Darcs.Util.Path ( FileName, fn2fp, fp2fn )
 import Darcs.Util.Show ( appPrec )
 
-#include "impossible.h"
-
 listConflictedFiles :: Conflict p => p wX wY -> [FilePath]
 listConflictedFiles p =
     nubSort $ concatMap (unseal listTouchedFiles) $ concat $ resolveConflicts p
@@ -61,14 +59,32 @@ instance CommuteNoConflicts p => CommuteNoConflicts (FL p) where
                                          return $ ys' :> reverseRL rxs'
 
 instance (CommuteNoConflicts p, Conflict p) => Conflict (RL p) where
+    -- By definition, a conflicting (primitive) patch is resolved if
+    -- another (primitive) patch depends on the conflict.
+    -- 
+    -- So, when looking for conflicts in a list of patches, we go
+    -- through the whole list looking for individual patches that are
+    -- in conflict. But then we try to commute them past all the
+    -- patches we've already seen. If we fail, i.e. there's something
+    -- that depends on the conflict, then we forget about the conflict;
+    -- this is the Nothing case of the 'commuteNoConflictsFL' call.
+    --
+    -- Note that 'primitive' does not mean Prim (this is a case of bad
+    -- naming) but rather a RepoPatchV1 or RepoPatchV2. Prim patches
+    -- are merely a 'base class' containing everything common to V1 and
+    -- V2 primitive patches.
     resolveConflicts x = rcs x NilFL
-        where rcs :: RL p wX wY -> FL p wY wW -> [[Sealed (FL (PrimOf p) wW)]]
-              rcs NilRL _ = []
-              rcs (ps:<:p) passedby | (_:_) <- resolveConflicts p =
-                  case commuteNoConflictsFL (p:>passedby) of
-                    Just (_:> p') -> resolveConflicts p' ++ rcs ps (p:>:passedby)
-                    Nothing -> rcs ps (p:>:passedby)
-              rcs (ps:<:p) passedby = seq passedby $ rcs ps (p:>:passedby)
+      where
+        rcs :: RL p wX wY -> FL p wY wW -> [[Sealed (FL (PrimOf p) wW)]]
+        rcs NilRL _ = []
+        rcs (ps :<: p) passedby
+          | null (resolveConflicts p) = seq passedby rest -- TODO why seq here?
+          | otherwise =
+            case commuteNoConflictsFL (p :> passedby) of
+              Just (_ :> p') -> resolveConflicts p' ++ rest
+              Nothing -> rest
+          where
+            rest = rcs ps (p :>: passedby)
     conflictedEffect = concat . reverse . mapRL conflictedEffect
 
 instance CommuteNoConflicts p => CommuteNoConflicts (RL p) where
