@@ -248,10 +248,10 @@ splitOnTag _ (PatchSet NilRL NilRL) = Nothing
 
 -- |'unwrapOneTagged' unfolds a single Tagged object in a PatchSet, adding the
 -- tag and patches to the PatchSet's patch list.
-unwrapOneTagged :: (Monad m) => PatchSet rt p wX wY -> m (PatchSet rt p wX wY)
+unwrapOneTagged :: PatchSet rt p wX wY -> Maybe (PatchSet rt p wX wY)
 unwrapOneTagged (PatchSet (ts :<: Tagged t _ tps) ps) =
-    return $ PatchSet ts (tps :<: t +<+ ps)
-unwrapOneTagged _ = fail "called unwrapOneTagged with no Tagged's in the set"
+    Just $ PatchSet ts (tps :<: t +<+ ps)
+unwrapOneTagged _ = Nothing
 
 -- | @getUncovered ps@ returns the 'PatchInfo' for all the patches in
 --   @ps@ that are not depended on by anything else *through explicit
