@@ -17,15 +17,21 @@
 -- Boston, MA 02110-1301, USA.
 
 
-{-# LANGUAGE CPP,GeneralizedNewtypeDeriving #-}
+{-# LANGUAGE GeneralizedNewtypeDeriving #-}
 {-# LANGUAGE MultiParamTypeClasses #-}
 
-module Darcs.Patch.Index.Monad ( FileModMonad, withPatchMods ) where
+module Darcs.Patch.Index.Monad
+    ( withPatchMods
+    , applyToFileMods
+    , makePatchID
+    ) where
 
 import Prelude ()
 import Darcs.Prelude
 
-import Darcs.Patch.Index.Types ( PatchMod(..) )
+import Darcs.Patch.Index.Types ( PatchMod(..), PatchId(..) )
+import Darcs.Patch.Info ( makePatchname, PatchInfo )
+import Darcs.Patch.Apply ( Apply(..) )
 import Darcs.Patch.ApplyMonad ( ApplyMonad(..), ApplyMonadTree(..) )
 import Control.Monad.State
 import Control.Arrow
@@ -34,7 +40,6 @@ import qualified Data.Set as S
 import Data.Set ( Set )
 import Data.List ( isPrefixOf )
 import Darcs.Util.Tree (Tree)
-#include "impossible.h"
 
 newtype FileModMonad a = FMM (State (Set FileName, [PatchMod FileName]) a)
   deriving (Functor, Applicative, Monad, MonadState (Set FileName, [PatchMod FileName]))
@@ -79,7 +84,6 @@ instance ApplyMonadTree FileModMonad where
                addMod (PRename fn newfn)
 
     mModifyFilePS f _ = addMod (PTouch f)
-    mModifyFilePSs f _ = addMod (PTouch f)
 
 -- ---------------------------------------------------------------------
 -- State Handling Functions
@@ -117,3 +121,11 @@ remove f = addMod (PRemove f) >> modifyFps (S.delete f)
 
 modifyFps :: (Set FileName -> Set FileName) -> FileModMonad ()
 modifyFps f = modify $ first f
+
+makePatchID :: PatchInfo -> PatchId
+makePatchID = PID . makePatchname
+
+--------------------------------------------------------------------------------
+-- | Apply a patch to set of 'FileName's, yielding the new set of 'FileName's and 'PatchMod's
+applyToFileMods :: (Apply p, ApplyState p ~ Tree) => p wX wY -> Set FileName -> (Set FileName, [PatchMod FileName])
+applyToFileMods patch = withPatchMods (apply patch)
