@@ -70,6 +70,8 @@ module Darcs.Patch.Match
 import Prelude ()
 import Darcs.Prelude
 
+import Control.Exception ( throw )
+
 import Text.ParserCombinators.Parsec
     ( parse
     , CharParser
@@ -421,7 +423,7 @@ getNonrangeMatchS fs repo =
         Just m -> if nonrangeMatcherIsTag fs
                         then getTagS m repo
                         else getMatcherS Exclusive m repo
-        Nothing -> fail "Pattern not specified in getNonrangeMatch."
+        Nothing -> throw $ userError "Pattern not specified in getNonrangeMatch."
 
 -- | @firstMatch fs@ tells whether @fs@ implies a "first match", that
 -- is if we match against patches from a point in the past on, rather
@@ -441,7 +443,7 @@ getFirstMatchS fs repo =
      Just (_,b) -> unpullLastN repo b -- b is chronologically earlier than a
      Nothing    ->
       case firstMatcher fs of
-               Nothing -> fail "Pattern not specified in getFirstMatchS."
+               Nothing -> throw $ userError "Pattern not specified in getFirstMatchS."
                Just m -> if firstMatcherIsTag fs
                          then getTagS m repo
                          else getMatcherS Inclusive m repo
@@ -462,7 +464,7 @@ checkMatchSyntax :: [MatchFlag] -> IO ()
 checkMatchSyntax opts =
  case getMatchPattern opts of
   Nothing -> return ()
-  Just p  -> either fail (const $ return ()) (parseMatch p::Either String (MatchFun rt DummyPatch))
+  Just p  -> either (throw . userError) (const $ return ()) (parseMatch p::Either String (MatchFun rt DummyPatch))
 
 getMatchPattern :: [MatchFlag] -> Maybe String
 getMatchPattern [] = Nothing
@@ -718,7 +720,7 @@ getMatcherS :: (ApplyMonad (ApplyState p) m, Matchable p) =>
 getMatcherS ioe m repo =
     if matchExists m repo
     then applyInvToMatcher ioe m repo
-    else fail $ "Couldn't match pattern "++ show m
+    else throw $ userError $ "Couldn't match pattern "++ show m
 
 getTagS :: (ApplyMonad (ApplyState p) m, MonadProgress m, Matchable p) =>
              Matcher rt p -> PatchSet rt p Origin wX -> m ()
