@@ -56,7 +56,7 @@ import Darcs.Patch.Witnesses.Ordered
 
 import Darcs.Util.IsoDate ( getIsoDateTime )
 import Darcs.Util.Text ( formatParas )
-import Darcs.Util.Printer ( ($$), (<>), vcat, prefix )
+import Darcs.Util.Printer ( ($$), vcat, prefix )
 
 import Control.Applicative ( (<|>) )
 
