@@ -1,4 +1,4 @@
-{-# LANGUAGE ViewPatterns, OverloadedStrings #-}
+{-# LANGUAGE CPP, ViewPatterns, OverloadedStrings #-}
 {-# OPTIONS_GHC -fno-warn-orphans #-}
 module Darcs.Patch.Prim.FileUUID.Read () where
 
@@ -8,7 +8,7 @@ import Darcs.Prelude
 import Darcs.Patch.Read ( ReadPatch(..) )
 import Darcs.Patch.ReadMonads
 import Darcs.Patch.Prim.Class( PrimRead(..) )
-import Darcs.Patch.Prim.FileUUID.Core( Prim(..), Hunk(..), Location(..) )
+import Darcs.Patch.Prim.FileUUID.Core( Prim(..), Hunk(..) )
 import Darcs.Patch.Prim.FileUUID.ObjectMap
 import Darcs.Patch.Witnesses.Sealed( seal )
 import Darcs.Util.Path ( unsafeMakeName )
@@ -58,5 +58,6 @@ decodeWhite (BC.uncons -> Just ('\\', cs)) =
     _ -> error "malformed filename"
 decodeWhite (BC.uncons -> Just (c, cs)) = c `BC.cons` decodeWhite cs
 decodeWhite (BC.uncons -> Nothing) = BC.empty
+#if !MIN_VERSION_base(4,14,0)
 decodeWhite _ = impossible
-
+#endif
