@@ -22,7 +22,7 @@ import Darcs.Util.ByteString ( linesPS )
 import Darcs.Util.Path ( Name, encodeWhiteName )
 import Darcs.Util.Printer
     ( text, packedString, blueText, prefix
-    , (<+>), (<>), ($$), Doc, vcat
+    , (<+>), ($$), Doc, vcat
     )
 
 -- TODO this instance shouldn't really be necessary, as Prims aren't used generically
