@@ -1,9 +1,6 @@
 {-# OPTIONS_GHC -fno-warn-orphans #-}
 module Darcs.Patch.Prim.V1 ( Prim ) where
 
-import Prelude ()
-import Darcs.Prelude
-
 import Darcs.Patch.Prim.V1.Apply ()
 import Darcs.Patch.Prim.V1.Coalesce ()
 import Darcs.Patch.Prim.V1.Commute ()
@@ -12,13 +9,6 @@ import Darcs.Patch.Prim.V1.Details ()
 import Darcs.Patch.Prim.V1.Read ()
 import Darcs.Patch.Prim.V1.Show ()
 
-import Darcs.Patch.Prim.Class ( PrimPatch, PrimPatchBase(..), FromPrim(..) )
-import Darcs.Patch.Patchy ( Patchy )
+import Darcs.Patch.Prim.Class ( PrimPatchCommon )
 
-instance PrimPatch Prim
-instance Patchy Prim
-instance PrimPatchBase Prim where
-  type PrimOf Prim = Prim
-
-instance FromPrim Prim where
-  fromPrim = id
+instance PrimPatchCommon Prim
