@@ -26,9 +26,10 @@ import Darcs.Patch.Witnesses.Unsafe ( unsafeCoercePStart )
 import Darcs.Util.ByteString ( unlinesPS )
 import Darcs.Util.Printer( renderString )
 
+import Control.Exception ( throw )
+
 import qualified Data.ByteString            as B
 import qualified Data.ByteString.Internal   as BI
-import qualified Data.ByteString as B ( ByteString, empty, null, concat )
 import qualified Data.ByteString.Char8 as BC (pack, unpack, unlines)
 
 type FileContents = B.ByteString
@@ -41,13 +42,13 @@ instance Apply Prim where
     apply (FP f (TokReplace t o n)) = mModifyFilePS f doreplace
         where doreplace fc =
                   case tryTokReplace t (BC.pack o) (BC.pack n) fc of
-                  Nothing -> fail $ "replace patch to " ++ fn2fp f
+                  Nothing -> throw $ userError $ "replace patch to " ++ fn2fp f
                              ++ " couldn't apply."
                   Just fc' -> return fc'
     apply (FP f (Binary o n)) = mModifyFilePS f doapply
         where doapply oldf = if o == oldf
                              then return n
-                             else fail $ "binary patch to " ++ fn2fp f
+                             else throw $ userError $ "binary patch to " ++ fn2fp f
                                   ++ " couldn't apply."
     apply (DP d AddDir) = mCreateDirectory d
     apply (DP d RmDir) = mRemoveDirectory d
@@ -115,7 +116,7 @@ applyHunk f h fc =
   case applyHunkLines h fc of
     Right fc' -> return fc'
     Left msg ->
-      fail $
+      throw $ userError $
       "### Error applying:\n" ++ renderHunk h ++
       "\n### to file " ++ fn2fp f ++ ":\n" ++ BC.unpack fc ++
       "### Reason: " ++ msg
