@@ -1,5 +1,4 @@
 {-# OPTIONS_GHC -fno-warn-orphans #-}
-{-# LANGUAGE CPP #-}
 module Darcs.Patch.Prim.V1.Coalesce
     ()
     where
@@ -23,7 +22,7 @@ import Darcs.Patch.Prim.V1.Core
     , comparePrim, isIdentity
     )
 import Darcs.Patch.Prim.V1.Show ()
-import Darcs.Patch.Witnesses.Eq ( MyEq(..), EqCheck(..) )
+import Darcs.Patch.Witnesses.Eq ( Eq2(..), EqCheck(..) )
 import Darcs.Patch.Witnesses.Ordered
     ( FL(..), RL(..), (:>)(..)
     , reverseRL, mapFL, mapFL_FL
@@ -41,8 +40,6 @@ import qualified Darcs.Util.Diff as D ( DiffAlgorithm )
 import Darcs.Util.Global ( darcsdir )
 import Darcs.Util.Path ( FileName, fp2fn )
 
-#include "impossible.h"
-
 -- | 'coalesceFwd' @p1 :> p2@ tries to combine @p1@ and @p2@ into a single
 --   patch without intermediary changes.  For example, two hunk patches
 --   modifying adjacent lines can be coalesced into a bigger hunk patch.
@@ -81,9 +78,11 @@ mapPrimFL f x =
 
 
 
-data Simple wX wY = SFP !(FilePatchType wX wY) | SDP !(DirPatchType wX wY)
-                   | SCP String String String
-                   deriving ( Show )
+data Simple wX wY
+    = SFP !(FilePatchType wX wY)
+    | SDP !(DirPatchType wX wY)
+    | SCP String String String
+    deriving ( Show )
 
 toSimple :: Prim wX wY -> Maybe (FileName, Simple wX wY)
 toSimple (FP a b) = Just (a, SFP b)
