@@ -1,3 +1,4 @@
+{-# LANGUAGE CPP #-}
 {-# OPTIONS_GHC -fno-warn-orphans #-}
 module Darcs.Patch.Prim.V1.Commute
     ( Perhaps(..)
@@ -58,6 +59,9 @@ instance  Monad Perhaps where
     Failed   >>= _      =  Failed
     Unknown  >>= _      =  Unknown
     return              =  Succeeded
+#if MIN_VERSION_base(4,13,0)
+instance MonadFail Perhaps where
+#endif
     fail _              =  Unknown
 
 instance Alternative Perhaps where
