@@ -15,8 +15,6 @@
 -- the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 -- Boston, MA 02110-1301, USA.
 
-{-# LANGUAGE CPP #-}
-
 module Darcs.Patch.Read ( ReadPatch(..),
                           readPatch, readPatchPartial,
                           bracketedFL, peekfor,
@@ -26,11 +24,11 @@ module Darcs.Patch.Read ( ReadPatch(..),
 import Prelude ()
 import Darcs.Prelude
 
-import Darcs.Util.ByteString ( dropSpace )
+import Darcs.Util.ByteString ( dropSpace, unpackPSFromUTF8, decodeLocale )
 import qualified Data.ByteString       as B  (ByteString, null)
 
 import Darcs.Patch.Bracketed ( Bracketed(..), unBracketedFL )
-import Darcs.Util.Path ( FileName, fp2fn, ps2fn, decodeWhite )
+import Darcs.Util.Path ( FileName, fp2fn, decodeWhite )
 import Darcs.Patch.Format ( PatchListFormat(..), ListFormat(..), FileNameFormat(..) )
 import Darcs.Patch.ReadMonads (ParserM,
                                parseStrictly,
@@ -42,9 +40,9 @@ import Darcs.Patch.Witnesses.Sealed ( Sealed(..), mapSeal )
 
 import Control.Applicative ( (<|>) )
 import Control.Monad ( mzero )
-import qualified Data.ByteString.Char8 as BC ( ByteString, unpack )
-
+import qualified Data.ByteString.Char8 as BC ( ByteString, pack )
 
+-- | This class is used to decode patches from their binary representation.
 class ReadPatch p where
     readPatch'
         :: ParserM m => m (Sealed (p wX))
@@ -116,6 +114,8 @@ peekfor ps ifstr ifnot = choice [ do lexString ps
                                 , ifnot ]
 {-# INLINE peekfor #-}
 
+-- See also Darcs.Patch.Show.formatFileName.
 readFileName :: FileNameFormat -> B.ByteString -> FileName
-readFileName OldFormat = ps2fn
-readFileName NewFormat = fp2fn . decodeWhite . BC.unpack
+readFileName OldFormat = fp2fn . decodeWhite . decodeLocale . BC.pack . unpackPSFromUTF8
+readFileName NewFormat = fp2fn . decodeWhite . decodeLocale
+readFileName UserFormat = error "readFileName called with UserFormat"
