@@ -2,7 +2,6 @@
 --
 --  BSD3
 
-{-# LANGUAGE CPP #-}
 module Darcs.Patch.Rebase.Name
     ( RebaseName(..)
     , commuteNamePrim, commutePrimName
@@ -18,15 +17,16 @@ import Darcs.Patch.Effect ( Effect(..) )
 import Darcs.Patch.Info ( PatchInfo, isInverted, showPatchInfo, readPatchInfo )
 import Darcs.Patch.Inspect ( PatchInspect(..) )
 import Darcs.Patch.Named ( Named(..) )
-import Darcs.Patch.Patchy
-    ( Invert(..), Commute(..), Patchy, Apply(..)
-    , ShowPatch(..), ReadPatch(..)
-    )
+import Darcs.Patch.Apply ( Apply(..) )
+import Darcs.Patch.Commute ( Commute(..) )
+import Darcs.Patch.Invert ( Invert(..) )
+import Darcs.Patch.Read ( ReadPatch(..) )
+import Darcs.Patch.Show ( ShowPatch(..) )
 import Darcs.Patch.Permutations ( inverseCommuter )
-import Darcs.Patch.Prim ( PrimPatchBase, PrimOf )
+import Darcs.Patch.Prim ( PrimPatchBase(..) )
 import Darcs.Patch.ReadMonads ( lexString )
 import Darcs.Patch.Show ( ShowPatchBasic(..) )
-import Darcs.Patch.Witnesses.Eq ( MyEq(..) )
+import Darcs.Patch.Witnesses.Eq ( Eq2(..) )
 import Darcs.Patch.Witnesses.Ordered ( (:>)(..), FL(..) )
 import Darcs.Patch.Witnesses.Sealed ( Sealed(..) )
 import Darcs.Patch.Witnesses.Show
@@ -40,8 +40,6 @@ import Darcs.Util.Printer ( empty, blueText, ($$) )
 import Control.Applicative ( (<|>) )
 import qualified Data.ByteString.Char8 as BC ( pack )
 
-#include "impossible.h"
-
 -- Note: in principle this is a general concept not limited to
 -- rebase, and we might be able to generalise this type and
 -- refactor named patches to use it too.
@@ -63,9 +61,9 @@ instance Show2 (RebaseName p) where
     showDict2 = ShowDictClass
 
 instance ShowPatchBasic (RebaseName p) where
-   showPatch (AddName n) = blueText "addname" $$ showPatchInfo n
-   showPatch (DelName n) = blueText "delname" $$ showPatchInfo n
-   showPatch (Rename old new) = blueText "rename" $$ showPatchInfo old $$ showPatchInfo new
+   showPatch f (AddName n) = blueText "addname" $$ showPatchInfo f n
+   showPatch f (DelName n) = blueText "delname" $$ showPatchInfo f n
+   showPatch f (Rename old new) = blueText "rename" $$ showPatchInfo f old $$ showPatchInfo f new
 
 instance ShowPatch (RebaseName p) where
    summary _ = empty -- TODO improve this?
@@ -130,19 +128,17 @@ instance PatchInspect (RebaseName p) where
     listTouchedFiles _ = []
     hunkMatches _ _ = False
 
-instance Apply p => Apply (RebaseName p) where
+instance Apply (RebaseName p) where
    type ApplyState (RebaseName p) = ApplyState p
    apply _ = return ()
 
-instance Apply p => Patchy (RebaseName p)
-
 instance PrimPatchBase p => PrimPatchBase (RebaseName p) where
    type PrimOf (RebaseName p) = PrimOf p
 
 instance Effect (RebaseName p) where
    effect _ = unsafeCoerceP NilFL
 
-instance MyEq (RebaseName p) where
+instance Eq2 (RebaseName p) where
    AddName n1 `unsafeCompare` AddName n2 = n1 == n2
    AddName _ `unsafeCompare` _ = False
    _ `unsafeCompare` AddName _ = False
@@ -158,12 +154,12 @@ instance MyEq (RebaseName p) where
 
 -- |Commute a name patch and a primitive patch. They trivially
 -- commute so this just involves changing the witnesses.
-commuteNamePrim :: PrimPatchBase p => (RebaseName p :> PrimOf p) wX wY -> (PrimOf p :> RebaseName p) wX wY
+commuteNamePrim :: (RebaseName p :> PrimOf p) wX wY -> (PrimOf p :> RebaseName p) wX wY
 commuteNamePrim (n :> f) = unsafeCoerceP f :> unsafeCoerceP n
 
 -- |Commute a primitive patch and a name patch. They trivially
 -- commute so this just involves changing the witnesses.
-commutePrimName :: PrimPatchBase p => (PrimOf p :> RebaseName p) wX wY -> (RebaseName p :> PrimOf p) wX wY
+commutePrimName :: (PrimOf p :> RebaseName p) wX wY -> (RebaseName p :> PrimOf p) wX wY
 commutePrimName (f :> n) = unsafeCoerceP n :> unsafeCoerceP f
 
 -- |Commute a name patch and a named patch. In most cases this is
