@@ -15,7 +15,7 @@
 -- the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 -- Boston, MA 02110-1301, USA.
 
-{-# LANGUAGE CPP, EmptyDataDecls, StandaloneDeriving #-}
+{-# LANGUAGE EmptyDataDecls, StandaloneDeriving #-}
 
 module Darcs.Patch.Set
     ( PatchSet(..)
@@ -26,8 +26,8 @@ module Darcs.Patch.Set
     , tags
     , emptyPatchSet
     , appendPSFL
-    , newset2RL
-    , newset2FL
+    , patchSet2RL
+    , patchSet2FL
     , patchSetfMap
     ) where
 
@@ -89,18 +89,18 @@ instance Show2 p => Show2 (Tagged rt p) where
     showDict2 = ShowDictClass
 
 
--- |'newset2RL' takes a 'PatchSet' and returns an equivalent, linear 'RL' of
+-- |'patchSet2RL' takes a 'PatchSet' and returns an equivalent, linear 'RL' of
 -- patches.
-newset2RL :: PatchSet rt p wStart wX -> RL (PatchInfoAnd rt p) wStart wX
-newset2RL (PatchSet ts ps) = concatRL (mapRL_RL ts2rl ts) +<+ ps
+patchSet2RL :: PatchSet rt p wStart wX -> RL (PatchInfoAnd rt p) wStart wX
+patchSet2RL (PatchSet ts ps) = concatRL (mapRL_RL ts2rl ts) +<+ ps
   where
     ts2rl :: Tagged rt p wY wZ -> RL (PatchInfoAnd rt p) wY wZ
     ts2rl (Tagged t _ ps2) = ps2 :<: t
 
--- |'newset2FL' takes a 'PatchSet' and returns an equivalent, linear 'FL' of
+-- |'patchSet2FL' takes a 'PatchSet' and returns an equivalent, linear 'FL' of
 -- patches.
-newset2FL :: PatchSet rt p wStart wX -> FL (PatchInfoAnd rt p) wStart wX
-newset2FL = reverseRL . newset2RL
+patchSet2FL :: PatchSet rt p wStart wX -> FL (PatchInfoAnd rt p) wStart wX
+patchSet2FL = reverseRL . patchSet2RL
 
 -- |'appendPSFL' takes a 'PatchSet' and a 'FL' of patches that "follow" the
 -- PatchSet, and concatenates the patches into the PatchSet.
@@ -128,4 +128,4 @@ tags (PatchSet ts _) = mapRL taggedTagInfo ts
     taggedTagInfo (Tagged t _ _) = info t
 
 patchSetfMap:: (forall wW wZ . PatchInfoAnd rt p wW wZ -> IO a) -> PatchSet rt p wW' wZ' -> IO [a]
-patchSetfMap f = sequence . mapRL f . newset2RL
+patchSetfMap f = sequence . mapRL f . patchSet2RL
