@@ -16,7 +16,7 @@ import Darcs.Patch.Witnesses.Ordered ( FL, mapFL )
 
 import Darcs.Util.Printer ( Doc, empty, vcat,
                  text,
-                 minus, plus, ($$), (<+>), (<>),
+                 minus, plus, ($$), (<+>)
                )
 
 
