@@ -1,5 +1,8 @@
 module Darcs.Patch.SummaryData ( SummDetail(..), SummOp(..) ) where
 
+import Prelude ()
+import Darcs.Prelude
+
 import Darcs.Util.Path ( FileName )
 
 data SummDetail = SummAddDir FileName
