@@ -4,6 +4,7 @@ module Darcs.Patch.V1.Apply () where
 import Prelude ()
 import Darcs.Prelude
 
+import Darcs.Patch.Annotate ( Annotate(..) )
 import Darcs.Patch.Apply ( ApplyState, Apply, apply )
 import Darcs.Patch.Prim ( PrimPatch, applyPrimFL )
 import Darcs.Patch.Repair ( RepairToFL, applyAndTryToFixFL,
@@ -12,7 +13,6 @@ import Darcs.Patch.Effect ( effect )
 
 import Darcs.Patch.V1.Commute ()
 import Darcs.Patch.V1.Core ( RepoPatchV1(..) )
-
 import Darcs.Patch.Witnesses.Ordered ( mapFL_FL )
 
 
@@ -23,3 +23,6 @@ instance PrimPatch prim => Apply (RepoPatchV1 prim) where
 instance PrimPatch prim => RepairToFL (RepoPatchV1 prim) where
     applyAndTryToFixFL (PP x) = mapMaybeSnd (mapFL_FL PP) `fmap` applyAndTryToFixFL x
     applyAndTryToFixFL x = do apply x; return Nothing
+
+instance (PrimPatch prim, Annotate prim) => Annotate (RepoPatchV1 prim) where
+    annotate = annotate . effect
