@@ -15,6 +15,7 @@
 --  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 --  Boston, MA 02110-1301, USA.
 
+{-# LANGUAGE CPP #-}
 {-# OPTIONS_GHC -fno-warn-orphans #-}
 
 module Darcs.Patch.V1.Commute
@@ -93,6 +94,9 @@ instance  Monad Perhaps where
     Failed   >>= _      =  Failed
     Unknown  >>= _      =  Unknown
     return              =  Succeeded
+#if MIN_VERSION_base(4,13,0)
+instance MonadFail Perhaps where
+#endif
     fail _              =  Unknown
 
 instance Alternative Perhaps where
