@@ -16,7 +16,7 @@
 -- Boston, MA 02110-1301, USA.
 
 {-# OPTIONS_GHC -fno-warn-orphans -fno-warn-name-shadowing #-}
-{-# LANGUAGE CPP, FlexibleContexts, UndecidableInstances #-}
+{-# LANGUAGE FlexibleContexts, UndecidableInstances #-}
 
 
 module Darcs.Patch.V2.Non
@@ -44,18 +44,22 @@ import Darcs.Prelude hiding ( (*>) )
 
 import Data.List ( delete )
 import Control.Monad ( liftM, mzero )
+import Darcs.Patch.Apply ( Apply(..) )
 import Darcs.Patch.Commute ( commuteFL )
 import Darcs.Patch.Effect ( Effect(..) )
-import Darcs.Patch.Format ( PatchListFormat, FileNameFormat(..) )
+import Darcs.Patch.Format ( PatchListFormat )
 import Darcs.Patch.Invert ( Invert, invertFL, invertRL )
-import Darcs.Patch.Prim ( FromPrim(..), ToFromPrim(..),
-                          PrimOf, PrimPatchBase,
-                          showPrim, sortCoalesceFL,
-                          readPrim )
-import Darcs.Patch.Patchy ( Patchy, showPatch, ReadPatch(..),
-                            Commute(..), invert )
+import Darcs.Patch.Prim
+    ( FromPrim(..), ToFromPrim(..)
+    , PrimOf, PrimPatchBase
+    , sortCoalesceFL
+    )
+import Darcs.Patch.Commute ( Commute(..) )
+import Darcs.Patch.Invert ( Invert(invert) )
+import Darcs.Patch.Read ( ReadPatch(..) )
+import Darcs.Patch.Show ( showPatch )
 import Darcs.Patch.ReadMonads ( ParserM, lexChar )
-import Darcs.Patch.Witnesses.Eq ( MyEq(..), EqCheck(..) )
+import Darcs.Patch.Witnesses.Eq ( Eq2(..), EqCheck(..) )
 import Darcs.Patch.Witnesses.Ordered
     ( FL(..), RL(..), (+>+), mapRL_RL
     , (:>)(..), reverseFL, reverseRL )
@@ -64,7 +68,7 @@ import Darcs.Patch.Witnesses.Show
     , showsPrec2 )
 import Darcs.Patch.Witnesses.Sealed ( Sealed(Sealed) )
 import Darcs.Patch.Read ( peekfor )
-import Darcs.Patch.Show ( ShowPatchBasic )
+import Darcs.Patch.Show ( ShowPatchBasic, ShowPatchFor )
 import Darcs.Patch.Viewing ()
 import Darcs.Patch.Permutations ( removeFL, commuteWhatWeCanFL )
 import Darcs.Util.Printer ( Doc, empty, vcat, hiddenPrefix, blueText, ($$) )
@@ -90,16 +94,18 @@ instance (Show2 p, Show2 (PrimOf p)) => Show1 (Non p) where
 
 -- |showNons creates a Doc representing a list of Nons.
 showNons :: (ShowPatchBasic p, PatchListFormat p, PrimPatchBase p)
-         => [Non p wX] -> Doc
-showNons [] = empty
-showNons xs = blueText "{{" $$ vcat (map showNon xs) $$ blueText "}}"
+         => ShowPatchFor -> [Non p wX] -> Doc
+showNons _ [] = empty
+showNons f xs = blueText "{{" $$ vcat (map (showNon f) xs) $$ blueText "}}"
 
 -- |showNon creates a Doc representing a Non.
-showNon :: (ShowPatchBasic p, PatchListFormat p, PrimPatchBase p) => Non p wX
+showNon :: (ShowPatchBasic p, PatchListFormat p, PrimPatchBase p)
+        => ShowPatchFor
+        -> Non p wX
         -> Doc
-showNon (Non c p) = hiddenPrefix "|" (showPatch c)
-                    $$ hiddenPrefix "|" (blueText ":")
-                    $$ showPrim NewFormat p
+showNon f (Non c p) = hiddenPrefix "|" (showPatch f c)
+                      $$ hiddenPrefix "|" (blueText ":")
+                      $$ showPatch f p
 
 -- |readNons is a parser that attempts to read a list of Nons.
 readNons :: (ReadPatch p, PatchListFormat p, PrimPatchBase p, ParserM m)
@@ -108,20 +114,20 @@ readNons = peekfor (BC.pack "{{") rns (return [])
   where rns = peekfor (BC.pack "}}") (return []) $
               do Sealed ps <- readPatch'
                  lexChar ':'
-                 Sealed p <- readPrim NewFormat
+                 Sealed p <- readPatch'
                  (Non ps p :) `liftM` rns
 
 -- |readNon is a parser that attempts to read a single Non.
 readNon :: (ReadPatch p, PatchListFormat p, PrimPatchBase p, ParserM m)
         => m (Non p wX)
 readNon = do Sealed ps <- readPatch'
-             let doReadPrim = do Sealed p <- readPrim NewFormat
+             let doReadPrim = do Sealed p <- readPatch'
                                  return $ Non ps p
              peekfor (BC.singleton ':') doReadPrim mzero
 
 -- |Nons are equal if their context patches are equal, and they have an equal
 -- prim patch.
-instance (Commute p, MyEq p, MyEq (PrimOf p)) => Eq (Non p wX) where
+instance (Commute p, Eq2 p, Eq2 (PrimOf p)) => Eq (Non p wX) where
     Non (cx :: FL p wX wY1) (x :: PrimOf p wY1 wZ1)
      == Non (cy :: FL p wX wY2) (y :: PrimOf p wY2 wZ2) =
       case cx =\/= cy of
@@ -138,7 +144,7 @@ class Nonable p where
 -- returns some variant @cy'@. If commutation suceeds, the variant is just
 -- straightforwardly the commuted version. If commutation fails, the variant
 -- consists of @x@ prepended to the context of @cy@.
-commuteOrAddToCtx :: (Patchy p, ToFromPrim p) => p wX wY -> Non p wY
+commuteOrAddToCtx :: (Commute p, ToFromPrim p) => p wX wY -> Non p wY
                   -> Non p wX
 commuteOrAddToCtx p n | Just n' <- p >* n = n'
 commuteOrAddToCtx p (Non c x) = Non (p:>:c) x
@@ -155,7 +161,7 @@ commuteOrAddToCtx p (Non c x) = Non (p:>:c) x
 -- > x1 [c1'' c2'' y''] x2' x3'
 --
 -- and return @[x1 c1'' c2'' y'']@
-commuteOrAddToCtxRL :: (Patchy p, ToFromPrim p) => RL p wX wY -> Non p wY
+commuteOrAddToCtxRL :: (Apply p, Commute p, Invert p, ToFromPrim p) => RL p wX wY -> Non p wY
                     -> Non p wX
 commuteOrAddToCtxRL NilRL n = n
 commuteOrAddToCtxRL (ps:<:p) n = commuteOrAddToCtxRL ps $ commuteOrAddToCtx p n
@@ -175,7 +181,7 @@ instance WL RL where
 
 -- |commutePrimsOrAddToCtx takes a WL of prims and attempts to commute them
 -- past a Non.
-commutePrimsOrAddToCtx :: (WL l, Patchy p, ToFromPrim p) => l (PrimOf p) wX wY
+commutePrimsOrAddToCtx :: (WL l, Apply p, Commute p, Invert p, ToFromPrim p) => l (PrimOf p) wX wY
          -> Non p wY -> Non p wX
 commutePrimsOrAddToCtx q = commuteOrAddToCtxRL (mapRL_RL fromPrim $ toRL q)
 
@@ -194,14 +200,14 @@ commutePrimsOrAddToCtx q = commuteOrAddToCtxRL (mapRL_RL fromPrim $ toRL q)
 -- modify p2, so that it doesn't have any of a1' in its context.
 
 -- remNons really only works right if the relevant nons are conflicting...
-remNons :: (Nonable p, Effect p, Patchy p, MyEq p, ToFromPrim p, PrimPatchBase p,
-        MyEq (PrimOf p)) => [Non p wX] -> Non p wX -> Non p wX
+remNons :: (Nonable p, Effect p, Apply p, Commute p, Invert p, Eq2 p, ToFromPrim p, PrimPatchBase p)
+        => [Non p wX] -> Non p wX -> Non p wX
 remNons ns n@(Non c x) = case remNonHelper ns c of
                              NilFL :> c' -> Non c' x
                              _ -> n
   where
-    remNonHelper :: (Nonable p, Effect p, Patchy p, MyEq p, ToFromPrim p,
-                 PrimPatchBase p, MyEq (PrimOf p)) => [Non p wX]
+    remNonHelper :: (Nonable p, Effect p, Apply p, Commute p, Invert p, Eq2 p, ToFromPrim p,
+                 PrimPatchBase p) => [Non p wX]
                  -> FL p wX wY -> (FL (PrimOf p) :> FL p) wX wY
     remNonHelper [] x = NilFL :> x
     remNonHelper _ NilFL = NilFL :> NilFL
@@ -223,26 +229,26 @@ remNons ns n@(Non c x) = case remNonHelper ns c of
 --
 -- TODO: understand if there is any case where p is equal to the prim patch of
 -- the Non, in which case, we return the original Non, is that right?
-commuteOrRemFromCtx :: (Patchy p, MyEq p, ToFromPrim p) => p wX wY -> Non p wX
+commuteOrRemFromCtx :: (Commute p, Invert p, Eq2 p, ToFromPrim p) => p wX wY -> Non p wX
      -> Maybe (Non p wY)
 commuteOrRemFromCtx p n | n'@(Just _) <- n *> p = n'
 commuteOrRemFromCtx p (Non pc x) = removeFL p pc >>= \c -> return (Non c x)
 
 -- |commuteOrRemFromCtxFL attempts to remove a FL of patches from a Non,
 -- returning Nothing if any of the individual removes fail.
-commuteOrRemFromCtxFL :: (Patchy p, MyEq p, ToFromPrim p) => FL p wX wY -> Non p wX
+commuteOrRemFromCtxFL :: (Apply p, Commute p, Invert p, Eq2 p, ToFromPrim p) => FL p wX wY -> Non p wX
                       -> Maybe (Non p wY)
 commuteOrRemFromCtxFL NilFL n = Just n
 commuteOrRemFromCtxFL (p:>:ps) n = do n' <- commuteOrRemFromCtx p n
                                       commuteOrRemFromCtxFL ps n'
 
 -- |(*>) attemts to modify a Non by commuting it past a given patch.
-(*>) :: (Patchy p, ToFromPrim p) => Non p wX -> p wX wY
+(*>) :: (Commute p, Invert p, ToFromPrim p) => Non p wX -> p wX wY
      -> Maybe (Non p wY)
 n *> p = invert p >* n
 
 -- |(>*) attempts to modify a Non, by commuting a given patch past it.
-(>*) :: (Patchy p, ToFromPrim p) => p wX wY -> Non p wY
+(>*) :: (Commute p, ToFromPrim p) => p wX wY -> Non p wY
      -> Maybe (Non p wX)
 y >* (Non c x) = do
     c' :> y' <- commuteFL (y :> c)
@@ -251,16 +257,16 @@ y >* (Non c x) = do
     return (Non c' x')
 
 -- |(*>>) attempts to modify a Non by commuting it past a given WL of patches.
-(*>>) :: (WL l, Patchy p, ToFromPrim p, PrimPatchBase p) => Non p wX
+(*>>) :: (WL l, Apply p, Commute p, Invert p, ToFromPrim p, PrimPatchBase p) => Non p wX
       -> l (PrimOf p) wX wY -> Maybe (Non p wY)
 n *>> p = invertWL p >>* n
 
 -- |(>>*) attempts to modify a Non by commuting a given WL of patches past it.
-(>>*) :: (WL l, Patchy p, ToFromPrim p) => l (PrimOf p) wX wY -> Non p wY
+(>>*) :: (WL l, Apply p, Commute p, Invert p, ToFromPrim p) => l (PrimOf p) wX wY -> Non p wY
       -> Maybe (Non p wX)
 ps >>* n = commuteRLPastNon (toRL ps) n
   where
-    commuteRLPastNon :: (Patchy p, ToFromPrim p) => RL (PrimOf p) wX wY
+    commuteRLPastNon :: (Apply p, Commute p, Invert p, ToFromPrim p) => RL (PrimOf p) wX wY
                      -> Non p wY -> Maybe (Non p wX)
     commuteRLPastNon NilRL n = Just n
     commuteRLPastNon (xs:<:x) n = fromPrim x >* n >>= commuteRLPastNon xs
