@@ -16,12 +16,9 @@
 -- Boston, MA 02110-1301, USA.
 
 {-# OPTIONS_GHC -fno-warn-orphans -fno-warn-name-shadowing #-}
-{-# LANGUAGE CPP #-}
-
 
 module Darcs.Patch.V2.RepoPatch
     ( RepoPatchV2(..)
-    , prim2repopatchV2
     , isConsistent
     , isForward
     , isDuplicate
@@ -35,43 +32,45 @@ import Control.Monad ( mplus, liftM )
 import qualified Data.ByteString.Char8 as BC ( ByteString, pack )
 import Data.Maybe ( fromMaybe )
 import Data.List ( partition, nub )
+import Data.List.Ordered ( nubSort )
 
+import Darcs.Patch.Annotate ( Annotate(..) )
 import Darcs.Patch.Commute ( commuteFL, commuteFLorComplain, commuteRL
-                           , commuteRLFL )
+                           , commuteRLFL, Commute(..) )
 import Darcs.Patch.Conflict ( Conflict(..), CommuteNoConflicts(..)
                             , IsConflictedPrim(..), ConflictState(..)
                             , mangleUnravelled )
 import Darcs.Patch.Debug
 import Darcs.Patch.Effect ( Effect(..) )
 import Darcs.Patch.FileHunk ( IsHunk(..) )
-import Darcs.Patch.Format ( PatchListFormat(..), ListFormat(..)
-                          , FileNameFormat(NewFormat) )
-import Darcs.Patch.Invert ( invertFL, invertRL )
-import Darcs.Patch.Merge ( Merge(..) )
-import Darcs.Patch.Prim ( FromPrim(..), ToFromPrim(..), showPrim, showPrimFL
-                        , readPrim, PrimOf, PrimPatchBase, PrimPatch )
-import Darcs.Patch.Read ( bracketedFL )
+import Darcs.Patch.Format ( PatchListFormat(..), ListFormat(ListFormatV2) )
+import Darcs.Patch.Invert ( invertFL, invertRL, Invert(..) )
+import Darcs.Patch.Merge ( Merge(..), naturalMerge )
+import Darcs.Patch.Prim ( FromPrim(..), ToFromPrim(..)
+                        , PrimPatchBase(..), PrimPatch )
+import Darcs.Patch.Read ( bracketedFL, ReadPatch(..) )
 import Darcs.Patch.ReadMonads ( skipSpace, string, choice )
 import Darcs.Patch.Repair ( mapMaybeSnd, RepairToFL(..), Check(..) )
-import Darcs.Patch.Patchy ( Patchy, Apply(..), Commute(..), PatchInspect(..)
-                          , ReadPatch(..), ShowPatch(..), Invert(..) )
+import Darcs.Patch.Apply ( Apply(..) )
+import Darcs.Patch.Inspect ( PatchInspect(..) )
 import Darcs.Patch.Permutations ( commuteWhatWeCanFL, commuteWhatWeCanRL
                                 , genCommuteWhatWeCanRL, removeRL, removeFL
                                 , removeSubsequenceFL )
-import Darcs.Patch.Show ( ShowPatchBasic(..) )
+import Darcs.Patch.Show
+    ( ShowPatch(..), ShowPatchBasic(..), ShowContextPatch(..), ShowPatchFor(..)
+    , displayPatch )
 import Darcs.Patch.Summary ( plainSummary )
 import Darcs.Patch.V2.Non ( Non(..), Nonable(..), unNon, showNons, showNon
                           , readNons, readNon, commutePrimsOrAddToCtx
                           , commuteOrAddToCtx, commuteOrAddToCtxRL
                           , commuteOrRemFromCtx, commuteOrRemFromCtxFL
                           , remNons, (*>), (>*), (*>>), (>>*) )
-import Data.List.Ordered ( nubSort )
 import Darcs.Patch.Witnesses.Unsafe ( unsafeCoerceP )
-import Darcs.Patch.Witnesses.Eq ( MyEq(..), EqCheck(..) )
+import Darcs.Patch.Witnesses.Eq ( Eq2(..), EqCheck(..) )
 import Darcs.Patch.Witnesses.Ordered
     ( FL(..), RL(..), (:>)(..), (+>+), (+<+)
-    , mapFL_FL, reverseFL, (:\/:)(..), (:/\:)(..)
-    , reverseRL, lengthFL, lengthRL, nullFL )
+    , mapFL, mapFL_FL, reverseFL, (:\/:)(..), (:/\:)(..)
+    , reverseRL, lengthFL, lengthRL, nullFL, initsFL )
 import Darcs.Patch.Witnesses.Sealed
     ( FlippedSeal(..), Sealed(Sealed), mapSeal
     , unseal )
@@ -79,11 +78,9 @@ import Darcs.Patch.Witnesses.Show
     ( Show1(..), Show2(..), ShowDict(..)
     , showsPrec2, appPrec
     )
+
 import Darcs.Util.Printer.Color ( errorDoc, assertDoc )
-
-import Darcs.Util.Printer ( Doc, blueText, redText, (<+>), ($$) )
-
-#include "impossible.h"
+import Darcs.Util.Printer ( Doc, blueText, redText, (<+>), ($$), vcat )
 
 -- |'RepoPatchV2' is used to represents prim patches that are duplicates of, or
 -- conflict with, another prim patch in the repository.
@@ -141,7 +138,7 @@ isForward p = case p of
     p@(Etacilpud _) -> justRedP "An inverse duplicate" p
     _ -> Nothing
   where
-    justRedP msg p = Just $ redText msg $$ showPatch p
+    justRedP msg p = Just $ redText msg $$ displayPatch p
 
 -- |'mergeUnravelled' is used when converting from Darcs V1 patches (Mergers)
 -- to Darcs V2 patches (Conflictors).
@@ -155,7 +152,7 @@ mergeUnravelled ws =
         Just NilRL -> bug "found no patches in mergeUnravelled"
         Just (_ :<: z) -> Just $ FlippedSeal z
   where
-    notNullS :: PrimPatch prim => Sealed ((FL prim) wX) -> Bool
+    notNullS :: Sealed ((FL prim) wX) -> Bool
     notNullS (Sealed NilFL) = False
     notNullS _ = True
 
@@ -191,7 +188,7 @@ mergeConflictingNons ns = mcn $ map unNon ns
                         qs' :/\: _ -> mcn (Sealed (c +>+ ps +>+ qs'):zs)
 
 joinEffects :: forall p wX wY . (Effect p, Invert (PrimOf p),
-            Commute (PrimOf p), MyEq (PrimOf p)) => p wX wY
+            Commute (PrimOf p), Eq2 (PrimOf p)) => p wX wY
             -> FL (PrimOf p) wX wY
 joinEffects = joinInverses . effect
     where joinInverses :: FL (PrimOf p) wA wB -> FL (PrimOf p) wA wB
@@ -204,7 +201,7 @@ assertConsistent :: PrimPatch prim => RepoPatchV2 prim wX wY
                  -> RepoPatchV2 prim wX wY
 assertConsistent x = flip assertDoc x $ do
     e <- isConsistent x
-    Just (redText "Inconsistent patch:" $$ showPatch x $$ e)
+    Just (redText "Inconsistent patch:" $$ displayPatch x $$ e)
 
 -- | @mergeAfterConflicting@ takes as input a sequence of conflicting patches
 -- @xxx@ (which therefore have no effect) and a sequence of primitive patches
@@ -267,7 +264,7 @@ geteff ix xx =
     case mergeConflictingNons ix of
         Nothing -> errorDoc $
             redText "mergeConflictingNons failed in geteff: ix" $$
-            showNons ix $$ redText "xx" $$ showPatch xx
+            displayNons ix $$ redText "xx" $$ displayPatch xx
         Just rix ->
             case mergeAfterConflicting rix xx of
                 Just (a, x) ->
@@ -276,9 +273,9 @@ geteff ix xx =
                 Nothing ->
                     errorDoc $
                         redText "mergeAfterConflicting failed in geteff" $$
-                        redText "where ix" $$ showNons ix $$
-                        redText "and xx" $$ showPatch xx $$
-                        redText "and rix" $$ showPatch rix
+                        redText "where ix" $$ displayNons ix $$
+                        redText "and xx" $$ displayPatch xx $$
+                        redText "and rix" $$ displayPatch rix
 
 xx2nons :: PrimPatch prim => [Non (RepoPatchV2 prim) wX] -> FL prim wX wY
         -> [Non (RepoPatchV2 prim) wX]
@@ -309,18 +306,18 @@ isConsistent (Conflictor im mm m@(Non deps _))
         Just $ redText "m doesn't conflict with mm in isConsistent"
     | any (\x -> any (x `conflictsWith`) nmm) im =
         Just $ redText "mm conflicts with im in isConsistent where nmm is" $$
-               showNons nmm
+               displayNons nmm
     | Nothing <- (nmm ++ im) `minus` toNons deps =
         Just $ redText "dependencies not in conflict:" $$
-               showNons (toNons deps) $$
+               displayNons (toNons deps) $$
                redText "compared with deps itself:" $$
-               showPatch deps
+               displayPatch deps
     | otherwise =
         case allConflictsWith m im of
             (im1, []) | im1 `eqSet` im -> Nothing
             (_, imnc) -> Just $ redText ("m doesn't conflict with im in "
                                          ++ "isConsistent. unconflicting:") $$
-                                showNons imnc
+                                displayNons imnc
     where (nmm, rmm) = geteff im mm
 
 everyoneConflicts :: PrimPatch prim => [Non (RepoPatchV2 prim) wX] -> Bool
@@ -329,10 +326,6 @@ everyoneConflicts (x : xs) = case allConflictsWith x xs of
                                  ([], _) -> False
                                  (_, xs') -> everyoneConflicts xs'
 
-prim2repopatchV2 :: prim wX wY -> RepoPatchV2 prim wX wY
-prim2repopatchV2 = Normal
-
-instance PrimPatch prim => Patchy (RepoPatchV2 prim)
 instance PatchDebug prim => PatchDebug (RepoPatchV2 prim)
 
 mergeWith :: PrimPatch prim => Non (RepoPatchV2 prim) wX
@@ -470,13 +463,13 @@ instance PrimPatch prim => Check (RepoPatchV2 prim) where
     isInconsistent = isConsistent
 
 instance FromPrim (RepoPatchV2 prim) where
-    fromPrim = prim2repopatchV2
+    fromPrim = Normal
 
 instance ToFromPrim (RepoPatchV2 prim) where
     toPrim (Normal p) = Just p
     toPrim _ = Nothing
 
-instance PrimPatch prim => MyEq (RepoPatchV2 prim) where
+instance PrimPatch prim => Eq2 (RepoPatchV2 prim) where
     (Duplicate x) =\/= (Duplicate y) | x == y = IsEq
     (Etacilpud x) =\/= (Etacilpud y) | x == y = IsEq
     (Normal x) =\/= (Normal y) = x =\/= y
@@ -519,8 +512,8 @@ nonHunkMatches :: PatchInspect prim => (BC.ByteString -> Bool)
                -> Non (RepoPatchV2 prim) wX -> Bool
 nonHunkMatches f (Non c x) = hunkMatches f c || hunkMatches f x
 
-toNons :: forall p wX wY . (Conflict p, Patchy p, PatchListFormat p,
-       ToFromPrim p, Nonable p, ShowPatchBasic (PrimOf p), ShowPatchBasic p)
+toNons :: forall p wX wY . (Commute p, PatchListFormat p,
+       Nonable p, ShowPatchBasic (PrimOf p), ShowPatchBasic p)
        => FL p wX wY -> [Non p wX]
 toNons xs = map lastNon $ initsFL xs
     where lastNon :: Sealed ((p :> FL p) wX) -> Non p wX
@@ -533,9 +526,9 @@ toNons xs = map lastNon $ initsFL xs
                                errorDoc $ redText "Weird case in toNons" $$
                                           redText "please report this bug!" $$
                                           (case xxx of
-                                           z :> zs -> showPatch (z :>: zs)) $$
-                                          redText "ds are" $$ showPatch ds $$
-                                          redText "pp is" $$ showPatch pp
+                                           z :> zs -> displayPatch (z :>: zs)) $$
+                                          redText "ds are" $$ displayPatch ds $$
+                                          redText "pp is" $$ displayPatch pp
 
           reverseFoo :: (p :> FL p) wX wZ -> (RL p :> p) wX wZ
           reverseFoo (p :> ps) = rf NilRL p ps
@@ -548,12 +541,6 @@ toNons xs = map lastNon $ initsFL xs
           lastNon_aux :: (p :> FL p) wX wZ -> (RL p :> p :> RL p) wX wZ
           lastNon_aux = commuteWhatWeCanRL . reverseFoo
 
-initsFL :: Patchy p => FL p wX wY -> [Sealed ((p :> FL p) wX)]
-initsFL NilFL = []
-initsFL (x :>: xs) =
-    Sealed (x :> NilFL) :
-    map (\(Sealed (y :> xs')) -> Sealed (x :> y :>: xs')) (initsFL xs)
-
 filterConflictsFL :: PrimPatch prim => Non (RepoPatchV2 prim) wX
                   -> FL prim wX wY -> (FL prim :> FL prim) wX wY
 filterConflictsFL _ NilFL = NilFL :> NilFL
@@ -651,17 +638,11 @@ instance PrimPatch prim => Merge (RepoPatchV2 prim) where
     -- Handle using the swap merge and the previous case.
     merge m@(_ :\/: Duplicate _) = swapMerge m
 
-    -- When merging x and y, we do a bunch of what look like "consistency"
-    -- check merges. If the resulting y'' and y are equal, then we succeed.
-    -- If the first case fails, we check for equal patches (which wouldn't
-    -- commute) and return a Duplicate on both sides of the merge, in that
-    -- case.
     merge (x :\/: y)
-        | Just (y' :> ix') <-
-            commute (invert (assertConsistent x) :> assertConsistent y)
-        , Just (y'' :> _) <- commute (x :> y')
-        , IsEq <- y'' =\/= y =
-            assertConsistent y' :/\: invert (assertConsistent ix')
+        -- First try the natural (non-conflicting) merge.
+        | Just (y' :/\: x') <-
+            naturalMerge ((assertConsistent x) :\/: (assertConsistent y))
+              = assertConsistent y' :/\: assertConsistent x'
         -- If we detect equal patches, we have a duplicate.
         | IsEq <- x =\/= y
         , n <- commuteOrAddToCtx (invert x) $ non x =
@@ -768,12 +749,12 @@ x +++ [] = x
 (x:xs) +++ xys | Just ys <- remove1 x xys = x : (xs +++ ys)
                | otherwise = x : (xs +++ xys)
 
-swapMerge :: PrimPatch prim => (RepoPatchV2 prim :\/: RepoPatchV2 prim) wX wY
-          -> (RepoPatchV2 prim :/\: RepoPatchV2 prim) wX wY
+swapMerge :: Merge p => (p :\/: p) wX wY
+          -> (p :/\: p) wX wY
 swapMerge (x :\/: y) = case merge (y :\/: x) of x' :/\: y' -> y' :/\: x'
 
-invertCommute :: PrimPatch prim => (RepoPatchV2 prim :> RepoPatchV2 prim) wX wY
-              -> Maybe ((RepoPatchV2 prim :> RepoPatchV2 prim) wX wY)
+invertCommute :: (Invert p, Commute p) => (p :> p) wX wY
+              -> Maybe ((p :> p) wX wY)
 invertCommute (x :> y) = do ix' :> iy' <- commute (invert y :> invert x)
                             return (invert iy' :> invert ix')
 
@@ -785,7 +766,7 @@ invertCommuteNC (x :> y) = do
 
 -- | 'pullCommon' @xs ys@ returns the set of patches that can be commuted out
 -- of both @xs@ and @ys@ along with the remnants of both lists
-pullCommon :: (Patchy p, MyEq p) => FL p wO wX -> FL p wO wY -> Common p wO wX wY
+pullCommon :: (Commute p, Eq2 p) => FL p wO wX -> FL p wO wY -> Common p wO wX wY
 pullCommon NilFL ys = Common NilFL NilFL ys
 pullCommon xs NilFL = Common NilFL xs NilFL
 pullCommon (x :>: xs) xys | Just ys <- removeFL x xys =
@@ -803,7 +784,7 @@ data Common p wO wX wY where
 
 -- | 'pullCommonRL' @xs ys@ returns the set of patches that can be commuted
 --   out of both @xs@ and @ys@ along with the remnants of both lists
-pullCommonRL :: (Patchy p, MyEq p) => RL p wX wO -> RL p wY wO -> CommonRL p wX wY wO
+pullCommonRL :: (Commute p, Eq2 p) => RL p wX wO -> RL p wY wO -> CommonRL p wX wY wO
 pullCommonRL NilRL ys = CommonRL NilRL ys NilRL
 pullCommonRL xs NilRL = CommonRL xs NilRL NilRL
 pullCommonRL (xs :<: x) xys | Just ys <- removeRL x xys =
@@ -829,6 +810,9 @@ instance PrimPatch prim => RepairToFL (RepoPatchV2 prim) where
         mapMaybeSnd (mapFL_FL Normal) `liftM` applyAndTryToFixFL p
     applyAndTryToFixFL x = do apply x; return Nothing
 
+instance (PrimPatch prim, Annotate prim) => Annotate (RepoPatchV2 prim) where
+    annotate = annotate . effect
+
 instance PatchListFormat (RepoPatchV2 prim) where
    -- In principle we could use ListFormatDefault when prim /= V1 Prim patches,
    -- as those are the only case where we need to support a legacy on-disk
@@ -843,27 +827,29 @@ conflictor = "conflictor"
 rotcilfnoc = "rotcilfnoc"
 
 instance PrimPatch prim => ShowPatchBasic (RepoPatchV2 prim) where
-    showPatch (Duplicate d) = blueText duplicate $$ showNon d
-    showPatch (Etacilpud d) = blueText etacilpud $$ showNon d
-    showPatch (Normal p) = showPrim NewFormat p
-    showPatch (Conflictor i NilFL p) =
-        blueText conflictor <+> showNons i <+> blueText "[]" $$ showNon p
-    showPatch (Conflictor i cs p) =
-        blueText conflictor <+> showNons i <+> blueText "[" $$
-        showPrimFL NewFormat cs $$
+    showPatch f (Duplicate d) = blueText duplicate $$ showNon f d
+    showPatch f (Etacilpud d) = blueText etacilpud $$ showNon f d
+    showPatch f (Normal p) = showPatch f p
+    showPatch f (Conflictor i NilFL p) =
+        blueText conflictor <+> showNons f i <+> blueText "[]" $$ showNon f p
+    showPatch f (Conflictor i cs p) =
+        blueText conflictor <+> showNons f i <+> blueText "[" $$
+        showFL f cs $$
         blueText "]" $$
-        showNon p
-    showPatch (InvConflictor i NilFL p) =
-        blueText rotcilfnoc <+> showNons i <+> blueText "[]" $$ showNon p
-    showPatch (InvConflictor i cs p) =
-        blueText rotcilfnoc <+> showNons i <+> blueText "[" $$
-        showPrimFL NewFormat cs $$
+        showNon f p
+    showPatch f (InvConflictor i NilFL p) =
+        blueText rotcilfnoc <+> showNons f i <+> blueText "[]" $$ showNon f p
+    showPatch f (InvConflictor i cs p) =
+        blueText rotcilfnoc <+> showNons f i <+> blueText "[" $$
+        showFL f cs $$
         blueText "]" $$
-        showNon p
+        showNon f p
+
+instance PrimPatch prim => ShowContextPatch (RepoPatchV2 prim) where
+    showContextPatch f (Normal p) = showContextPatch f p
+    showContextPatch f p = return $ showPatch f p
 
 instance PrimPatch prim => ShowPatch (RepoPatchV2 prim) where
-    showContextPatch (Normal p) = showContextPatch p
-    showContextPatch c = return $ showPatch c
     summary = plainSummary
     summaryFL = plainSummary
     thing _ = "change"
@@ -874,7 +860,7 @@ instance PrimPatch prim => ReadPatch (RepoPatchV2 prim) where
         let str = string . BC.pack
             readConflictorPs = do
                i <- readNons
-               ps <- bracketedFL (readPrim NewFormat) '[' ']'
+               ps <- bracketedFL readPatch' '[' ']'
                p <- readNon
                return (i, ps, p)
         choice [ do str duplicate
@@ -889,29 +875,29 @@ instance PrimPatch prim => ReadPatch (RepoPatchV2 prim) where
                , do str rotcilfnoc
                     (i, Sealed ps, p) <- readConflictorPs
                     return $ Sealed $ InvConflictor i ps p
-               , do Sealed p <- readPrim NewFormat
+               , do Sealed p <- readPatch'
                     return $ Sealed $ Normal p
                ]
 
 instance Show2 prim => Show (RepoPatchV2 prim wX wY) where
     showsPrec d (Normal prim) =
-        showParen (d > appPrec) $ showString "Darcs.Patch.V2.RepoPatch.Normal " . showsPrec2 (appPrec + 1) prim
+        showParen (d > appPrec) $ showString "Normal " . showsPrec2 (appPrec + 1) prim
 
     showsPrec d (Duplicate x) =
-        showParen (d > appPrec) $ showString "Darcs.Patch.V2.RepoPatch.Duplicate " . showsPrec (appPrec + 1) x
+        showParen (d > appPrec) $ showString "Duplicate " . showsPrec (appPrec + 1) x
 
     showsPrec d (Etacilpud x) =
-        showParen (d > appPrec) $ showString "Darcs.Patch.V2.Etacilpud " . showsPrec (appPrec + 1) x
+        showParen (d > appPrec) $ showString "Etacilpud " . showsPrec (appPrec + 1) x
 
     showsPrec d (Conflictor ix xx x) =
         showParen (d > appPrec) $
-            showString "Darcs.Patch.V2.RepoPatch.Conflictor " . showsPrec (appPrec + 1) ix .
+            showString "Conflictor " . showsPrec (appPrec + 1) ix .
             showString " " . showsPrec (appPrec + 1) xx .
             showString " " . showsPrec (appPrec + 1) x
 
     showsPrec d (InvConflictor ix xx x) =
         showParen (d > appPrec) $
-            showString "Darcs.Patch.V2.RepoPatch.InvConflictor " . showsPrec (appPrec + 1) ix .
+            showString "InvConflictor " . showsPrec (appPrec + 1) ix .
             showString " " . showsPrec (appPrec + 1) xx .
             showString " " . showsPrec (appPrec + 1) x
 
@@ -943,3 +929,10 @@ instance PrimPatch prim => Effect (RepoPatchV2 prim) where
 instance IsHunk prim => IsHunk (RepoPatchV2 prim) where
     isHunk rp = do Normal p <- return rp
                    isHunk p
+
+displayNons :: (PatchListFormat p, ShowPatchBasic p, PrimPatchBase p) =>
+               [Non p wX] -> Doc
+displayNons p = showNons ForDisplay p
+
+showFL :: ShowPatchBasic p => ShowPatchFor -> FL p wX wY -> Doc
+showFL f = vcat . mapFL (showPatch f)
