@@ -16,18 +16,16 @@
 -- Boston, MA 02110-1301, USA.
 
 {-# OPTIONS_GHC -fno-warn-orphans -fno-warn-unused-imports #-}
-{-# LANGUAGE CPP #-}
 
 module Darcs.Patch.Viewing
     ( showContextHunk
-    , showContextSeries
     ) where
 
 import Prelude ()
 import Darcs.Prelude
 
 import Control.Applicative( (<$>) )
-import qualified Data.ByteString as BS ( null )
+import qualified Data.ByteString as B ( null )
 import Prelude hiding ( pi, readFile )
 import Darcs.Util.Tree ( Tree )
 import Darcs.Util.Tree.Monad ( virtualTreeMonad )
@@ -38,24 +36,26 @@ import Darcs.Patch.ApplyMonad ( getApplyState,
 import Darcs.Patch.FileHunk ( IsHunk(..), FileHunk(..), showFileHunk )
 import Darcs.Patch.Format ( PatchListFormat(..), ListFormat(..),
                             FileNameFormat(..) )
-import Darcs.Patch.Show ( ShowPatchBasic(..), ShowPatch(..), formatFileName )
+import Darcs.Patch.Show
+    ( ShowPatchBasic(..), ShowPatch(..)
+    , formatFileName, ShowPatchFor(..), ShowContextPatch(..) )
 import Darcs.Patch.Witnesses.Ordered ( RL(..), FL(..), mapFL, mapFL_FL,
                                        reverseRL, concatFL )
 import Darcs.Util.ByteString ( linesPS )
 import Darcs.Util.Printer ( Doc, empty, vcat, text, blueText, Color(Cyan, Magenta),
                  lineColor, ($$), (<+>), prefix, userchunkPS )
 
-showContextSeries :: forall p m wX wY . (Apply p, ShowPatch p, IsHunk p,
+showContextSeries :: forall p m wX wY . (Apply p, ShowContextPatch p, IsHunk p,
                                          ApplyMonad (ApplyState p) m)
-                  => FL p wX wY -> m Doc
-showContextSeries = scs Nothing
+                  => ShowPatchFor -> FileNameFormat -> FL p wX wY -> m Doc
+showContextSeries use fmt = scs Nothing
   where
     scs :: forall wWw wXx wYy . Maybe (FileHunk wWw wXx) -> FL p wXx wYy -> m Doc
     scs pold (p :>: ps) = do
         (_, s') <- nestedApply (apply p) =<< getApplyState
         case isHunk p of
             Nothing -> do
-                a <- showContextPatch p
+                a <- showContextPatch use p
                 b <- nestedApply (scs Nothing ps) s'
                 return $ a $$ fst b
             Just fh -> case ps of
@@ -69,29 +69,30 @@ showContextSeries = scs Nothing
     cool :: Maybe (FileHunk wA wB) -> FileHunk wB wC -> Maybe (FileHunk wC wD)
          -> (ApplyState p) (ApplyMonadBase m) -> (ApplyMonadBase m) Doc
     cool pold fh ps s =
-        fst <$> virtualTreeMonad (coolContextHunk pold fh ps) (toTree s)
+        fst <$> virtualTreeMonad (coolContextHunk fmt pold fh ps) (toTree s)
 
-showContextHunk :: (ApplyMonad Tree m) => FileHunk wX wY -> m Doc
-showContextHunk h = coolContextHunk Nothing h Nothing
+showContextHunk :: (ApplyMonad Tree m) => FileNameFormat -> FileHunk wX wY -> m Doc
+showContextHunk fmt h = coolContextHunk fmt Nothing h Nothing
 
 coolContextHunk :: (ApplyMonad Tree m)
-                => Maybe (FileHunk wA wB) -> FileHunk wB wC
+                => FileNameFormat
+                -> Maybe (FileHunk wA wB) -> FileHunk wB wC
                 -> Maybe (FileHunk wC wD) -> m Doc
-coolContextHunk prev fh@(FileHunk f l o n) next = do
+coolContextHunk fmt prev fh@(FileHunk f l o n) next = do
     have <- mDoesFileExist f
     content <- if have then Just `fmap` mReadFilePS f else return Nothing
     case linesPS `fmap` content of
-        -- This is a weird error...
-        Nothing -> return $ showFileHunk OldFormat fh
+        -- FIXME This is a weird error...
+        Nothing -> return $ showFileHunk fmt fh
         Just ls ->
             let pre = take numpre $ drop (l - numpre - 1) ls
                 cleanedls = case reverse ls of
                     (x : xs)
-                        | BS.null x -> reverse xs
+                        | B.null x -> reverse xs
                     _ -> ls
                 post = take numpost $ drop (max 0 $ l+length o-1) cleanedls in
             return $
-                blueText "hunk" <+> formatFileName OldFormat f
+                blueText "hunk" <+> formatFileName fmt f
                     <+> text (show l)
                 $$ prefix " " (vcat $ map userchunkPS pre)
                 $$ lineColor Magenta (prefix "-" (vcat $ map userchunkPS o))
@@ -111,33 +112,36 @@ coolContextHunk prev fh@(FileHunk f l o n) next = do
         _ -> 3
 
 instance (PatchListFormat p, ShowPatchBasic p) => ShowPatchBasic (FL p) where
-    showPatch = showPatchInternal patchListFormat
+    showPatch ForDisplay = vcat . mapFL (showPatch ForDisplay)
+    showPatch ForStorage = showPatchInternal patchListFormat
       where
         showPatchInternal :: ListFormat p -> FL p wX wY -> Doc
-        showPatchInternal ListFormatV1 (p :>: NilFL) = showPatch p
+        showPatchInternal ListFormatV1 (p :>: NilFL) = (showPatch ForStorage) p
         showPatchInternal ListFormatV1 NilFL = blueText "{" $$ blueText "}"
         showPatchInternal ListFormatV1 ps = blueText "{"
-                                            $$ vcat (mapFL showPatch ps)
+                                            $$ vcat (mapFL (showPatch ForStorage) ps)
                                             $$ blueText "}"
-        showPatchInternal ListFormatV2 ps = vcat (mapFL showPatch ps)
-        showPatchInternal ListFormatDefault ps = vcat (mapFL showPatch ps)
+        showPatchInternal ListFormatV2 ps = vcat (mapFL (showPatch ForStorage) ps)
+        showPatchInternal ListFormatDefault ps = vcat (mapFL (showPatch ForStorage) ps)
 
-instance (Apply p, IsHunk p, PatchListFormat p, ShowPatch p)
-        => ShowPatch (FL p) where
-    showContextPatch = showContextPatchInternal patchListFormat
+instance (Apply p, IsHunk p, PatchListFormat p, ShowContextPatch p)
+        => ShowContextPatch (FL p) where
+    showContextPatch ForDisplay = showContextSeries ForDisplay UserFormat
+    showContextPatch ForStorage = showContextPatchInternal patchListFormat
       where
         showContextPatchInternal :: (ApplyMonad (ApplyState (FL p)) m)
                                  => ListFormat p -> FL p wX wY -> m Doc
         showContextPatchInternal ListFormatV1 (p :>: NilFL) =
-            showContextPatch p
+            showContextPatch ForStorage p
         showContextPatchInternal ListFormatV1 NilFL =
             return $ blueText "{" $$ blueText "}"
         showContextPatchInternal ListFormatV1 ps = do
-            x <- showContextSeries ps
+            x <- showContextSeries ForStorage OldFormat ps
             return $ blueText "{" $$ x $$ blueText "}"
-        showContextPatchInternal ListFormatV2 ps = showContextSeries ps
-        showContextPatchInternal ListFormatDefault ps = showContextSeries ps
+        showContextPatchInternal ListFormatV2 ps = showContextSeries ForStorage NewFormat ps
+        showContextPatchInternal ListFormatDefault ps = showContextSeries ForStorage NewFormat ps
 
+instance (PatchListFormat p, ShowPatch p) => ShowPatch (FL p) where
     description = vcat . mapFL description
 
     summary = summaryFL
@@ -152,12 +156,13 @@ instance (Apply p, IsHunk p, PatchListFormat p, ShowPatch p)
     things = thing
 
 instance (PatchListFormat p, ShowPatchBasic p) => ShowPatchBasic (RL p) where
-    showPatch = showPatch . reverseRL
+    showPatch f = showPatch f . reverseRL
 
-instance (Apply p, IsHunk p, PatchListFormat p, ShowPatch p)
-        => ShowPatch (RL p) where
-    showContextPatch = showContextPatch . reverseRL
+instance (ShowContextPatch p, Apply p, IsHunk p, PatchListFormat p)
+        => ShowContextPatch (RL p) where
+    showContextPatch use = showContextPatch use . reverseRL
 
+instance (PatchListFormat p, ShowPatch p) => ShowPatch (RL p) where
     description = description . reverseRL
 
     summary = summary . reverseRL
