@@ -1,6 +1,6 @@
 module Darcs.Patch.Witnesses.Eq
     ( EqCheck(..)
-    , MyEq(..)
+    , Eq2(..)
     , isIsEq
     ) where
 
@@ -26,7 +26,7 @@ instance Show (EqCheck wA wB) where
 
 -- |An witness aware equality class.
 -- A minimal definition defines any one of 'unsafeCompare', '=\/=' and '=/\='.
-class MyEq p where
+class Eq2 p where
     -- |It is unsafe to define a class instance via this method, because
     -- if it returns True then the default implementations of '=\/=' and '=/\='
     -- will coerce the equality of two witnesses.
