@@ -15,8 +15,6 @@
 -- the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 -- Boston, MA 02110-1301, USA.
 
-{-# LANGUAGE CPP #-}
-
 module Darcs.Patch.Witnesses.Ordered
     ( 
     -- * Directed Types
@@ -30,50 +28,58 @@ module Darcs.Patch.Witnesses.Ordered
     , (:/\:)(..)
     , (:||:)(..)
     , Fork(..)
-    -- * Functions
+    -- * Functions for 'FL's and 'RL's
+    , nullFL
+    , nullRL
     , lengthFL
+    , lengthRL
     , mapFL
+    , mapRL
     , mapFL_FL
-    , spanFL
+    , mapRL_RL
     , foldlFL
+    , foldlRL
     , allFL
+    , allRL
     , anyFL
+    , anyRL
     , filterFL
+    , filterRL
     , splitAtFL
     , splitAtRL
-    , bunchFL
-    , foldlRL
-    , lengthRL
-    , isShorterThanRL
-    , mapRL
-    , mapRL_RL
-    , zipWithFL
     , filterOutFLFL
     , filterOutRLRL
-    , filterRL
     , reverseFL
     , reverseRL
     , (+>+)
     , (+<+)
-    , nullFL
+    , (+>>+)
+    , (+<<+)
     , concatFL
     , concatRL
-    , snocRLSealed
-    , nullRL
-    , toFL
     , dropWhileFL
     , dropWhileRL
+    -- * 'FL' only
+    , bunchFL
+    , foldFL_M
+    , spanFL
     , spanFL_M
+    , zipWithFL
+    , toFL
     , mapFL_FL_M
+    , sequenceFL_
     , eqFL
     , eqFLRev
     , eqFLUnsafe
+    , initsFL
+    -- * 'RL' only
+    , isShorterThanRL
+    , snocRLSealed
     ) where
 
 import Prelude ()
 import Darcs.Prelude
 
-#include "impossible.h"
 import Darcs.Patch.Witnesses.Show
 import Darcs.Patch.Witnesses.Sealed
     ( FlippedSeal(..)
@@ -84,7 +90,7 @@ import Darcs.Patch.Witnesses.Sealed
     , Sealed2(..)
     , seal
     )
-import Darcs.Patch.Witnesses.Eq ( MyEq(..), EqCheck(..) )
+import Darcs.Patch.Witnesses.Eq ( Eq2(..), EqCheck(..) )
 
 -- * Directed Types
 
@@ -221,11 +227,11 @@ data (a1 :||: a2) wX wY = (a1 wX wY) :||: (a2 wX wY)
 instance (Show2 a, Show2 b) => Show ( (a :> b) wX wY ) where
     showsPrec d (x :> y) = showOp2 1 ":>" d x y
 
-instance (MyEq a, MyEq b) => MyEq (a :> b) where
+instance (Eq2 a, Eq2 b) => Eq2 (a :> b) where
     (a1 :> b1) =\/= (a2 :> b2) | IsEq <- a1 =\/= a2 = b1 =\/= b2
                                | otherwise = NotEq
 
-instance (MyEq a, MyEq b) => Eq ((a :> b) wX wY) where
+instance (Eq2 a, Eq2 b) => Eq ((a :> b) wX wY) where
     (==) = unsafeCompare
 
 instance (Show2 a, Show2 b) => Show2 (a :> b) where
@@ -262,7 +268,7 @@ nullRL _ = False
 filterOutFLFL :: (forall wX wY . p wX wY -> EqCheck wX wY) -> FL p wW wZ -> FL p wW wZ
 filterOutFLFL _ NilFL = NilFL
 filterOutFLFL f (x:>:xs) | IsEq <- f x = filterOutFLFL f xs
-                    | otherwise = x :>: filterOutFLFL f xs
+                         | otherwise = x :>: filterOutFLFL f xs
 
 filterOutRLRL :: (forall wX wY . p wX wY -> EqCheck wX wY) -> RL p wW wZ -> RL p wW wZ
 filterOutRLRL _ NilRL = NilRL
@@ -344,6 +350,14 @@ bunchFL n (x:>:xs) = (x :>: NilFL) :>: bFL xs
           bFL bs = case splitAtFL n bs of
                    a :> b -> a :>: bFL b
 
+-- | Monadic fold over an 'FL'
+-- associating to the left, i.e. from left to right.
+-- The order of arguments follows the standard 'foldM' from base.
+foldFL_M :: Monad m
+         => (forall wA wB. r wA -> p wA wB -> m (r wB))
+         -> r wX -> FL p wX wY -> m (r wY)
+foldFL_M _ r NilFL = return r
+foldFL_M f r (x :>: xs) = f r x >>= \r' -> foldFL_M f r' xs
 
 allFL :: (forall wX wY . a wX wY -> Bool) -> FL a wW wZ -> Bool
 allFL f xs = and $ mapFL f xs
@@ -351,6 +365,12 @@ allFL f xs = and $ mapFL f xs
 anyFL :: (forall wX wY . a wX wY -> Bool) -> FL a wW wZ -> Bool
 anyFL f xs = or $ mapFL f xs
 
+allRL :: (forall wA wB . a wA wB -> Bool) -> RL a wX wY -> Bool
+allRL f xs = and $ mapRL f xs
+
+anyRL :: (forall wA wB . a wA wB -> Bool) -> RL a wX wY -> Bool
+anyRL f xs = or $ mapRL f xs
+
 foldlFL :: (forall wW wY . a -> b wW wY -> a) -> a -> FL b wX wZ -> a
 foldlFL _ x NilFL = x
 foldlFL f x (y:>:ys) = foldlFL f (f x y) ys
@@ -369,6 +389,9 @@ mapFL_FL_M f (a:>:as) = do b <- f a
                            bs <- mapFL_FL_M f as
                            return (b:>:bs)
 
+sequenceFL_ :: Monad m => (forall wW wZ . a wW wZ -> m b) -> FL a wX wY -> m ()
+sequenceFL_ f = sequence_ . mapFL f
+
 zipWithFL :: (forall wX wY . a -> p wX wY -> q wX wY)
           -> [a] -> FL p wW wZ -> FL q wW wZ
 zipWithFL f (x:xs) (y :>: ys) = f x y :>: zipWithFL f xs ys
@@ -430,19 +453,38 @@ dropWhileRL p xs@(xs':<:x)
           | otherwise = seal xs
 
 -- |Check that two 'FL's are equal element by element.
--- This differs from the 'MyEq' instance for 'FL' which
+-- This differs from the 'Eq2' instance for 'FL' which
 -- uses commutation.
-eqFL :: MyEq a => FL a wX wY -> FL a wX wZ -> EqCheck wY wZ
+eqFL :: Eq2 a => FL a wX wY -> FL a wX wZ -> EqCheck wY wZ
 eqFL NilFL NilFL = IsEq
 eqFL (x:>:xs) (y:>:ys) | IsEq <- x =\/= y, IsEq <- eqFL xs ys = IsEq
 eqFL _ _ = NotEq
 
-eqFLRev :: MyEq a => FL a wX wZ -> FL a wY wZ -> EqCheck wX wY
+eqFLRev :: Eq2 a => FL a wX wZ -> FL a wY wZ -> EqCheck wX wY
 eqFLRev NilFL NilFL = IsEq
 eqFLRev (x:>:xs) (y:>:ys) | IsEq <- eqFLRev xs ys, IsEq <- x =/\= y = IsEq
 eqFLRev _ _ = NotEq
 
-eqFLUnsafe :: MyEq a => FL a wX wY -> FL a wZ wW -> Bool
+eqFLUnsafe :: Eq2 a => FL a wX wY -> FL a wZ wW -> Bool
 eqFLUnsafe NilFL NilFL = True
 eqFLUnsafe (x:>:xs) (y:>:ys) = unsafeCompare x y && eqFLUnsafe xs ys
 eqFLUnsafe _ _ = False
+
+infixr 5 +>>+
+infixl 5 +<<+
+
+-- | Prepend an 'RL' to an 'FL'. This traverses only the left hand side.
+(+>>+) :: RL p wX wY -> FL p wY wZ -> FL p wX wZ
+NilRL +>>+ ys = ys
+(xs:<:x) +>>+ ys = xs +>>+ (x :>: ys)
+
+-- | Append an 'FL' to an 'RL'. This traverses only the right hand side.
+(+<<+) :: RL p wX wY -> FL p wY wZ -> RL p wX wZ
+xs +<<+ NilFL = xs
+xs +<<+ (y:>:ys) = (xs:<:y) +<<+ ys
+
+initsFL :: FL p wX wY -> [Sealed ((p :> FL p) wX)]
+initsFL NilFL = []
+initsFL (x :>: xs) =
+    Sealed (x :> NilFL) :
+    map (\(Sealed (y :> xs')) -> Sealed (x :> y :>: xs')) (initsFL xs)
