@@ -1,4 +1,3 @@
-{-# LANGUAGE MagicHash #-}
 module Darcs.Patch.Witnesses.Unsafe
     ( unsafeCoerceP
     , unsafeCoercePStart
@@ -7,19 +6,19 @@ module Darcs.Patch.Witnesses.Unsafe
     , unsafeCoerceP1
     ) where
 
-import GHC.Base (unsafeCoerce#)
+import Unsafe.Coerce
 
 unsafeCoerceP :: a wX wY -> a wB wC
-unsafeCoerceP = unsafeCoerce#
+unsafeCoerceP = unsafeCoerce
 
 unsafeCoercePStart :: a wX1 wY -> a wX2 wY
-unsafeCoercePStart = unsafeCoerce#
+unsafeCoercePStart = unsafeCoerce
 
 unsafeCoercePEnd :: a wX wY1 -> a wX wY2
-unsafeCoercePEnd = unsafeCoerce#
+unsafeCoercePEnd = unsafeCoerce
 
 unsafeCoerceP2 :: t wW wX wY wZ -> t wA wB wC wD
-unsafeCoerceP2 = unsafeCoerce#
+unsafeCoerceP2 = unsafeCoerce
 
 unsafeCoerceP1 :: a wX -> a wY
-unsafeCoerceP1 = unsafeCoerce#
+unsafeCoerceP1 = unsafeCoerce
