@@ -20,13 +20,13 @@ If something is needed from the Prelude that's hidden by default, then add
 it to the Prelude import.
 -}
 
+{-# LANGUAGE CPP #-}
 module Darcs.Prelude
     ( module Prelude
     , module Control.Applicative
     , module Data.Monoid
-    , module Data.Semigroup
+    , Semigroup(..)
     , module Data.Traversable
-    , module Data.Maybe
     , impossible, bug
     ) where
 
@@ -42,9 +42,11 @@ import Prelude hiding
       -- because it's in the new Prelude but only in Data.Monoid in older GHCs
       Monoid(..)
     ,
+#if MIN_VERSION_base(4,11,0)
       -- because it's in the new Prelude but only in Data.Semigroup in older GHCs
       Semigroup(..)
     ,
+#endif
       -- because it's in the new Prelude but only in Data.Traversable in older GHCs
       traverse
     ,
@@ -62,7 +64,6 @@ import Control.Applicative ( Applicative(..), (<$>), (<*>) )
 import Data.Monoid ( Monoid(..) )
 import Data.Semigroup ( Semigroup(..) )
 import Data.Traversable ( traverse )
-import Data.Maybe ( fromJust )
 
 impossible :: a
 impossible = error "Impossible case"
