@@ -1,9 +1,6 @@
-{-# LANGUAGE CPP #-}
-
 module Darcs.Repository.Cache
     ( cacheHash
     , okayHash
-    , takeHash
     , Cache(..)
     , CacheType(..)
     , CacheLoc(..)
@@ -29,11 +26,10 @@ module Darcs.Repository.Cache
     , reportBadSources
     ) where
 
-import Control.Monad ( liftM, when, guard, unless, filterM, forM_, mplus )
-import qualified Data.ByteString as B (length, drop, ByteString )
-import qualified Data.ByteString.Char8 as BC (unpack)
+import Control.Monad ( liftM, when, unless, filterM, forM_, mplus )
+import qualified Data.ByteString as B (length, ByteString )
 import Data.List ( nub, intercalate )
-import Data.Maybe ( catMaybes, listToMaybe, fromMaybe )
+import Data.Maybe ( catMaybes, fromMaybe )
 import System.FilePath.Posix ( (</>), joinPath, dropFileName )
 import System.Directory ( createDirectoryIfMissing, removeFile, doesFileExist,
                           doesDirectoryExist, getDirectoryContents,
@@ -42,7 +38,7 @@ import qualified System.Directory as SD ( writable )
 import System.IO ( hPutStrLn, stderr )
 import System.Posix.Files ( createLink, linkCount, getSymbolicLinkStatus )
 
-import Darcs.Util.ByteString ( gzWriteFilePS, linesPS )
+import Darcs.Util.ByteString ( gzWriteFilePS )
 import Darcs.Util.Global ( darcsdir, addBadSource, isBadSource, addReachableSource,
                       isReachableSource, getBadSourcesList, defaultRemoteDarcsCmd )
 import Darcs.Util.External ( gzFetchFilePS, fetchFilePS
@@ -54,8 +50,7 @@ import Darcs.Util.Lock ( writeAtomicFilePS, gzWriteAtomicFilePS,
 import Darcs.Util.SignalHandler ( catchNonSignal )
 import Darcs.Util.URL ( isValidLocalPath, isHttpUrl, isSshUrl )
 import Darcs.Util.File ( withCurrentDirectory )
-import Darcs.Util.Crypt.SHA1 ( sha1PS )
-import Darcs.Util.Crypt.SHA256 ( sha256sum )
+import Darcs.Util.Hash ( sha256sum )
 import Darcs.Util.English ( englishNum, Noun(..), Pronoun(..) )
 import Darcs.Util.Exception ( catchall )
 import Darcs.Util.Progress ( progressList, debugMessage, debugFail )
@@ -178,18 +173,10 @@ cacheHash ps = if sizeStrLen > 10
     shaOfPs = sha256sum ps
 
 okayHash :: String -> Bool
-okayHash s = length s `elem` [40, 64, 75]
-
-takeHash :: B.ByteString -> Maybe (String, B.ByteString)
-takeHash ps = do
-    h <- listToMaybe $ linesPS ps
-    let v = BC.unpack h
-    guard $ okayHash v
-    return (v, B.drop (B.length h) ps)
+okayHash s = length s `elem` [64, 75]
 
 checkHash :: String -> B.ByteString -> Bool
 checkHash h s
-    | length h == 40 = (show $ sha1PS s) == h
     | length h == 64 = sha256sum s == h
     | length h == 75 =
         B.length s == read (take 10 h) && sha256sum s == drop 11 h
