@@ -1,53 +1,53 @@
-{-# LANGUAGE CPP, ScopedTypeVariables #-}
 module Darcs.Repository.Clone
-    ( createRepository
-    , cloneRepository
+    ( cloneRepository
     , replacePristine
     , writePatchSet
-    , patchSetToRepository
     ) where
 
 import Prelude ()
 import Darcs.Prelude
 
 import Control.Exception ( catch, SomeException )
-import Control.Monad ( when, void )
-import qualified Data.ByteString.Char8 as BS
+import Control.Monad ( when )
+import qualified Data.ByteString.Char8 as BC
 import Data.List( intercalate )
-import Data.Maybe( catMaybes, isJust )
+import Data.Maybe( catMaybes )
 import System.FilePath( (</>) )
 import System.Directory
-    ( createDirectory
-    , removeFile
+    ( removeFile
     , getDirectoryContents
-    , getCurrentDirectory
-    , setCurrentDirectory
     )
 import System.IO ( stderr )
-import System.IO.Error ( isAlreadyExistsError )
 
-import Darcs.Repository.State ( invalidateIndex, readWorking )
-
-import Darcs.Repository.Internal
-    ( Repository(..)
-    , IdentifyRepo(..)
+import Darcs.Repository.Create
+    ( EmptyRepository(..)
+    , createRepository
+    , writePristine
+    )
+import Darcs.Repository.State ( invalidateIndex )
+import Darcs.Repository.Pending ( tentativelyAddToPending )
+import Darcs.Repository.Identify
+    ( IdentifyRepo(..)
     , identifyRepositoryFor
-    , identifyRepository
-    , maybeIdentifyRepository
-    , readRepo
+    , maybeIdentifyRepository )
+import Darcs.Repository.Hashed
+    ( readRepo
     , tentativelyRemovePatches
-    , tentativelyAddToPending
     , finalizeRepositoryChanges
     , createPristineDirectoryTree
-    , setScriptsExecutable
-    , setScriptsExecutablePatches
-    , seekRepo
-    , repoPatchType
     , revertRepositoryChanges
     )
+import Darcs.Repository.Working
+    ( setScriptsExecutable
+    , setScriptsExecutablePatches )
 import Darcs.Repository.InternalTypes
-    ( modifyCache )
-import Darcs.Repository.Job ( RepoJob(..), withRepoLock, withRepository )
+    ( Repository
+    , repoLocation
+    , repoFormat
+    , repoCache
+    , modifyCache
+    , repoPatchType )
+import Darcs.Repository.Job ( withUMaskFlag )
 import Darcs.Repository.Cache
     ( unionRemoteCaches
     , unionCaches
@@ -60,22 +60,19 @@ import Darcs.Repository.Cache
     )
 import qualified Darcs.Repository.Cache as DarcsCache
 
-import qualified Darcs.Repository.HashedRepo as HashedRepo
-import Darcs.Repository.ApplyPatches ( applyPatches, runDefault )
-import Darcs.Repository.HashedRepo
-    ( applyToTentativePristine
-    , pris2inv
-    , inv2pris
+import qualified Darcs.Repository.Hashed as HashedRepo
+import Darcs.Repository.ApplyPatches ( runDefault )
+import Darcs.Repository.Hashed
+    ( applyToTentativePristineCwd
+    , peekPristineHash
     )
 import Darcs.Repository.Format
     ( RepoProperty ( HashedInventory, Darcs2 )
     , RepoFormat
-    , createRepoFormat
     , formatHas
-    , writeRepoFormat
     , readProblem
     )
-import Darcs.Repository.Prefs ( writeDefaultPrefs, addRepoSource, deleteSources )
+import Darcs.Repository.Prefs ( addRepoSource, deleteSources )
 import Darcs.Repository.Match ( getOnePatchset )
 import Darcs.Util.External
     ( copyFileOrUrl
@@ -83,8 +80,7 @@ import Darcs.Util.External
     , gzFetchFilePS
     )
 import Darcs.Repository.PatchIndex
-    ( createOrUpdatePatchIndexDisk
-    , doesPatchIndexExist
+    ( doesPatchIndexExist
     , createPIWithInterrupt
     )
 import Darcs.Repository.Packs
@@ -92,11 +88,7 @@ import Darcs.Repository.Packs
     , fetchAndUnpackPatches
     , packsDir
     )
-import Darcs.Util.Lock
-    ( writeBinFile
-    , writeDocBinFile
-    , appendBinFile
-    )
+import Darcs.Util.Lock ( appendTextFile, withNewDirectory )
 import Darcs.Repository.Flags
     ( UpdateWorking(..)
     , UseCache(..)
@@ -116,12 +108,12 @@ import Darcs.Repository.Flags
     , PatchFormat (..)
     )
 
-import Darcs.Patch ( RepoPatch, IsRepoType, apply, invert, effect, PrimOf )
+import Darcs.Patch ( RepoPatch, IsRepoType, apply, invert, effect )
 import Darcs.Patch.Depends ( findCommonWithThem, countUsThem )
 import Darcs.Patch.Set ( Origin
-                       , PatchSet(..)
-                       , newset2RL
-                       , newset2FL
+                       , PatchSet
+                       , patchSet2RL
+                       , patchSet2FL
                        , progressPatchSet
                        )
 import Darcs.Patch.Match ( MatchFlag(..), havePatchsetMatch )
@@ -140,19 +132,15 @@ import Darcs.Patch.Witnesses.Ordered
     )
 import Darcs.Patch.PatchInfoAnd ( PatchInfoAnd, extractHash, hopefully )
 
-import Darcs.Util.Hash( encodeBase16 )
 import Darcs.Util.Tree( Tree, emptyTree )
-import Darcs.Util.Tree.Hashed( writeDarcsHashed, darcsAddMissingHashes )
 
-import Darcs.Util.ByteString( gzReadFilePS )
 import Darcs.Util.Download ( maxPipelineLength )
 import Darcs.Util.Exception ( catchall )
-import Darcs.Util.File ( withCurrentDirectory )
 import Darcs.Util.English ( englishNum, Noun(..) )
 import Darcs.Util.Global ( darcsdir )
 import Darcs.Util.URL ( isValidLocalPath )
 import Darcs.Util.SignalHandler ( catchInterrupt )
-import Darcs.Util.Printer ( Doc, text, hPutDocLn, putDocLn, errorDoc, RenderMode(..) )
+import Darcs.Util.Printer ( Doc, text, hPutDocLn, putDocLn, errorDoc )
 import Darcs.Util.Progress
     ( debugMessage
     , tediousSize
@@ -160,31 +148,6 @@ import Darcs.Util.Progress
     , endTedious
     )
 
-#include "impossible.h"
-
-createRepository :: PatchFormat -> WithWorkingDir -> WithPatchIndex -> IO ()
-createRepository patchfmt withWorkingDir createPatchIndex = do
-  createDirectory darcsdir `catch`
-      (\e-> if isAlreadyExistsError e
-            then fail "Tree has already been initialized!"
-            else fail $ "Error creating directory `"++darcsdir++"'.")
-  cwd <- getCurrentDirectory
-  x <- seekRepo
-  when (isJust x) $ do
-      setCurrentDirectory cwd
-      putStrLn "WARNING: creating a nested repository."
-  createDirectory $ darcsdir </> "pristine.hashed"
-  createDirectory $ darcsdir </> "patches"
-  createDirectory $ darcsdir </> "inventories"
-  createDirectory $ darcsdir </> "prefs"
-  writeDefaultPrefs
-  writeRepoFormat (createRepoFormat patchfmt withWorkingDir) (darcsdir </> "format")
-  writeBinFile (darcsdir </> "hashed_inventory") ""
-  writePristine "." emptyTree
-  withRepository NoUseCache $ RepoJob $ \repo -> case createPatchIndex of
-      NoPatchIndex -> return () -- default
-      YesPatchIndex -> createOrUpdatePatchIndexDisk repo
-
 joinUrl :: [String] -> String
 joinUrl = intercalate "/"
 
@@ -203,41 +166,45 @@ cloneRepository ::
     -> Bool   -- use packs
     -> ForgetParent
     -> IO ()
-cloneRepository repodir mysimplename v uc cloneKind um rdarcs sse remoteRepos
-                setDefault matchFlags rfsource withWorkingDir usePatchIndex usePacks forget = do
-  createDirectory mysimplename
-  setCurrentDirectory mysimplename
-  createRepository (if formatHas Darcs2 rfsource then PatchFormat2 else PatchFormat1)
-                   withWorkingDir
-                   (if cloneKind == LazyClone then NoPatchIndex else usePatchIndex)
-  debugMessage "Finished initializing new repository."
-  addRepoSource repodir NoDryRun remoteRepos setDefault
+cloneRepository repodir mysimplename v useCache cloneKind um rdarcs sse remoteRepos
+                setDefault matchFlags rfsource withWorkingDir usePatchIndex usePacks
+                forget =
+  withUMaskFlag um $ withNewDirectory mysimplename $ do
+      let patchfmt = if formatHas Darcs2 rfsource then PatchFormat2 else PatchFormat1
+      EmptyRepository toRepo' <-
+        createRepository patchfmt withWorkingDir
+          (if cloneKind == LazyClone then NoPatchIndex else usePatchIndex) useCache
+      debugMessage "Finished initializing new repository."
+      addRepoSource repodir NoDryRun remoteRepos setDefault
 
-  debugMessage "Grabbing lock in new repository."
-  withRepoLock NoDryRun uc YesUpdateWorking um
-   $ RepoJob $ \repository -> do
       debugMessage "Identifying and copying repository..."
-      fromRepo@(Repo fromDir rffrom _ fromCache) <- identifyRepositoryFor repository uc repodir
+      fromRepo <- identifyRepositoryFor toRepo' useCache repodir
+      let fromLoc = repoLocation fromRepo
+      let rffrom = repoFormat fromRepo
       case readProblem rffrom of
-        Just e ->  fail $ "Incompatibility with repository " ++ fromDir ++ ":\n" ++ e
+        Just e ->  fail $ "Incompatibility with repository " ++ fromLoc ++ ":\n" ++ e
         Nothing -> return ()
-      debugMessage "Copying prefs"
-      copyFileOrUrl (remoteDarcs rdarcs) (joinUrl [fromDir, darcsdir, "prefs", "prefs"])
+      debugMessage "Copying prefs..."
+      copyFileOrUrl (remoteDarcs rdarcs)
+        (joinUrl [fromLoc, darcsdir, "prefs", "prefs"])
         (darcsdir </> "prefs/prefs") (MaxAge 600) `catchall` return ()
-      -- prepare sources and cache
-      (Repo toDir toFormat toPristine toCache) <- identifyRepository uc "."
-      toCache2 <- unionRemoteCaches toCache fromCache fromDir
-      toRepo <- copySources (Repo toDir toFormat toPristine toCache2) fromDir
+      debugMessage "Copying sources..."
+      cache <- unionRemoteCaches (repoCache toRepo') (repoCache fromRepo) fromLoc
+      appendTextFile (darcsdir </> "prefs/sources")
+                     (show $ repo2cache fromLoc `unionCaches` dropNonRepos cache)
+      debugMessage "Done copying and filtering sources."
+      -- put remote source last
+      let toRepo = modifyCache toRepo' (const $ cache `unionCaches` repo2cache fromLoc)
       if formatHas HashedInventory rffrom then do
        -- copying basic repository (hashed_inventory and pristine)
-       if usePacks && (not . isValidLocalPath) fromDir
+       if usePacks && (not . isValidLocalPath) fromLoc
          then copyBasicRepoPacked    fromRepo toRepo v rdarcs withWorkingDir
          else copyBasicRepoNotPacked fromRepo toRepo v rdarcs withWorkingDir
        when (cloneKind /= LazyClone) $ do
          when (cloneKind /= CompleteClone) $
            putInfo v $ text "Copying patches, to get lazy repository hit ctrl-C..."
        -- copying complete repository (inventories and patches)
-         if usePacks && (not . isValidLocalPath) fromDir
+         if usePacks && (not . isValidLocalPath) fromLoc
            then copyCompleteRepoPacked    fromRepo toRepo v cloneKind
            else copyCompleteRepoNotPacked fromRepo toRepo v cloneKind
       else
@@ -245,8 +212,8 @@ cloneRepository repodir mysimplename v uc cloneKind um rdarcs sse remoteRepos
        -- we need to copy all patches first and then build pristine
        copyRepoOldFashioned fromRepo toRepo v withWorkingDir
       when (sse == YesSetScriptsExecutable) setScriptsExecutable
-      when (havePatchsetMatch (repoPatchType repository) matchFlags) $ do
-        putStrLn "Going to specified version..."
+      when (havePatchsetMatch (repoPatchType toRepo) matchFlags) $ do
+        putInfo v $ text "Going to specified version..."
         -- the following is necessary to be able to read repo's patches
         revertRepositoryChanges toRepo YesUpdateWorking
         patches <- readRepo toRepo
@@ -266,24 +233,33 @@ cloneRepository repodir mysimplename v uc cloneKind um rdarcs sse remoteRepos
         when (sse == YesSetScriptsExecutable) $ setScriptsExecutablePatches (invert $ effect ps)
       when (forget == YesForgetParent) deleteSources
 
+-- | This keeps only NonWritable Repo entries.
+dropNonRepos :: Cache -> Cache
+dropNonRepos (Ca cache) = Ca $ filter notRepo cache where
+  notRepo xs = case xs of
+    Cache DarcsCache.Directory _ _ -> False
+    -- we don't want to write thisrepo: entries to the disk
+    Cache DarcsCache.Repo DarcsCache.Writable _ -> False
+    _ -> True
+
 putInfo :: Verbosity -> Doc -> IO ()
 putInfo Quiet _ = return ()
-putInfo _ d = hPutDocLn Encode stderr d
+putInfo _ d = hPutDocLn stderr d
 
 putVerbose :: Verbosity -> Doc -> IO ()
 putVerbose Verbose d = putDocLn d
 putVerbose _ _ = return ()
 
-copyBasicRepoNotPacked  :: forall rt p wR wU wT. (RepoPatch p, ApplyState p ~ Tree)
-                        => Repository rt p wR wU wT -- remote
+copyBasicRepoNotPacked  :: forall rt p wR wU wT.
+                           Repository rt p wR wU wT -- remote
                         -> Repository rt p wR wU wT -- existing empty local
                         -> Verbosity
                         -> RemoteDarcs
                         -> WithWorkingDir
                         -> IO ()
-copyBasicRepoNotPacked (Repo fromDir _ _ _) toRepo verb rdarcs withWorkingDir = do
+copyBasicRepoNotPacked fromRepo toRepo verb rdarcs withWorkingDir = do
   putVerbose verb $ text "Copying hashed inventory from remote repo..."
-  HashedRepo.copyHashedInventory toRepo rdarcs fromDir
+  HashedRepo.copyHashedInventory toRepo rdarcs (repoLocation fromRepo)
   putVerbose verb $ text "Writing pristine and working directory contents..."
   createPristineDirectoryTree toRepo "." withWorkingDir
 
@@ -293,31 +269,33 @@ copyCompleteRepoNotPacked :: forall rt p wR wU wT. (IsRepoType rt, RepoPatch p,
                         -> Verbosity
                         -> CloneKind
                         -> IO ()
-copyCompleteRepoNotPacked _ torepository@(Repo todir _ _ _) verb cloneKind = do
+copyCompleteRepoNotPacked _ toRepo verb cloneKind = do
        let cleanup = putInfo verb $ text "Using lazy repository."
        allowCtrlC cloneKind cleanup $ do
-         fetchPatchesIfNecessary torepository
-         pi <- doesPatchIndexExist todir
-         when pi $ createPIWithInterrupt torepository
+         fetchPatchesIfNecessary toRepo
+         pi <- doesPatchIndexExist (repoLocation toRepo)
+         ps <- readRepo toRepo
+         when pi $ createPIWithInterrupt toRepo ps
 
 copyBasicRepoPacked ::
-  forall rt p wR wU wT. (IsRepoType rt, RepoPatch p, ApplyState (PrimOf p) ~ Tree, ApplyState p ~ Tree)
-  => Repository rt p wR wU wT -- remote
+  forall rt p wR wU wT.
+     Repository rt p wR wU wT -- remote
   -> Repository rt p wR wU wT -- existing empty local repository
   -> Verbosity
   -> RemoteDarcs
   -> WithWorkingDir
   -> IO ()
-copyBasicRepoPacked r@(Repo fromDir _ _ _) toRepo verb rdarcs withWorkingDir =
-  do let hashURL = joinUrl [fromDir, darcsdir, packsDir, "pristine"]
+copyBasicRepoPacked fromRepo toRepo verb rdarcs withWorkingDir =
+  do let fromLoc = repoLocation fromRepo
+     let hashURL = joinUrl [fromLoc, darcsdir, packsDir, "pristine"]
      mPackHash <- (Just <$> gzFetchFilePS hashURL Uncachable) `catchall` (return Nothing)
-     let hiURL = joinUrl [fromDir, darcsdir, "hashed_inventory"]
+     let hiURL = joinUrl [fromLoc, darcsdir, "hashed_inventory"]
      i <- gzFetchFilePS hiURL Uncachable
-     let currentHash = BS.pack $ inv2pris i
-     let copyNormally = copyBasicRepoNotPacked r toRepo verb rdarcs withWorkingDir
+     let currentHash = BC.pack $ peekPristineHash i
+     let copyNormally = copyBasicRepoNotPacked fromRepo toRepo verb rdarcs withWorkingDir
      case mPackHash of
       Just packHash | packHash == currentHash
-              -> ( copyBasicRepoPacked2 r toRepo verb withWorkingDir
+              -> ( copyBasicRepoPacked2 fromRepo toRepo verb withWorkingDir
                     `catch` \(e :: SomeException) ->
                                do putStrLn ("Exception while getting basic pack:\n" ++ show e)
                                   copyNormally)
@@ -325,53 +303,53 @@ copyBasicRepoPacked r@(Repo fromDir _ _ _) toRepo verb rdarcs withWorkingDir =
                     copyNormally
 
 copyBasicRepoPacked2 ::
-  forall rt p wR wU wT. (IsRepoType rt, RepoPatch p, ApplyState (PrimOf p) ~ Tree, ApplyState p ~ Tree)
-  => Repository rt p wR wU wT -- remote
+  forall rt p wR wU wT.
+     Repository rt p wR wU wT -- remote
   -> Repository rt p wR wU wT -- existing empty local repository
   -> Verbosity
   -> WithWorkingDir
   -> IO ()
-copyBasicRepoPacked2 (Repo fromDir _ _ _) toRepo@(Repo _ _ _ toCache) verb withWorkingDir = do
+copyBasicRepoPacked2 fromRepo toRepo verb withWorkingDir = do
   putVerbose verb $ text "Cloning packed basic repository."
   -- unpack inventory & pristine cache
   cleanDir $ darcsdir </> "pristine.hashed"
   removeFile $ darcsdir </> "hashed_inventory"
-  fetchAndUnpackBasic toCache fromDir
+  fetchAndUnpackBasic (repoCache toRepo) (repoLocation fromRepo)
   putInfo verb $ text "Done fetching and unpacking basic pack."
   createPristineDirectoryTree toRepo "." withWorkingDir
 
 copyCompleteRepoPacked ::
-  forall rt p wR wU wT. (IsRepoType rt, RepoPatch p, ApplyState (PrimOf p) ~ Tree, ApplyState p ~ Tree)
+  forall rt p wR wU wT. (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree)
   => Repository rt p wR wU wT -- remote
   -> Repository rt p wR wU wT -- existing basic local repository
   -> Verbosity
   -> CloneKind
   -> IO ()
-copyCompleteRepoPacked r to verb cloneKind =
-  ( copyCompleteRepoPacked2 r to verb cloneKind
-   `catch` \(e :: SomeException) ->
-             do putStrLn ("Exception while getting patches pack:\n" ++ show e)
-                putVerbose verb $ text "Problem while copying patches pack, copying normally."
-                copyCompleteRepoNotPacked r to verb cloneKind )
+copyCompleteRepoPacked from to verb cloneKind =
+    copyCompleteRepoPacked2 from to verb cloneKind
+  `catch`
+    \(e :: SomeException) -> do
+      putStrLn ("Exception while getting patches pack:\n" ++ show e)
+      putVerbose verb $ text "Problem while copying patches pack, copying normally."
+      copyCompleteRepoNotPacked from to verb cloneKind
 
 copyCompleteRepoPacked2 ::
-  forall rt p wR wU wT. (IsRepoType rt, RepoPatch p, ApplyState (PrimOf p) ~ Tree, ApplyState p ~ Tree)
+  forall rt p wR wU wT. (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree)
   => Repository rt p wR wU wT
   -> Repository rt p wR wU wT
   -> Verbosity
   -> CloneKind
   -> IO ()
-copyCompleteRepoPacked2 (Repo fromDir _ _ _)
-                        toRepo@(Repo toDir _ _ toCache)
-                        verb cloneKind = do
+copyCompleteRepoPacked2 fromRepo toRepo verb cloneKind = do
   us <- readRepo toRepo
   -- get old patches
   let cleanup = putInfo verb $ text "Using lazy repository."
   allowCtrlC cloneKind cleanup $ do
     putVerbose verb $ text "Using patches pack."
-    fetchAndUnpackPatches (mapRL hashedPatchFileName $ newset2RL us) toCache fromDir
-    pi <- doesPatchIndexExist toDir
-    when pi $ createPIWithInterrupt toRepo
+    fetchAndUnpackPatches (mapRL hashedPatchFileName $ patchSet2RL us)
+      (repoCache toRepo) (repoLocation fromRepo)
+    pi <- doesPatchIndexExist (repoLocation toRepo)
+    when pi $ createPIWithInterrupt toRepo us -- TODO or do another readRepo?
 
 cleanDir :: FilePath -> IO ()
 cleanDir d = mapM_ (\x -> removeFile $ d </> x) .
@@ -383,35 +361,35 @@ copyRepoOldFashioned :: forall rt p wR wU wT. (IsRepoType rt, RepoPatch p, Apply
                         -> Verbosity
                         -> WithWorkingDir
                         -> IO ()
-copyRepoOldFashioned fromrepository toRepo@(Repo _ _ _ toCache) verb withWorkingDir = do
+copyRepoOldFashioned fromrepository toRepo verb withWorkingDir = do
   HashedRepo.revertTentativeChanges
   patches <- readRepo fromrepository
   let k = "Copying patch"
   beginTedious k
-  tediousSize k (lengthRL $ newset2RL patches)
+  tediousSize k (lengthRL $ patchSet2RL patches)
   let patches' = progressPatchSet k patches
-  HashedRepo.writeTentativeInventory toCache GzipCompression patches'
+  HashedRepo.writeTentativeInventory (repoCache toRepo) GzipCompression patches'
   endTedious k
   HashedRepo.finalizeTentativeChanges toRepo GzipCompression
   -- apply all patches into current hashed repository
   HashedRepo.revertTentativeChanges
   local_patches <- readRepo toRepo
   replacePristine toRepo emptyTree
-  let patchesToApply = progressFL "Applying patch" $ newset2FL local_patches
-  sequence_ $ mapFL applyToTentativePristine $ bunchFL 100 patchesToApply
+  let patchesToApply = progressFL "Applying patch" $ patchSet2FL local_patches
+  sequence_ $ mapFL applyToTentativePristineCwd $ bunchFL 100 patchesToApply
   finalizeRepositoryChanges toRepo YesUpdateWorking GzipCompression
   putVerbose verb $ text "Writing pristine and working directory contents..."
   createPristineDirectoryTree toRepo "." withWorkingDir
 
 -- | This function fetches all patches that the given repository has
 --   with fetchFileUsingCache, unless --lazy is passed.
-fetchPatchesIfNecessary :: forall rt p wR wU wT. (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree)
+fetchPatchesIfNecessary :: forall rt p wR wU wT. (IsRepoType rt, RepoPatch p)
                         => Repository rt p wR wU wT
                         -> IO ()
-fetchPatchesIfNecessary torepository@(Repo _ _ _ c) =
-  do  r <- readRepo torepository
+fetchPatchesIfNecessary toRepo =
+  do  ps <- readRepo toRepo
       pipelineLength <- maxPipelineLength
-      let patches = newset2RL r
+      let patches = patchSet2RL ps
           ppatches = progressRLShowTags "Copying patches" patches
           (first, other) = splitAt (pipelineLength - 1) $ tail $ hashes patches
           speculate | pipelineLength > 1 = [] : first : map (:[]) other
@@ -423,56 +401,54 @@ fetchPatchesIfNecessary torepository@(Repo _ _ _ c) =
         fetchAndSpeculate (f, ss) = do
           _ <- fetchFileUsingCache c HashedPatchesDir f
           mapM_ (speculateFileUsingCache c HashedPatchesDir) ss
+        c = repoCache toRepo
 
+{-
 -- | patchSetToRepository takes a patch set, and writes a new repository
 --   in the current directory that contains all the patches in the patch
 --   set.  This function is used when 'darcs get'ing a repository with
 --   the --to-match flag.
+-- bf: no it is not used anywhere
 patchSetToRepository :: (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree)
                      => Repository rt p wR1 wU1 wR1
                      -> PatchSet rt p Origin wX
                      -> UseCache -> RemoteDarcs
                      -> IO ()
-patchSetToRepository (Repo fromrepo rf _ _) patchset useCache rDarcs = do
-    when (formatHas HashedInventory rf) $ -- set up sources and all that
+patchSetToRepository fromRepo patchset useCache rDarcs = do
+    when (formatHas HashedInventory (repoFormat fromRepo)) $
+       -- set up sources and all that
        do writeFile (darcsdir </> "tentative_pristine") "" -- this is hokey
           repox <- writePatchSet patchset useCache
-          HashedRepo.copyHashedInventory repox rDarcs fromrepo
-          void $ copySources repox fromrepo
-    repo@(Repo dir _ _ _) <- writePatchSet patchset useCache
-    readRepo repo >>= (runDefault . applyPatches . newset2FL)
+          let fromLoc = repoLocation fromRepo
+          HashedRepo.copyHashedInventory repox rDarcs fromLoc
+          void $ copySources repox fromLoc
+    repo <- writePatchSet patchset useCache
+    readRepo repo >>= (runDefault . applyPatches . patchSet2FL)
     debugMessage "Writing the pristine"
-    withCurrentDirectory dir $ readWorking >>= replacePristine repo
+    withRepoLocation repo $ readWorking >>= replacePristine repo
+-}
 
 -- | writePatchSet is like patchSetToRepository, except that it doesn't
 -- touch the working directory or pristine cache.
-writePatchSet :: (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree)
+writePatchSet :: (IsRepoType rt, RepoPatch p)
               => PatchSet rt p Origin wX
               -> UseCache
               -> IO (Repository rt p wR wU wT)
 writePatchSet patchset useCache = do
     maybeRepo <- maybeIdentifyRepository useCache "."
-    let repo@(Repo _ _ _ c) =
+    let repo =
           case maybeRepo of
             GoodRepository r -> r
             BadRepository e -> bug ("Current directory is a bad repository in writePatchSet: " ++ e)
             NonRepository e -> bug ("Current directory not a repository in writePatchSet: " ++ e)
     debugMessage "Writing inventory"
-    HashedRepo.writeTentativeInventory c GzipCompression patchset
+    HashedRepo.writeTentativeInventory (repoCache repo) GzipCompression patchset
     HashedRepo.finalizeTentativeChanges repo GzipCompression
     return repo
 
 -- | Replace the existing pristine with a new one (loaded up in a Tree object).
 replacePristine :: Repository rt p wR wU wT -> Tree IO -> IO ()
-replacePristine (Repo r _ _ _) = writePristine r
-
-writePristine :: FilePath -> Tree IO -> IO ()
-writePristine r tree = withCurrentDirectory r $
-    do let t = darcsdir </> "hashed_inventory"
-       i <- gzReadFilePS t
-       tree' <- darcsAddMissingHashes tree
-       root <- writeDarcsHashed tree' $ darcsdir </> "pristine.hashed"
-       writeDocBinFile t $ pris2inv (BS.unpack $ encodeBase16 root) i
+replacePristine = writePristine . repoLocation
 
 allowCtrlC :: CloneKind -> IO () -> IO () -> IO ()
 allowCtrlC CompleteClone _       action = action
@@ -482,30 +458,3 @@ hashedPatchFileName :: PatchInfoAnd rt p wA wB -> String
 hashedPatchFileName x = case extractHash x of
   Left _ -> fail "unexpected unhashed patch"
   Right h -> h
-
--- |'copySources' does two things:
--- * it copies the prefs/sources file to the local repo, from the
---   remote, having first filtered the local filesystem sources.
--- * it returns the original list of sources of the local repo
---   updated with the remote repo as an additional source
-copySources :: RepoPatch p
-            => Repository rt p wR wU wT
-            -> String
-            -> IO (Repository rt p wR wU wT)
-copySources repo@(Repo outr _ _ cache0) inr = do
-    let (Repo s f p newCache1) = modifyCache repo dropNonRepos
-    let sourcesToWrite = repo2cache inr `unionCaches` newCache1
-    appendBinFile (outr ++ "/" ++ darcsdir ++ "/prefs/sources")
-                  (show sourcesToWrite)
-    debugMessage "Done copying and filtering pref/sources."
-
-    -- put remote source last:
-    let newSources = cache0 `unionCaches` repo2cache inr
-    return (Repo s f p newSources)
-  where
-    dropNonRepos (Ca cache) = Ca $ filter notRepo cache
-    notRepo xs = case xs of
-        Cache DarcsCache.Directory _                   _ -> False
-        -- we don't want to write thisrepo: entries to the disk
-        Cache DarcsCache.Repo      DarcsCache.Writable _ -> False
-        _                              -> True
