@@ -30,6 +30,8 @@ module Darcs.Repository.Flags
     , ForgetParent (..)
     , PatchFormat (..)
     , IncludeBoring (..)
+    , HooksConfig (..)
+    , HookConfig (..)
     ) where
 
 import Darcs.Util.Diff ( DiffAlgorithm(..) )
@@ -129,3 +131,13 @@ data ForgetParent = YesForgetParent | NoForgetParent
 
 data PatchFormat = PatchFormat1 | PatchFormat2
     deriving ( Eq, Show )
+
+data HooksConfig = HooksConfig
+  { pre :: HookConfig
+  , post :: HookConfig
+  }
+
+data HookConfig = HookConfig
+  { cmd :: Maybe String
+  , prompt :: Bool
+  }
