@@ -63,7 +63,7 @@ instance Show RepoProperty where
     show HashedInventory = hashedInventoryFormat
     show NoWorkingDir = noWorkingDirFormat
     show RebaseInProgress = rebaseInProgressFormat
-    show (UnknownFormat f) = "Unknown format: " ++ f
+    show (UnknownFormat f) = f
 
 readRepoProperty :: String -> RepoProperty
 readRepoProperty input
@@ -167,7 +167,7 @@ writeProblem target = readProblem target `mplus` findProblems target wp
     wp x = case partition isKnown x of
                (_, []) -> Nothing
                (_, unknowns) -> Just . unwords $
-                    "Can't write repository format: " : map show unknowns
+                    "Can't write repository: unknown formats:" : map show unknowns
 
 -- | @'transferProblem' source target@ returns 'Just' an error message if we
 -- cannot transfer patches from a repo in format @source@ to a repo in format
@@ -192,7 +192,7 @@ readProblem source = findProblems source rp
   where
     rp x | any isKnown x = Nothing
     rp [] = impossible
-    rp x = Just . unwords $ "Can't understand repository format:" : map show x
+    rp x = Just . unwords $ "Can't read repository: unknown formats:" : map show x
 
 -- |'findProblems' applies a function that maps format-entries to an optional
 -- error message, to each repoformat entry. Returning any errors.
