@@ -37,11 +37,11 @@ module Darcs.Repository.PatchIndex (
 import Prelude ()
 import Darcs.Prelude
 
-import Data.Binary ( encodeFile, decodeFile )
+import Data.Binary ( Binary, encodeFile, decodeFileOrFail )
 import Data.Word ( Word32 )
 import Data.Int ( Int8 )
 import Data.List ( group, mapAccumL, sort, isPrefixOf, nub, (\\) )
-import Data.Maybe ( fromMaybe, isJust )
+import Data.Maybe ( fromJust, fromMaybe, isJust )
 import Data.Set (Set)
 import Data.Map (Map)
 import qualified Data.Map as M
@@ -49,7 +49,6 @@ import qualified Data.Set as S
 import Control.Exception ( catch )
 import Control.Monad ( forM_, unless, when )
 import Control.Monad.State.Strict ( evalState, execState, State, gets, modify )
-import Control.Applicative ( (<$>) )
 import System.Directory ( createDirectory, renameDirectory, doesFileExist, doesDirectoryExist )
 import Darcs.Repository.Format ( formatHas, RepoProperty( HashedInventory ) )
 import Darcs.Repository.InternalTypes ( Repository, repoLocation, repoFormat )
@@ -512,6 +511,16 @@ storePatchIndex repodir cdir inv_hash (PatchIndex pids fidspans fpspans infom) =
   rmRecursive cdir `catch` \(_ :: IOError) -> return ()
   renameDirectory tmpdir cdir
 
+decodeFile :: Binary a => FilePath -> IO a
+decodeFile path = do
+  result <- decodeFileOrFail path
+  case result of
+    Left (offset, msg) ->
+      fail $
+        "Patch index is corrupt (file "++path++" at offset "++show offset++"): "++msg++
+        "\nPlease remove the corrupt file and then try again."
+    Right r -> return r
+
 storeRepoState :: FilePath -> String -> IO ()
 storeRepoState fp inv_hash = encodeFile fp (version,inv_hash)
 
