@@ -36,7 +36,7 @@ import Control.Applicative
 import qualified Data.ByteString as B ( empty )
 
 import Control.Exception ( catch, IOException )
-import Data.Maybe ( fromMaybe )
+import Data.Maybe ( fromJust, fromMaybe )
 
 import Darcs.Util.Printer ( errorDoc )
 import Darcs.Util.Global ( darcsdir )
@@ -82,7 +82,7 @@ import Darcs.Patch.Witnesses.Ordered
     , lengthFL, allFL, filterOutFLFL
     , reverseFL, mapFL )
 import Darcs.Util.ByteString ( gzReadFilePS )
-import Darcs.Util.Printer ( Doc, ($$), (<>), text, vcat, (<+>) )
+import Darcs.Util.Printer ( Doc, ($$), text, vcat, (<+>) )
 import Darcs.Util.Progress ( debugMessage )
 
 pendingName :: String
