@@ -44,13 +44,18 @@ import Darcs.Prelude
 import Control.Monad ( when, foldM, forM )
 import Control.Monad.State ( StateT, runStateT, get, put, liftIO )
 import Control.Exception ( catch, IOException )
-import Data.Maybe ( isJust )
+import Data.Maybe ( fromJust, isJust )
 import Data.Ord ( comparing )
 import Data.List ( sortBy, union, delete )
 import Text.Regex( matchRegex )
 
 import System.Directory( removeFile, doesFileExist, doesDirectoryExist, renameFile )
-import System.FilePath ( (</>), (<.>) )
+import System.FilePath
+    ( (</>)
+#if mingw32_HOST_OS
+    , (<.>)
+#endif
+    )
 import qualified Data.ByteString as B
     ( ByteString, readFile, drop, writeFile, empty, concat )
 import qualified Data.ByteString.Char8 as BC
