@@ -35,29 +35,23 @@ import Darcs.Util.File ( withCurrentDirectory )
 import Darcs.Util.Path ( AbsolutePath )
 import Darcs.Util.Prompt ( askUser )
 import Darcs.Repository.Prefs ( getPrefval )
-import Darcs.Patch
-    ( RepoPatch )
 
-import Darcs.Repository.Internal
-    ( setScriptsExecutable
-    , withTentative
-    )
+import Darcs.Repository.Hashed ( withTentative )
+import Darcs.Repository.Working ( setScriptsExecutable )
 import Darcs.Repository.Flags
     ( LeaveTestDir(..)
     , Verbosity(..)
     , SetScriptsExecutable(..)
     , RunTest (..)
+    , HookConfig (..)
     )
 import Darcs.Repository.InternalTypes
-    ( Repository(..) )
+    ( Repository, repoLocation )
 import Darcs.Util.Progress ( debugMessage )
 import Darcs.Util.Lock
     ( withTempDir
     , withPermDir
     )
-import Darcs.Patch.Apply ( ApplyState )
-import Darcs.Util.Tree ( Tree )
-
 
 getTest :: Verbosity -> IO (IO ExitCode)
 getTest verb =
@@ -71,8 +65,8 @@ getTest verb =
      putInfo "Running test...\n"
      runTest testcode putInfo
 
-runPosthook :: Maybe String -> Bool -> Verbosity -> AbsolutePath -> IO ExitCode
-runPosthook mPostHook askPostHook verb repodir
+runPosthook :: HookConfig -> Verbosity -> AbsolutePath -> IO ExitCode
+runPosthook (HookConfig mPostHook askPostHook) verb repodir
     = do ph <- getPosthook mPostHook askPostHook
          withCurrentDirectory repodir $ runHook verb "Posthook" ph
 
@@ -91,8 +85,8 @@ getPosthook mPostHookCmd askPostHook =
                 _ -> putStrLn "Posthook cancelled..." >> return Nothing
       else return $ Just command
 
-runPrehook :: Maybe String -> Bool -> Verbosity -> AbsolutePath -> IO ExitCode
-runPrehook mPreHookCmd askPreHook verb repodir =
+runPrehook :: HookConfig -> Verbosity -> AbsolutePath -> IO ExitCode
+runPrehook (HookConfig mPreHookCmd askPreHook) verb repodir =
     do ph <- getPrehook mPreHookCmd askPreHook
        withCurrentDirectory repodir $ runHook verb "Prehook" ph
 
@@ -121,8 +115,7 @@ runHook verb cname (Just command) =
          else hPutStrLn stderr $ cname++" failed!"
        return ec
 
-testTentative :: (RepoPatch p, ApplyState p ~ Tree)
-              => Repository rt p wR wU wT
+testTentative :: Repository rt p wR wU wT
               -> RunTest
               -> LeaveTestDir
               -> SetScriptsExecutable
@@ -139,8 +132,7 @@ runTest testcode putInfo = do
     return ec
 
 
-testAny :: RepoPatch p
-        => (Repository rt p wR wU wT
+testAny :: (Repository rt p wR wU wT
             -> ((AbsolutePath -> IO ExitCode) -> IO ExitCode)
             -> (AbsolutePath -> IO ExitCode) -> IO ExitCode
            )
@@ -150,10 +142,12 @@ testAny :: RepoPatch p
         -> SetScriptsExecutable
         -> Verbosity
         -> IO ExitCode
-testAny withD repository@(Repo dir _ _ _) doRunTest ltd sse verb =
+testAny withD repository doRunTest ltd sse verb =
     debugMessage "Considering whether to test..." >>
-    if doRunTest == NoRunTest then return ExitSuccess else withCurrentDirectory dir $
-    do let putInfo = if verb == Quiet then const (return ()) else putStrLn
+    if doRunTest == NoRunTest
+      then return ExitSuccess
+      else withCurrentDirectory (repoLocation repository) $ do
+       let putInfo = if verb == Quiet then const (return ()) else putStrLn
        debugMessage "About to run test if it exists."
        testline <- getPrefval "test"
        case testline of
