@@ -39,7 +39,7 @@ import Control.Exception ( finally )
 import Control.Applicative ( (<|>) )
 
 import System.Time ( toClockTime )
-import Data.Maybe ( catMaybes, fromMaybe )
+import Data.Maybe ( catMaybes, fromJust, fromMaybe )
 import qualified Data.IntMap as M
 
 import qualified Data.ByteString as B
