@@ -23,18 +23,15 @@ import Darcs.Prelude
 import Prelude hiding ( (^) )
 import Darcs.UI.Commands
     ( DarcsCommand(..), withStdOpts, nodefaults, amNotInRepository, putInfo )
-import Darcs.UI.Flags ( DarcsFlag( WorkRepoDir ), withWorkingDir, patchFormat, runPatchIndex )
-import Darcs.UI.Options ( DarcsOption, (^), odesc, ocheck, onormalise, defaultFlags )
+import Darcs.UI.Completion ( noArgs )
+import Darcs.UI.Flags ( DarcsFlag( WorkRepoDir ) )
+import Darcs.UI.Options ( (^), odesc, ocheck, onormalise, defaultFlags, (?) )
 import qualified Darcs.UI.Options.All as O
-    ( patchFormat, useWorkingDir, workingRepoDir, patchIndex, hashed
-    , PatchFormat, WithWorkingDir, WithPatchIndex
-    , StdCmdAction, Verbosity, UseCache, PatchFormat(..)
-    )
 import Darcs.UI.Options.All (  )
 import Darcs.Util.Printer ( text )
 import Darcs.Util.Path ( AbsolutePath )
 import Darcs.Util.Text ( quote )
-import Darcs.Repository ( createRepository )
+import Darcs.Repository ( createRepository, withUMaskFlag )
 
 initializeDescription :: String
 initializeDescription = "Create an empty repository."
@@ -55,31 +52,6 @@ initializeHelp =
  "\n" ++
  "Initialize is commonly abbreviated to `init`.\n"
 
-initBasicOpts :: DarcsOption a (O.PatchFormat -> O.WithWorkingDir -> Maybe String -> a)
-initBasicOpts = O.patchFormat ^ O.useWorkingDir ^ O.workingRepoDir
-
-initAdvancedOpts :: DarcsOption a (O.WithPatchIndex -> () -> a)
-initAdvancedOpts = O.patchIndex ^ O.hashed
-
-initOpts :: DarcsOption a
-            (O.PatchFormat
-             -> O.WithWorkingDir
-             -> Maybe String
-             -> Maybe O.StdCmdAction
-             -> Bool
-             -> Bool
-             -> O.Verbosity
-             -> Bool
-             -> O.WithPatchIndex
-             -> ()
-             -> O.UseCache
-             -> Maybe String
-             -> Bool
-             -> Maybe String
-             -> Bool
-             -> a)
-initOpts = initBasicOpts `withStdOpts` initAdvancedOpts
-
 initialize :: DarcsCommand [DarcsFlag]
 initialize = DarcsCommand
     { commandProgramName = "darcs"
@@ -90,7 +62,7 @@ initialize = DarcsCommand
     , commandExtraArgHelp = ["[<DIRECTORY>]"]
     , commandPrereq = \_ -> return $ Right ()
     , commandCommand = initializeCmd
-    , commandGetArgPossibilities = return []
+    , commandCompleteArgs = noArgs
     , commandArgdefaults = nodefaults
     , commandAdvancedOptions = odesc initAdvancedOpts
     , commandBasicOptions = odesc initBasicOpts
@@ -98,16 +70,25 @@ initialize = DarcsCommand
     , commandCheckOptions = ocheck initOpts
     , commandParseOptions = onormalise initOpts
     }
+  where
+    initBasicOpts = O.patchFormat ^ O.withWorkingDir ^ O.repoDir
+    initAdvancedOpts = O.patchIndexNo ^ O.hashed
+    initOpts = initBasicOpts `withStdOpts` initAdvancedOpts
 
 initializeCmd :: (AbsolutePath, AbsolutePath) -> [DarcsFlag] -> [String] -> IO ()
 initializeCmd aps opts [outname] | null [ () | WorkRepoDir _ <- opts ] =
   initializeCmd aps (WorkRepoDir outname:opts) []
-initializeCmd _ opts [] = do
-  location <- amNotInRepository opts
-  case location of
-    Left msg -> fail $ "Unable to " ++ quote ("darcs " ++ commandName initialize)
-                        ++ " here.\n\n" ++ msg
-    Right () -> do
-      createRepository (patchFormat opts) (withWorkingDir opts) (runPatchIndex opts)
-      putInfo opts $ text "Repository initialized."
+initializeCmd _ opts [] =
+  withUMaskFlag (O.umask ? opts) $ do
+    location <- amNotInRepository opts
+    case location of
+      Left msg -> fail $ "Unable to " ++ quote ("darcs " ++ commandName initialize)
+                          ++ " here.\n\n" ++ msg
+      Right () -> do
+        _ <- createRepository
+          (O.patchFormat ? opts)
+          (O.withWorkingDir ? opts)
+          (O.patchIndexNo ? opts)
+          (O.useCache ? opts)
+        putInfo opts $ text "Repository initialized."
 initializeCmd _ _ _ = fail "You must provide 'initialize' with either zero or one argument."
