@@ -84,7 +84,7 @@ import Darcs.Patch.Match
     )
 import Darcs.Patch.Matchable ( Matchable )
 import Darcs.Util.Printer ( Doc, simplePrinters, (<+>), prefix, text, vcat,
-                 vsep, (<>), ($$), errorDoc, insertBeforeLastline, empty )
+                 vsep, ($$), errorDoc, insertBeforeLastline, empty )
 import Darcs.Util.Printer.Color ( fancyPrinters )
 import Darcs.Util.Progress ( setProgressMode, debugMessage )
 import Darcs.Util.URL ( isValidLocalPath )
