@@ -16,8 +16,6 @@
 --  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 --  Boston, MA 02110-1301, USA.
 
-{-# LANGUAGE CPP #-}
-
 module Darcs.UI.Commands.Move ( move, mv ) where
 
 import Prelude ()
@@ -28,11 +26,17 @@ import Data.Maybe ( fromMaybe )
 import Darcs.Util.SignalHandler ( withSignalsBlocked )
 
 import Darcs.UI.Commands
-    ( DarcsCommand(..), withStdOpts, nodefaults, commandAlias, amInHashedRepository )
-import Darcs.UI.Flags ( DarcsFlag(Quiet)
-    , doAllowCaseOnly, doAllowWindowsReserved, useCache, dryRun, umask
-    , maybeFixSubPaths, fixSubPaths )
-import Darcs.UI.Options ( DarcsOption, (^), odesc, ocheck, onormalise, defaultFlags )
+    ( DarcsCommand(..), withStdOpts, nodefaults, commandAlias, amInHashedRepository
+    , putInfo
+    )
+import Darcs.UI.Completion ( knownFileArgs )
+import Darcs.UI.Flags
+    ( DarcsFlag
+    , allowCaseDifferingFilenames, allowWindowsReservedFilenames
+    , useCache, dryRun, umask
+    , maybeFixSubPaths, fixSubPaths
+    )
+import Darcs.UI.Options ( (^), odesc, ocheck, onormalise, defaultFlags, (?) )
 import qualified Darcs.UI.Options.All as O
 import Darcs.Repository.Diff ( treeDiff )
 import Darcs.Repository.Flags ( UpdateWorking (..), DiffAlgorithm(..) )
@@ -45,13 +49,12 @@ import Darcs.Repository
     , withRepoLock
     , RepoJob(..)
     , addPendingDiffToPending
-    , listFiles
     )
 import Darcs.Patch.Witnesses.Ordered ( FL(..), (+>+) )
 import Darcs.Patch.Witnesses.Sealed ( emptyGap, freeGap, joinGap, FreeLeft )
 import Darcs.Util.Global ( debugMessage )
 import qualified Darcs.Patch
-import Darcs.Patch ( RepoPatch, PrimPatch, PrimOf )
+import Darcs.Patch ( RepoPatch, PrimPatch )
 import Darcs.Patch.Apply( ApplyState )
 import Data.List ( nub, sort )
 import qualified System.FilePath.Windows as WindowsFilePath
@@ -68,6 +71,7 @@ import Darcs.Util.Path
     , toFilePath
     , AbsolutePath
     )
+import Darcs.Util.Printer ( text, hsep )
 import Darcs.Util.Workaround ( renameFile )
 
 moveDescription :: String
@@ -89,30 +93,6 @@ moveHelp =
  "`ReadMe` and `README`).  If `--case-ok` is used, the repository might be\n" ++
  "unusable on those systems!\n"
 
-moveBasicOpts :: DarcsOption a (Bool -> Bool -> Maybe String -> a)
-moveBasicOpts = O.allowProblematicFilenames ^ O.workingRepoDir
-
-moveAdvancedOpts :: DarcsOption a (O.UMask -> a)
-moveAdvancedOpts = O.umask
-
-moveOpts :: DarcsOption a
-            (Bool
-             -> Bool
-             -> Maybe String
-             -> Maybe O.StdCmdAction
-             -> Bool
-             -> Bool
-             -> O.Verbosity
-             -> Bool
-             -> O.UMask
-             -> O.UseCache
-             -> Maybe String
-             -> Bool
-             -> Maybe String
-             -> Bool
-             -> a)
-moveOpts = moveBasicOpts `withStdOpts` moveAdvancedOpts
-
 move :: DarcsCommand [DarcsFlag]
 move = DarcsCommand
     { commandProgramName = "darcs"
@@ -123,7 +103,7 @@ move = DarcsCommand
     , commandExtraArgHelp = ["<SOURCE> ... <DESTINATION>"]
     , commandCommand = moveCmd
     , commandPrereq = amInHashedRepository
-    , commandGetArgPossibilities = listFiles False
+    , commandCompleteArgs = knownFileArgs
     , commandArgdefaults = nodefaults
     , commandAdvancedOptions = odesc moveAdvancedOpts
     , commandBasicOptions = odesc moveBasicOpts
@@ -131,6 +111,10 @@ move = DarcsCommand
     , commandCheckOptions = ocheck moveOpts
     , commandParseOptions = onormalise moveOpts
     }
+  where
+    moveBasicOpts = O.allowProblematicFilenames ^ O.repoDir
+    moveAdvancedOpts = O.umask
+    moveOpts = moveBasicOpts `withStdOpts` moveAdvancedOpts
 
 moveCmd :: (AbsolutePath, AbsolutePath) -> [DarcsFlag] -> [String] -> IO ()
 moveCmd fps opts args
@@ -235,31 +219,27 @@ moveFilesToDir opts froms to = withRepoAndState opts $ \(repo, work, cur, _) ->
 
 withRepoAndState :: [DarcsFlag]
                  -> (forall rt p wR wU .
-                        (ApplyState p ~ Tree, ApplyState (PrimOf p) ~ Tree, RepoPatch p) =>
+                        (ApplyState p ~ Tree, RepoPatch p) =>
                             (Repository rt p wR wU wR, Tree IO, Tree IO, Tree IO)
                                 -> IO ())
                  -> IO ()
 withRepoAndState opts f =
-    withRepoLock dr uc YesUpdateWorking um $ RepoJob $ \repo -> do
+    withRepoLock (dryRun ? opts) (useCache ? opts) YesUpdateWorking (umask ? opts) $
+    RepoJob $ \repo -> do
         work <- readPlainTree "."
         cur <- readRecordedAndPending repo
         recorded <- readRecorded repo
         f (repo, work, cur, recorded)
-  where
-    dr = dryRun opts
-    uc = useCache opts
-    um = umask opts
 
-simpleMove :: (RepoPatch p, ApplyState p ~ Tree, ApplyState (PrimOf p) ~ Tree)
+simpleMove :: (RepoPatch p, ApplyState p ~ Tree)
            => Repository rt p wR wU wT
            -> [DarcsFlag] -> Tree IO -> Tree IO -> FilePath -> FilePath
            -> IO ()
 simpleMove repository opts cur work old_fp new_fp = do
     doMoves repository opts cur work [(old_fp, new_fp)]
-    unless (Quiet `elem` opts) $
-        putStrLn $ unwords ["Moved:", old_fp, "to:", new_fp]
+    putInfo opts $ hsep $ map text ["Moved:", old_fp, "to:", new_fp]
 
-moveToDir :: (RepoPatch p, ApplyState p ~ Tree, ApplyState (PrimOf p) ~ Tree)
+moveToDir :: (RepoPatch p, ApplyState p ~ Tree)
           => Repository rt p wR wU wT
           -> [DarcsFlag] -> Tree IO -> Tree IO -> [FilePath] -> FilePath
           -> IO ()
@@ -267,10 +247,9 @@ moveToDir repository opts cur work moved finaldir = do
     let movetargets = map ((finaldir </>) . takeFileName) moved
         moves = zip moved movetargets
     doMoves repository opts cur work moves
-    unless (Quiet `elem` opts) $
-      putStrLn $ unwords $ ["Moved:"] ++ moved ++ ["to:", finaldir]
+    putInfo opts $ hsep $ map text $ ["Moved:"] ++ moved ++ ["to:", finaldir]
 
-doMoves :: (RepoPatch p, ApplyState p ~ Tree, ApplyState (PrimOf p) ~ Tree)
+doMoves :: (RepoPatch p, ApplyState p ~ Tree)
           => Repository rt p wR wU wT
           -> [DarcsFlag] -> Tree IO -> Tree IO
           -> [(FilePath, FilePath)] -> IO ()
@@ -317,8 +296,7 @@ generatePreMovePatches opts cur work (old,new) = do
                 then Just <$> deleteNewFromRepoPatches
                 else return Nothing
         else do
-          unless (Quiet `elem` opts) $
-              putStrLn "Detected post-hoc move."
+          putInfo opts $ text "Detected post-hoc move."
           -- Post-hoc move - user has moved/deleted the file in working, so
           -- we can hopefully make a move patch to make the repository
           -- consistent.
@@ -331,7 +309,7 @@ generatePreMovePatches opts cur work (old,new) = do
                        else return $ emptyGap NilFL
   where
     newIsOkWindowsPath =
-        doAllowWindowsReserved opts || WindowsFilePath.isValid new
+        allowWindowsReservedFilenames ? opts || WindowsFilePath.isValid new
 
     newNotOkWindowsPathMsg =
         "The filename " ++ new ++ " is not valid under Windows.\n"
@@ -340,9 +318,8 @@ generatePreMovePatches opts cur work (old,new) = do
     -- If we're moving to a file/dir that was recorded, but has been deleted,
     -- we need to add patches to pending that remove the original.
     deleteNewFromRepoPatches = do
-        unless (Quiet `elem` opts) $
-            putStrLn $ "Existing recorded contents of " ++ new
-                       ++ " will be overwritten."
+        putInfo opts $ text $
+          "Existing recorded contents of " ++ new ++ " will be overwritten."
         ftf <- filetypeFunction
         let curNoNew = modifyTree cur (floatPath new) Nothing
         -- Return patches to remove new, so that the move patch
@@ -352,10 +329,10 @@ generatePreMovePatches opts cur work (old,new) = do
     -- Check if the passed tree has the new filepath. The old path is removed
     -- from the tree before checking if the new path is present.
     hasNew s = treeHas_case (modifyTree s (floatPath old) Nothing) new
-    treeHas_case = if doAllowCaseOnly opts then treeHas else treeHasAnycase
+    treeHas_case = if allowCaseDifferingFilenames ? opts then treeHas else treeHasAnycase
 
     alreadyExists inWhat =
-        if doAllowCaseOnly opts
+        if allowCaseDifferingFilenames ? opts
             then "A file or dir named "++new++" already exists in "
                   ++ inWhat ++ "."
             else "A file or dir named "++new++" (or perhaps differing "
