@@ -24,7 +24,7 @@ import Darcs.Prelude
 
 import Control.Monad ( when, unless, forM_ )
 import Data.List ( nub )
-import Data.Maybe ( isJust, fromJust )
+import Data.Maybe ( fromJust, isJust )
 import System.Directory
     ( getDirectoryContents
     , doesDirectoryExist
