@@ -27,7 +27,7 @@ import qualified Data.ByteString.Char8 as BC
 import qualified Data.ByteString.Lazy as BL
 import qualified Data.ByteString as B
 import Data.Char ( isSpace )
-import Data.Maybe ( isJust )
+import Data.Maybe ( fromJust, isJust )
 import Control.Exception ( catch, IOException )
 import Control.Monad ( unless, filterM, void )
 import Darcs.Util.Tree( readBlob, modifyTree, findFile, TreeItem(..), Tree
