@@ -128,7 +128,7 @@ import Darcs.Util.Progress ( debugMessage )
 import Darcs.UI.Email ( makeEmail )
 import Darcs.UI.Completion ( prefArgs )
 import Darcs.Util.Printer
-    ( Doc, vsep, text, ($$), (<+>), (<>), putDoc, putDocLn
+    ( Doc, vsep, text, ($$), (<+>), putDoc, putDocLn
     , renderPS, vcat
     )
 import Darcs.Util.English ( englishNum, Noun(..) )
