@@ -15,18 +15,17 @@
 --  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 --  Boston, MA 02110-1301, USA.
 
-{-# LANGUAGE CPP #-}
 module Darcs.UI.Commands.ShowFiles ( showFiles ) where
 
 import Prelude ()
 import Darcs.Prelude
 
-import Prelude hiding ( (^) )
 import Darcs.UI.Flags ( DarcsFlag, useCache )
-import Darcs.UI.Options ( DarcsOption, (^), oid, odesc, ocheck, onormalise
-                        , defaultFlags, parseFlags )
+import Darcs.UI.Options ( (^), oid, odesc, ocheck, onormalise
+                        , defaultFlags, parseFlags, (?) )
 import qualified Darcs.UI.Options.All as O
 import Darcs.UI.Commands ( DarcsCommand(..), withStdOpts, nodefaults, amInRepository )
+import Darcs.UI.Completion ( knownFileArgs )
 import Darcs.Repository ( Repository, withRepository,
                           RepoJob(..), repoPatchType )
 import Darcs.Patch ( IsRepoType, RepoPatch )
@@ -39,7 +38,7 @@ import System.FilePath ( splitDirectories )
 
 import Data.List( isPrefixOf )
 
-import Darcs.Patch.Match ( haveNonrangeMatch )
+import Darcs.Patch.Match ( haveNonrangeExplicitMatch )
 import Darcs.Repository.Match ( getNonrangeMatch )
 import Darcs.Util.Lock ( withDelayedDir )
 
@@ -70,53 +69,33 @@ showFilesHelp =
  "\n" ++
  "    darcs show files -0 | xargs -0 ls -ldS\n"
 
-showFilesBasicOpts :: DarcsOption a
-                      (Bool -> Bool -> Bool -> Bool -> [O.MatchFlag] -> Maybe String -> a)
-showFilesBasicOpts
-    = O.files
-    ^ O.directories
-    ^ O.pending
-    ^ O.nullFlag
-    ^ O.matchUpToOne
-    ^ O.workingRepoDir
-
-showFilesOpts :: DarcsOption a
-                 (Bool
-                  -> Bool
-                  -> Bool
-                  -> Bool
-                  -> [O.MatchFlag]
-                  -> Maybe String
-                  -> Maybe O.StdCmdAction
-                  -> Bool
-                  -> Bool
-                  -> O.Verbosity
-                  -> Bool
-                  -> O.UseCache
-                  -> Maybe String
-                  -> Bool
-                  -> Maybe String
-                  -> Bool
-                  -> a)
-showFilesOpts = showFilesBasicOpts `withStdOpts` oid
-
 showFiles :: DarcsCommand [DarcsFlag]
-showFiles = DarcsCommand {
-  commandProgramName = "darcs",
-  commandName = "files",
-  commandHelp = showFilesHelp,
-  commandDescription = showFilesDescription,
-  commandExtraArgs = -1,
-  commandExtraArgHelp = ["[FILE or DIRECTORY]..."],
-  commandCommand = manifestCmd toListFiles,
-  commandPrereq = amInRepository,
-  commandGetArgPossibilities = return [],
-  commandArgdefaults = nodefaults,
-  commandAdvancedOptions = [],
-  commandBasicOptions = odesc showFilesBasicOpts,
-  commandDefaults = defaultFlags showFilesOpts,
-  commandCheckOptions = ocheck showFilesOpts,
-  commandParseOptions = onormalise showFilesOpts }
+showFiles = DarcsCommand
+    { commandProgramName = "darcs"
+    , commandName = "files"
+    , commandHelp = showFilesHelp
+    , commandDescription = showFilesDescription
+    , commandExtraArgs = -1
+    , commandExtraArgHelp = ["[FILE or DIRECTORY]..."]
+    , commandCommand = manifestCmd toListFiles
+    , commandPrereq = amInRepository
+    , commandCompleteArgs = knownFileArgs
+    , commandArgdefaults = nodefaults
+    , commandAdvancedOptions = []
+    , commandBasicOptions = odesc showFilesBasicOpts
+    , commandDefaults = defaultFlags showFilesOpts
+    , commandCheckOptions = ocheck showFilesOpts
+    , commandParseOptions = onormalise showFilesOpts
+    }
+  where
+    showFilesBasicOpts
+      = O.files
+      ^ O.directories
+      ^ O.pending
+      ^ O.nullFlag
+      ^ O.matchUpToOne
+      ^ O.repoDir
+    showFilesOpts = showFilesBasicOpts `withStdOpts` oid
 
 toListFiles :: [DarcsFlag] -> Tree m -> [FilePath]
 toListFiles    opts = filesDirs (parseFlags O.files opts) (parseFlags O.directories opts)
@@ -137,7 +116,7 @@ manifestCmd to_list _ opts argList =
 
 manifestHelper :: ([DarcsFlag] -> Tree IO -> [FilePath]) -> [DarcsFlag] -> [String] -> IO [FilePath]
 manifestHelper to_list opts argList = do
-    list' <- to_list opts `fmap` withRepository (useCache opts) (RepoJob slurp)
+    list' <- to_list opts `fmap` withRepository (useCache ? opts) (RepoJob slurp)
     case argList of
         []       -> return list'
         prefixes -> return (onlysubdirs prefixes list')
@@ -146,7 +125,7 @@ manifestHelper to_list opts argList = do
     slurp :: (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree)
           => Repository rt p wR wU wR -> IO (Tree IO)
     slurp r = do
-      let fUpto = haveNonrangeMatch (repoPatchType r) matchFlags
+      let fUpto = haveNonrangeExplicitMatch (repoPatchType r) matchFlags
           fPending = parseFlags O.pending opts
       -- this covers all 4 possibilities
       case (fUpto,fPending) of
