@@ -74,7 +74,7 @@ import Darcs.UI.SelectChanges ( WhichChanges(..),
                                 selectionContext, runSelection )
 import qualified Darcs.UI.SelectChanges as S ( PatchSelectionOptions (..) )
 import Darcs.Util.English ( presentParticiple )
-import Darcs.Util.Printer ( text, putDoc, vcat, (<>), (<+>), ($$) )
+import Darcs.Util.Printer ( text, putDoc, vcat, (<+>), ($$) )
 import Darcs.Util.Progress ( debugMessage )
 
 unrecordDescription :: String
