@@ -15,9 +15,6 @@
 --  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 --  Boston, MA 02110-1301, USA.
 
-{-# LANGUAGE CPP #-}
-
-
 module Darcs.UI.Commands.WhatsNew
     (
       whatsnew
@@ -37,17 +34,16 @@ import Data.List.Ordered ( nubSort )
 import Darcs.Patch
     ( PrimOf, PrimPatch, RepoPatch
     , applyToTree, plainSummaryPrims, primIsHunk
-    , listTouchedFiles, IsRepoType
     )
-import Darcs.Patch.Apply ( ApplyState )
-import Darcs.Patch.Choices ( patchChoicesLps, lpPatch )
+import Darcs.Patch.Apply ( Apply, ApplyState )
+import Darcs.Patch.Choices ( mkPatchChoices, labelPatches, unLabel )
+import Darcs.Patch.Commute ( Commute )
 import Darcs.Patch.FileHunk ( IsHunk (..) )
-import Darcs.Patch.Format ( PatchListFormat (..) )
+import Darcs.Patch.Format ( PatchListFormat )
 import Darcs.Patch.Inspect ( PatchInspect (..) )
-import Darcs.Patch.Patchy ( Patchy )
 import Darcs.Patch.Permutations ( partitionRL )
 import Darcs.Patch.Prim.Class ( PrimDetails (..) )
-import Darcs.Patch.Show ( ShowPatch )
+import Darcs.Patch.Show ( ShowPatch, ShowContextPatch )
 import Darcs.Patch.Split ( primSplitter )
 import Darcs.Patch.TouchesFiles ( choosePreTouching )
 import Darcs.Patch.Witnesses.Ordered
@@ -58,29 +54,29 @@ import Darcs.Patch.Witnesses.Sealed
     ( Sealed (..), Sealed2 (..)
     , unFreeLeft
     )
-import Darcs.Patch.Witnesses.Unsafe ( unsafeCoerceP )
 import Darcs.Patch.Witnesses.WZipper ( FZipper (..) )
 import Darcs.Repository
     ( RepoJob (..), Repository
-    , listRegisteredFiles, readRecorded, readRepo
-    , unrecordedChangesWithPatches, withRepository
+    , readRecorded
+    , unrecordedChanges, withRepository
     )
 import Darcs.Repository.Diff ( treeDiff )
 import Darcs.Repository.Prefs ( filetypeFunction )
-import Darcs.Repository.State ( getMovesPs, getReplaces )
 import Darcs.UI.Commands
     ( DarcsCommand(..), withStdOpts, amInRepository
     , commandAlias, nodefaults
     )
-import Darcs.Repository.Resolution ( patchsetConflictResolutions )
+import Darcs.UI.Completion ( modifiedFileArgs )
 import Darcs.UI.Commands.Util ( announceFiles, filterExistingPaths )
 import Darcs.UI.Flags
-    ( DarcsFlag (Summary, LookForAdds, LookForMoves), diffAlgorithm, diffingOpts
-    , isUnified, useCache, fixSubPaths
-    , verbosity, isInteractive, isUnified, lookForAdds, lookForMoves, lookForReplaces, hasSummary
-    , scanKnown, useIndex
+    ( DarcsFlag, diffAlgorithm
+    , withContext, useCache, fixSubPaths
+    , verbosity, isInteractive
+    , lookForAdds, lookForMoves, lookForReplaces
+    , scanKnown, useIndex, diffingOpts
     )
-import Darcs.UI.Options ( DarcsOption, (^), odesc, ocheck, onormalise, defaultFlags, parseFlags )
+import Darcs.UI.Options
+    ( DarcsOption, (^), odesc, ocheck, onormalise, defaultFlags, parseFlags, (?) )
 import qualified Darcs.UI.Options.All as O
 import Darcs.UI.PrintPatch
     ( contextualPrintPatch, printPatch
@@ -98,73 +94,37 @@ import Darcs.UI.SelectChanges
     , skipOne, printSummary
     )
 import qualified Darcs.UI.SelectChanges as S ( PatchSelectionOptions (..) )
-import Darcs.Util.Path ( AbsolutePath, SubPath, toFilePath, fp2fn )
+import Darcs.Util.Path ( AbsolutePath, SubPath, toFilePath )
 import Darcs.Util.Printer
-    ( putDocLn, renderString, RenderMode(..)
+    ( putDocLn, renderString
     , text, vcat
     )
 import Darcs.Util.Prompt ( PromptConfig (..), promptChar )
 
-
-whatsnewBasicOpts :: DarcsOption a
-                     (Maybe O.Summary
-                      -> O.WithContext
-                      -> Bool
-                      -> O.LookFor
-                      -> O.DiffAlgorithm
-                      -> Maybe String
-                      -> Maybe Bool
-                      -> a)
-whatsnewBasicOpts
-    = O.summary
-    ^ O.withContext
-    ^ O.machineReadable
-    ^ O.lookfor
-    ^ O.diffAlgorithm
-    ^ O.workingRepoDir
-    ^ O.interactive -- False
-
-whatsnewAdvancedOpts :: DarcsOption a (O.UseIndex -> O.IncludeBoring -> a)
-whatsnewAdvancedOpts = O.useIndex ^ O.includeBoring
-
-whatsnewOpts :: DarcsOption a
-                (Maybe O.Summary
-                 -> O.WithContext
-                 -> Bool
-                 -> O.LookFor
-                 -> O.DiffAlgorithm
-                 -> Maybe String
-                 -> Maybe Bool
-                 -> Maybe O.StdCmdAction
-                 -> Bool
-                 -> Bool
-                 -> O.Verbosity
-                 -> Bool
-                 -> O.UseIndex
-                 -> O.IncludeBoring
-                 -> O.UseCache
-                 -> Maybe String
-                 -> Bool
-                 -> Maybe String
-                 -> Bool
-                 -> a)
-whatsnewOpts = whatsnewBasicOpts `withStdOpts` whatsnewAdvancedOpts
+commonAdvancedOpts :: DarcsOption a (O.UseIndex -> O.IncludeBoring -> a)
+commonAdvancedOpts = O.useIndex ^ O.includeBoring
 
 patchSelOpts :: [DarcsFlag] -> S.PatchSelectionOptions
 patchSelOpts flags = S.PatchSelectionOptions
-    { S.verbosity = verbosity flags
+    { S.verbosity = verbosity ? flags
     , S.matchFlags = []
     , S.interactive = isInteractive True flags
     , S.selectDeps = O.PromptDeps -- option not supported, use default
-    , S.summary = hasSummary (defaultSummary flags) flags
-    , S.withContext = isUnified flags
+    , S.summary = getSummary flags
+    , S.withContext = withContext ? flags
     }
 
-defaultSummary :: [DarcsFlag] -> O.Summary
-defaultSummary flags
-  | lookForAdds flags == O.YesLookForAdds = O.YesSummary
-  | parseFlags O.machineReadable flags    = O.YesSummary
-  | otherwise = O.NoSummary
+-- lookForAdds and machineReadable set YesSummary
+-- unless NoSummary was given expressly
+-- (or by default e.g. status)
+getSummary :: [DarcsFlag] -> O.Summary
+getSummary flags = case O.maybeSummary Nothing ? flags of
+  Just O.NoSummary -> O.NoSummary
+  Just O.YesSummary -> O.YesSummary
+  Nothing
+    | O.yes (lookForAdds flags) -> O.YesSummary
+    | O.machineReadable ? flags -> O.YesSummary
+    | otherwise -> O.NoSummary
 
 whatsnew :: DarcsCommand [DarcsFlag]
 whatsnew = DarcsCommand
@@ -176,14 +136,24 @@ whatsnew = DarcsCommand
     , commandExtraArgHelp = ["[FILE or DIRECTORY]..."]
     , commandCommand = whatsnewCmd
     , commandPrereq = amInRepository
-    , commandGetArgPossibilities = listRegisteredFiles
+    , commandCompleteArgs = modifiedFileArgs
     , commandArgdefaults = nodefaults
-    , commandAdvancedOptions = odesc whatsnewAdvancedOpts
+    , commandAdvancedOptions = odesc commonAdvancedOpts
     , commandBasicOptions = odesc whatsnewBasicOpts
     , commandDefaults = defaultFlags whatsnewOpts
     , commandCheckOptions = ocheck whatsnewOpts
     , commandParseOptions = onormalise whatsnewOpts
     }
+  where
+    whatsnewBasicOpts
+      = O.maybeSummary Nothing
+      ^ O.withContext
+      ^ O.machineReadable
+      ^ O.lookfor
+      ^ O.diffAlgorithm
+      ^ O.repoDir
+      ^ O.interactive -- False
+    whatsnewOpts = whatsnewBasicOpts `withStdOpts` commonAdvancedOpts
 
 whatsnewDescription :: String
 whatsnewDescription = "List unrecorded changes in the working tree."
@@ -207,9 +177,9 @@ whatsnewHelp =
  "* `a f` and `a d/` respectively mean a new, but unadded, file or\n" ++
  "  directory, when using `--look-for-adds`.\n" ++
  "\n" ++
- "  An exclamation mark (!) as in `R! foo.c`, means the hunk is known to\n" ++
- "  conflict with a hunk in another patch.  The phrase `duplicated`\n" ++
- "  means the hunk is known to be identical to a hunk in another patch.\n" ++
+ "  An exclamation mark (!) as in `R! foo.c`, means the change is known to\n" ++
+ "  conflict with a change in another patch.  The phrase `duplicated`\n" ++
+ "  means the change is known to be identical to a change in another patch.\n" ++
  "\n" ++
  "The `--machine-readable` option implies `--summary` while making it more\n" ++
  "parsable. Modified files are only shown as `M f`, and moves are shown in\n" ++
@@ -225,70 +195,96 @@ whatsnewHelp =
 
 whatsnewCmd :: (AbsolutePath, AbsolutePath) -> [DarcsFlag] -> [String] -> IO ()
 whatsnewCmd fps opts args =
-   withRepository (useCache opts) $ RepoJob $ \(repo :: Repository rt p wR wU wR) -> do
-    let scan = scanKnown (O.adds (parseFlags O.lookfor opts)) (parseFlags O.includeBoring opts)
+   withRepository (useCache ? opts) $ RepoJob $ \(repo :: Repository rt p wR wU wR) -> do
+    let scan = scanKnown (lookForAdds opts) (O.includeBoring ? opts)
     existing_files <- do
       files <- if null args then return Nothing
                else Just . nubSort <$> fixSubPaths fps args
       when (files == Just []) $ fail "No valid arguments were given."
-      files' <- traverse (filterExistingPaths repo (verbosity opts) (useIndex opts) scan) files
+      files' <- traverse
+        (filterExistingPaths
+          repo (verbosity ? opts) (useIndex ? opts) scan (lookForMoves opts))
+        files
       let files'' = fmap snd files'
       when (files'' == Just []) $
         fail "None of the files you specified exist."
       return files''
-    let isLookForMoves = lookForMoves opts == O.YesLookForMoves && parseFlags O.summary opts /= Just O.NoSummary
-        isLookForAdds = lookForAdds opts == O.YesLookForAdds && parseFlags O.summary opts /= Just O.NoSummary
-        isLookForReplaces = lookForReplaces opts == O.YesLookForReplaces
-        isMachineReadable = parseFlags O.machineReadable opts
-        -- LookForAdds and LookForMoves implies Summary, unless it's explcitly disabled.
-        opts' | isLookForAdds     = (Summary : filter (\o -> LookForAdds /= o &&
-                                                             LookForMoves /= o ) opts)
-              | isMachineReadable = (Summary:opts)
-              | otherwise         = opts
-    movesPs <- if isLookForMoves
-        then getMovesPs repo existing_files
-        else return NilFL
-    Sealed replacePs <- if isLookForReplaces
-        then getReplaces (diffingOpts opts) repo existing_files
-        else return (Sealed NilFL)
-    Sealed noLookChanges <- filteredUnrecordedChanges opts' repo existing_files movesPs
-                                                                       (unsafeCoerceP replacePs :: FL (PrimOf p) wR wR)
+
+    -- get all unrecorded changes, possibly including unadded or even boring
+    -- files if the appropriate options were supplied
+    Sealed allInterestingChanges <-
+      filteredUnrecordedChanges (diffingOpts opts)
+        (lookForMoves opts) (lookForReplaces opts)
+        repo existing_files
+
+    -- get the recorded state
     pristine <- readRecorded repo
-    -- If we are looking for moves, return the corresponding FL of changes.
-    -- If we are looking for adds, return the corresponding FL of changes.
-    Sealed unaddedNewPathsPs <- if isLookForAdds
-        then do
-            -- Use opts not opts', here, since we *do* want to look for adds.
-            Sealed lookChanges <- filteredUnrecordedChanges opts repo existing_files movesPs (unsafeCoerceP replacePs :: FL (PrimOf p) wR wR)
-            noLookAddsTree <- applyAddPatchesToPristine noLookChanges pristine
-            lookAddsTree <- applyAddPatchesToPristine lookChanges pristine
-            ftf <- filetypeFunction
-            -- Return the patches that create files/dirs that aren't yet added.
-            unFreeLeft <$> treeDiff (diffAlgorithm opts) ftf noLookAddsTree lookAddsTree
+
+    -- the case --look-for-adds and --summary must be handled specially
+    -- in order to distinguish added and unadded files
+
+    -- TODO: it would be nice if we could return the pair
+    -- (noLookChanges,unaddedNewPathsPs) in one go and also
+    -- with proper witnesses (e.g. as noLookChanges +>+ unaddedNewPathsPs)
+    -- This would also obviate the need for samePatchType.
+    Sealed noLookChanges <-
+      if haveLookForAddsAndSummary
+        then
+          -- do *not* look for adds here:
+          filteredUnrecordedChanges (O.useIndex ? opts, O.ScanKnown, O.diffAlgorithm ? opts)
+            (lookForMoves opts) (lookForReplaces opts)
+            repo existing_files
         else return (Sealed NilFL)
-    announceFiles (verbosity opts) existing_files "What's new in"
-    exitOnNoChanges (unaddedNewPathsPs, noLookChanges)
+    Sealed unaddedNewPathsPs <-
+      if haveLookForAddsAndSummary
+        then do
+          noLookAddsTree <- applyAddPatchesToPristine noLookChanges pristine
+          lookAddsTree <- applyAddPatchesToPristine allInterestingChanges pristine
+          ftf <- filetypeFunction
+          -- Return the patches that create files/dirs that aren't yet added.
+          unFreeLeft <$> treeDiff (diffAlgorithm ? opts) ftf noLookAddsTree lookAddsTree
+        else return (Sealed NilFL)
+    -- avoid ambiguous typing for unaddedNewPathsPs:
+    samePatchType noLookChanges unaddedNewPathsPs
+
+    exitOnNoChanges allInterestingChanges
+    announceFiles (verbosity ? opts) existing_files "What's new in"
     if maybeIsInteractive opts
-      then runInteractive (interactiveHunks pristine) opts' pristine noLookChanges
-      else do
-        printChanges repo opts' pristine noLookChanges
-        printUnaddedPaths unaddedNewPathsPs
+      then
+        runInteractive (interactiveHunks pristine) (patchSelOpts opts)
+          (diffAlgorithm ? opts) pristine allInterestingChanges
+      else
+        if haveLookForAddsAndSummary
+          then do
+            printChanges pristine noLookChanges
+            printUnaddedPaths unaddedNewPathsPs
+          else do
+            printChanges pristine allInterestingChanges
   where
-    -- |Filter out hunk patches (leaving add patches) and return the tree
+    haveSummary = O.yes (getSummary opts)
+    haveLookForAddsAndSummary = haveSummary && O.yes (lookForAdds opts)
+
+    -- Filter out hunk patches (leaving add patches) and return the tree
     -- resulting from applying the filtered patches to the pristine tree.
     applyAddPatchesToPristine ps pristine = do
         adds :> _ <- return $ partitionRL primIsHunk $ reverseFL ps
         applyToTree (reverseRL adds) pristine
 
-    exitOnNoChanges :: (FL p wX wY, FL p wU wV) -> IO ()
-    exitOnNoChanges (NilFL, NilFL) = do putStrLn "No changes!"
-                                        exitWith $ ExitFailure 1
+    exitOnNoChanges :: FL p wX wY -> IO ()
+    exitOnNoChanges NilFL = do putStrLn "No changes!"
+                               exitWith $ ExitFailure 1
     exitOnNoChanges _ = return ()
 
+    -- This function does nothing. Its purpose is to enforce the
+    -- same patch type for the two passed FLs. This is necessary
+    -- in order to avoid ambiguous typing for unaddedNewPathsPs.
+    samePatchType :: FL p wX wY -> FL p wU wV -> IO ()
+    samePatchType _ _ = return ()
+
     printUnaddedPaths :: PrimPatch p => FL p wX wY -> IO ()
     printUnaddedPaths NilFL = return ()
     printUnaddedPaths ps =
-        putDocLn . lowercaseAs . renderString Encode . (plainSummaryPrims False []) $ ps
+        putDocLn . lowercaseAs . renderString . (plainSummaryPrims False) $ ps
 
     -- Make any add markers lowercase, to distinguish new-but-unadded files
     -- from those that are unrecorded, but added.
@@ -296,56 +292,55 @@ whatsnewCmd fps opts args =
     lowercaseA ('A' : x) = 'a' : x
     lowercaseA x = x
 
-    -- |Appropriately print changes, according to the passed flags.
-    printChanges :: forall rt p wR wU wX wY. (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree, ApplyState (PrimOf p) ~ Tree)
-                 => Repository rt p wR wU wR -> [DarcsFlag] -> Tree IO -> FL (PrimOf p) wX wY
+    -- Appropriately print changes, according to the passed flags.
+    -- Note this cannot make distinction between unadded and added files.
+    printChanges :: ( IsHunk p, ShowPatch p, ShowContextPatch p
+                    , PatchListFormat p, Apply p
+                    , PrimDetails p, ApplyState p ~ Tree)
+                 => Tree IO -> FL p wX wY
                  -> IO ()
-    printChanges repo opts' pristine changes
-        | Summary `elem` opts' = do
-             r <- readRepo repo
-             Sealed res <- return $ patchsetConflictResolutions r
-             let conflictFns = map fp2fn $ nubSort $ listTouchedFiles res
-             putDocLn $ plainSummaryPrims machineReadable conflictFns changes
-        | isUnified opts' == O.YesContext = contextualPrintPatch pristine changes
+    printChanges pristine changes
+        | haveSummary = putDocLn $ plainSummaryPrims machineReadable changes
+        | O.yes (withContext ? opts) = contextualPrintPatch pristine changes
         | otherwise = printPatch changes
      where machineReadable = parseFlags O.machineReadable opts
 
-    -- |return the unrecorded changes that affect an optional list of paths.
-    filteredUnrecordedChanges :: forall rt p wR wU wT. (RepoPatch p, ApplyState p ~ Tree,
-                                                        ApplyState (PrimOf p) ~ Tree)
-                              => [DarcsFlag]
+    -- return the unrecorded changes that affect an optional list of paths.
+    filteredUnrecordedChanges :: forall rt p wR wU wT. (RepoPatch p, ApplyState p ~ Tree)
+                              => (O.UseIndex, O.ScanKnown, O.DiffAlgorithm)
+                              -> O.LookForMoves
+                              -> O.LookForReplaces
                               -> Repository rt p wR wU wT -> Maybe [SubPath]
-                              -> FL (PrimOf p) wR wT -- look-for-moves patches
-                              -> FL (PrimOf p) wT wT -- look-for-replaces patches
                               -> IO (Sealed (FL (PrimOf p) wT))
-    filteredUnrecordedChanges  opts' repo files movesPs replacesPs =
+    filteredUnrecordedChanges diffing lfm lfr repo files =
         let filePaths = map toFilePath <$> files in
-        choosePreTouching filePaths <$> unrecordedChangesWithPatches movesPs replacesPs (diffingOpts opts') repo files
+        choosePreTouching filePaths <$>
+          unrecordedChanges diffing lfm lfr repo files
 
 -- | Runs the 'InteractiveSelectionM' code
-runInteractive :: (PatchListFormat p, IsHunk p, Patchy p, ShowPatch p,
-                     PrimPatch p, PatchInspect p, PrimDetails p,
-                     ApplyState p ~ Tree)
+runInteractive :: PrimPatch p
                => InteractiveSelectionM p wX wY () -- Selection to run
-               -> [DarcsFlag]     -- Command-line options
+               -> S.PatchSelectionOptions
+               -> O.DiffAlgorithm
                -> Tree IO         -- Pristine
                -> FL p wX wY      -- A list of patches
                -> IO ()
-runInteractive i opts pristine ps' = do
-    let (choices',lps') = patchChoicesLps ps'
-    let ps = evalStateT i $
+runInteractive i patchsel diffalg pristine ps' = do
+    let lps' = labelPatches Nothing ps'
+        choices' = mkPatchChoices lps'
+        ps = evalStateT i $
              ISC { total   = lengthFL lps'
                  , current = 0
                  , lps     = FZipper NilRL lps'
                  , choices = choices'
                  }
     void $ runReaderT ps $
-           selectionContextPrim First "view" (patchSelOpts opts)
-             (Just (primSplitter (diffAlgorithm opts)))
+           selectionContextPrim First "view" patchsel
+             (Just (primSplitter diffalg))
              Nothing (Just pristine)
 
 -- | The interactive part of @darcs whatsnew@
-interactiveHunks :: (PatchListFormat p, IsHunk p, Patchy p, ShowPatch p,
+interactiveHunks :: (IsHunk p, ShowPatch p, ShowContextPatch p, Commute p,
                      PatchInspect p, PrimDetails p, ApplyState p ~ Tree)
                  => Tree IO -> InteractiveSelectionM p wX wY ()
 interactiveHunks pristine = do
@@ -353,7 +348,7 @@ interactiveHunks pristine = do
     case c of
         Nothing -> liftIO $ putStrLn "No more changes!"
         Just (Sealed2 lp) -> do
-            liftIO $ printPatch (lpPatch lp)
+            liftIO $ printPatch (unLabel lp)
             repeatThis lp
   where
     repeatThis lp = do
@@ -362,14 +357,14 @@ interactiveHunks pristine = do
                 (PromptConfig thePrompt (keysFor basic_options) (keysFor adv_options)
                  (Just 'n') "?h")
         case yorn of
-            -- View hunk in context
-            'v' -> liftIO (contextualPrintPatch pristine (lpPatch lp))
+            -- View change in context
+            'v' -> liftIO (contextualPrintPatch pristine (unLabel lp))
                    >> repeatThis lp
             -- View summary of the change
-            'x' -> liftIO (printSummary (lpPatch lp))
+            'x' -> liftIO (printSummary (unLabel lp))
                    >> repeatThis lp
-            -- View hunk and move on
-            'y' -> liftIO (contextualPrintPatch pristine (lpPatch lp))
+            -- View change and move on
+            'y' -> liftIO (contextualPrintPatch pristine (unLabel lp))
                    >> decide True lp >> next_hunk
             -- Go to the next patch
             'n' -> decide False lp >> next_hunk
@@ -379,12 +374,12 @@ interactiveHunks pristine = do
                     (return ())
                     (\f -> decideWholeFile f False)
                 next_hunk
-            -- View hunk in a pager
-            'p' -> liftIO (printPatchPager $ lpPatch lp)
+            -- View change in a pager
+            'p' -> liftIO (printPatchPager $ unLabel lp)
                    >> repeatThis lp
-            -- Next hunk
+            -- Next change
             'j' -> next_hunk
-            -- Previous hunk
+            -- Previous change
             'k' -> prev_hunk
             -- Start from the first change
             'g' -> start_over
@@ -397,34 +392,49 @@ interactiveHunks pristine = do
     next_hunk  = skipOne >> skipMundane >> interactiveHunks pristine
     prev_hunk  = backOne >> interactiveHunks pristine
     options_yn =
-        [ KeyPress 'v' "view this hunk in a context"
-        , KeyPress 'y'
-          "view this hunk in a context and go to the next one"
-        , KeyPress 'n' "go to the next hunk" ]
+        [ KeyPress 'v' "view this change in a context"
+        , KeyPress 'y' "view this change in a context and go to the next one"
+        , KeyPress 'n' "skip this change and its dependencies" ]
     optionsView =
-        [ KeyPress 'p' "view this hunk in context wih pager "
-        , KeyPress 'x' "view a summary of this patch"
+        [ KeyPress 'p' "view this change in context wih pager "
+        , KeyPress 'x' "view a summary of this change"
         ]
     optionsNav =
         [ KeyPress 'q' "quit whatsnew"
         , KeyPress 's' "skip the rest of the changes to this file"
-        , KeyPress 'j' "skip to the next hunk"
-        , KeyPress 'k' "back up to previous hunk"
-        , KeyPress 'g' "start over from the first hunk"
+        , KeyPress 'j' "go to the next change"
+        , KeyPress 'k' "back up to previous change"
+        , KeyPress 'g' "start over from the first change"
         ]
     basic_options = [ options_yn ]
     adv_options = [ optionsView, optionsNav ]
 
--- |status is an alias for whatsnew, with implicit Summary and LookForAdds
+
+-- | status is an alias for whatsnew, with implicit Summary and LookForAdds
 -- flags. We override the default description, to include the implicit flags.
 status :: DarcsCommand [DarcsFlag]
-status = statusAlias { commandCommand = statusCmd
-                     , commandDescription = statusDesc
-                     }
+status = statusAlias
+    { commandDescription = statusDesc
+    , commandAdvancedOptions = odesc commonAdvancedOpts
+    , commandBasicOptions = odesc statusBasicOpts
+    , commandDefaults = defaultFlags statusOpts
+    , commandCheckOptions = ocheck statusOpts
+    , commandParseOptions = onormalise statusOpts
+    }
   where
     statusAlias = commandAlias "status" Nothing whatsnew
-    statusCmd fps fs = commandCommand whatsnew fps (Summary : LookForAdds : fs)
     statusDesc = "Alias for `darcs " ++ commandName whatsnew ++ " -ls '."
+    statusBasicOpts
+      = O.maybeSummary (Just O.YesSummary)
+      ^ O.withContext
+      ^ O.machineReadable
+      ^ O.lookforadds O.YesLookForAdds
+      ^ O.lookforreplaces
+      ^ O.lookformoves
+      ^ O.diffAlgorithm
+      ^ O.repoDir
+      ^ O.interactive
+    statusOpts = statusBasicOpts `withStdOpts` commonAdvancedOpts
 
 maybeIsInteractive :: [DarcsFlag] -> Bool
 maybeIsInteractive = maybe False id . parseFlags O.interactive
