@@ -283,6 +283,9 @@ oempty = OptSpec {..} where
   ocheck _ = []
   odesc = []
 
+instance Semigroup (PrimOptSpec d f a [v]) where
+  (<>) = oappend
+
 -- | See 'oappend' and 'oempty'.
 instance Monoid (PrimOptSpec d f a [v]) where
   mappend = oappend
@@ -295,3 +298,13 @@ instance Monoid (PrimOptSpec d f a [v]) where
 -- prop> parseFlags o fs = oparse o id fs
 parseFlags :: (forall a. PrimOptSpec d f a v) -> [f] -> v
 parseFlags o fs = oparse o id fs
+
+-- no assoiativity, higher precedence than comparisons operators (4)
+-- and lower than arithemic operators (6,7,8)
+infix 5 ?
+
+-- | Operator version of 'parseFlags'
+--
+-- prop> opt ? flags = parseFlags opt flags
+(?) :: (forall a. PrimOptSpec d f a v) -> [f] -> v
+(?) = parseFlags
