@@ -15,8 +15,6 @@
 -- the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 -- Boston, MA 02110-1301, USA.
 
-{-# LANGUAGE CPP #-}
-
 module Darcs.UI.PrintPatch
     ( printPatch
     , contextualPrintPatch
@@ -35,17 +33,16 @@ import Darcs.Util.Printer.Color ( fancyPrinters )
 import Darcs.Patch.Apply ( ApplyState )
 import Darcs.Patch ( showContextPatch, showPatch, showNicely, description,
                      summary )
-import Darcs.Patch.Show ( ShowPatch )
+import Darcs.Patch.Show ( ShowPatch, ShowContextPatch, ShowPatchFor(ForDisplay) )
 import Darcs.UI.External ( viewDocWith )
---import Darcs.UI.Flags ( DarcsFlag(Summary, Verbose), isUnified )
 import Darcs.UI.Options.All ( Verbosity(..), Summary(..), WithContext(..) )
 
-import Darcs.Util.Printer ( Doc, putDocLnWith, RenderMode(..) )
+import Darcs.Util.Printer ( Doc, putDocLnWith )
 
 -- | @'printFriendly' opts patch@ prints @patch@ in accordance with the flags
 -- in opts, ie, whether @--verbose@ or @--summary@ were passed at the
 -- command-line.
-printFriendly :: (ShowPatch p, ApplyState p ~ Tree) => Maybe (Tree IO)
+printFriendly :: (ShowPatch p, ShowContextPatch p, ApplyState p ~ Tree) => Maybe (Tree IO)
               -> Verbosity -> Summary -> WithContext -> p wX wY -> IO ()
 printFriendly (Just pristine) _ _ YesContext = contextualPrintPatch pristine
 printFriendly _ v s _ = putDocLnWith fancyPrinters . showFriendly v s
@@ -59,16 +56,16 @@ showFriendly _       NoSummary  = description
 
 -- | 'printPatch' prints a patch on standard output.
 printPatch :: ShowPatch p => p wX wY -> IO ()
-printPatch p = putDocLnWith fancyPrinters $ showPatch p
+printPatch p = putDocLnWith fancyPrinters $ showPatch ForDisplay p
 
 -- | 'printPatchPager' runs '$PAGER' and shows a patch in it.
 printPatchPager :: ShowPatch p => p wX wY -> IO ()
-printPatchPager p = viewDocWith fancyPrinters Standard $ showPatch p
+printPatchPager p = viewDocWith fancyPrinters $ showPatch ForDisplay p
 
 -- | 'contextualPrintPatch' prints a patch, together with its context, on
 -- standard output.
-contextualPrintPatch :: (ShowPatch p, ApplyState p ~ Tree) => Tree IO
+contextualPrintPatch :: (ShowContextPatch p, ApplyState p ~ Tree) => Tree IO
                      -> p wX wY -> IO ()
 contextualPrintPatch s p = do
-    (contextedPatch, _) <- virtualTreeIO (showContextPatch p) s
+    (contextedPatch, _) <- virtualTreeIO (showContextPatch ForDisplay p) s
     putDocLnWith fancyPrinters contextedPatch
