@@ -1,5 +1,4 @@
-{-# LANGUAGE BangPatterns, ForeignFunctionInterface, CPP, ScopedTypeVariables #-}
-
+{-# LANGUAGE CPP #-}
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Darcs.Util.ByteString
@@ -15,119 +14,85 @@
 -- GZIp and MMap IO for ByteStrings, encoding utilities, and miscellaneous
 -- functions for Data.ByteString
 --
-
-module Darcs.Util.ByteString (
-
-        unsafeWithInternals,
-        unpackPSFromUTF8,
-        packStringToUTF8,
-
-        -- IO with mmap or gzip
-        gzReadFilePS,
-        mmapFilePS,
-        gzWriteFilePS,
-        gzWriteFilePSs,
-        gzReadStdin,
-        gzWriteHandle,
-        FileSegment,
-        readSegment,
-
-        -- gzip handling
-        isGZFile,
-        gzDecompress,
-
-        -- list utilities
-        dropSpace,
-        breakSpace,
-        linesPS,
-        unlinesPS,
-        hashPS,
-        breakFirstPS,
-        breakLastPS,
-        substrPS,
-        readIntPS,
-        isFunky,
-        fromHex2PS,
-        fromPS2Hex,
-        betweenLinesPS,
-        breakAfterNthNewline,
-        breakBeforeNthNewline,
-        intercalate,
-
-        -- encoding and unicode utilities
-        isAscii,
-        decodeLocale,
-        encodeLocale,
-        decodeString
+module Darcs.Util.ByteString
+    (
+    -- * IO with mmap or gzip
+      gzReadFilePS
+    , mmapFilePS
+    , gzWriteFilePS
+    , gzWriteFilePSs
+    , gzReadStdin
+    , gzWriteHandle
+    , FileSegment
+    , readSegment
+    -- * gzip handling
+    , isGZFile
+    , gzDecompress
+    -- * list utilities
+    , dropSpace
+    , breakSpace
+    , linesPS
+    , unlinesPS
+    , hashPS
+    , breakFirstPS
+    , breakLastPS
+    , substrPS
+    , readIntPS
+    , isFunky
+    , fromHex2PS
+    , fromPS2Hex
+    , betweenLinesPS
+    , intercalate
+    -- * encoding and unicode utilities
+    , isAscii
+    , decodeLocale
+    , encodeLocale
+    , unpackPSFromUTF8
+    , packStringToUTF8
+    -- * properties
+    , prop_unlinesPS_linesPS_left_inverse
+    , prop_linesPS_length
+    , prop_unlinesPS_length
+    , propHexConversion
+    , spec_betweenLinesPS
     ) where
 
 import Prelude ()
 import Darcs.Prelude
 
+import Codec.Binary.Base16 ( b16Enc, b16Dec )
+
 import qualified Data.ByteString            as B
 import qualified Data.ByteString.Char8      as BC
-import qualified Data.ByteString.Internal   as BI
-import qualified Data.ByteString.Lazy.Char8 as BL8
+import qualified Data.ByteString.Lazy       as BL
 import Data.ByteString (intercalate)
-import Data.ByteString.Internal (fromForeignPtr)
 
-import Control.Exception ( catch, SomeException )
 import System.IO ( withFile, IOMode(ReadMode)
                  , hSeek, SeekMode(SeekFromEnd,AbsoluteSeek)
                  , openBinaryFile, hClose, Handle, hGetChar
                  , stdin)
+import System.IO.Error          ( catchIOError )
 import System.IO.Unsafe         ( unsafePerformIO )
 
-import Foreign.Storable         ( peek )
-import Foreign.Marshal.Array    ( advancePtr )
-import Foreign.C.Types          ( CInt(..) )
-
 import Data.Bits                ( rotateL )
-import Data.Char                ( ord, isSpace )
+import Data.Char                ( ord, isSpace, toLower, toUpper )
 import Data.Word                ( Word8 )
 import Data.Int                 ( Int32, Int64 )
-import qualified Data.Text as T ( pack, unpack )
-import Data.Text.Encoding       ( encodeUtf8, decodeUtf8With )
-import Data.Text.Encoding.Error ( lenientDecode )
+import Data.List                ( intersperse )
 import Control.Monad            ( when )
-#if MIN_VERSION_zlib(0,6,0)
 import Control.Monad.ST.Lazy    ( ST )
-#endif
 
-import Foreign.Ptr              ( plusPtr, Ptr )
-import Foreign.ForeignPtr       ( withForeignPtr )
-
-#ifdef DEBUG_PS
-import Foreign.ForeignPtr       ( addForeignPtrFinalizer )
-import Foreign.Ptr              ( FunPtr )
-#endif
-
-import qualified Data.ByteString.Lazy as BL
 import qualified Codec.Compression.GZip as GZ
 import qualified Codec.Compression.Zlib.Internal as ZI
-import Darcs.Util.Encoding ( decode, encode )
+import Darcs.Util.Encoding ( decode, encode, decodeUtf8, encodeUtf8 )
 import Darcs.Util.Global ( addCRCWarning )
 
 #if mingw32_HOST_OS
 #else
 import System.IO.MMap( mmapFileByteString )
-import System.Posix.Files( fileSize, getSymbolicLinkStatus )
 #endif
 import System.Mem( performGC )
-import qualified Bundled.Posix as Bundled ( getFileStatus, fileSize )
-
--- -----------------------------------------------------------------------------
--- obsolete debugging code
-
--- -----------------------------------------------------------------------------
--- unsafeWithInternals
-
--- | Do something with the internals of a PackedString. Beware of
--- altering the contents!
-unsafeWithInternals :: B.ByteString -> (Ptr Word8 -> Int -> IO a) -> IO a
-unsafeWithInternals ps f
- = case BI.toForeignPtr ps of
-   (fp,s,l) -> withForeignPtr fp $ \p -> f (p `plusPtr` s) l
+import System.Posix.Files( fileSize, getSymbolicLinkStatus )
 
 -- | readIntPS skips any whitespace at the beginning of its argument, and
 -- reads an Int from the beginning of the PackedString.  If there is no
@@ -138,33 +103,19 @@ unsafeWithInternals ps f
 readIntPS :: B.ByteString -> Maybe (Int, B.ByteString)
 readIntPS = BC.readInt . BC.dropWhile isSpace
 
--- -----------------------------------------------------------------------------
--- Destructor functions (taking PackedStrings apart)
-
--- | Decodes a 'ByteString' containing UTF-8 to a 'String'. Decoding errors are
---   flagged with the U+FFFD character.
-unpackPSFromUTF8 :: B.ByteString -> String
-unpackPSFromUTF8  = T.unpack . decodeUtf8With lenientDecode
-
-packStringToUTF8 :: String -> B.ByteString
-packStringToUTF8 = encodeUtf8 . T.pack
-
 ------------------------------------------------------------------------
--- A reimplementation of Data.ByteString.Char8.dropSpace, but
--- specialised to darcs' need for a 4 way isspace.
---
--- TODO: if it is safe to use the expanded definition of isSpaceWord8
--- provided by Data.ByteString.Char8, then all this can go.
-
 -- A locale-independent isspace(3) so patches are interpreted the same everywhere.
 -- ((c) == ' ' || (c) == '\t' || (c) == '\n' || (c) == '\r')
 isSpaceWord8 :: Word8 -> Bool
 isSpaceWord8 = (`elem` [0x20, 0x09, 0x0A, 0x0D])
 {-# INLINE isSpaceWord8 #-}
 
+-- | Drop leading white space, where white space is defined as
+-- consisting of ' ', '\t', '\n', or '\r'.
 dropSpace :: B.ByteString -> B.ByteString
 dropSpace bs = B.dropWhile isSpaceWord8 bs
 
+-- | Split at first occurrence of ' ', '\t', '\n', or '\r'.
 breakSpace :: B.ByteString -> (B.ByteString, B.ByteString)
 breakSpace bs = B.break isSpaceWord8 bs
 
@@ -172,50 +123,24 @@ breakSpace bs = B.break isSpaceWord8 bs
 
 {-# INLINE isFunky #-}
 isFunky :: B.ByteString -> Bool
-isFunky ps = case BI.toForeignPtr ps of
-   (x,s,l) ->
-    unsafePerformIO $ withForeignPtr x $ \p->
-    (/=0) `fmap` has_funky_char (p `plusPtr` s) (fromIntegral l)
-
-foreign import ccall unsafe "fpstring.h has_funky_char" has_funky_char
-    :: Ptr Word8 -> CInt -> IO CInt
+isFunky ps = 0 `B.elem` ps || 26 `B.elem` ps
 
 ------------------------------------------------------------------------
 
--- ByteString rewrites break (=='x') to breakByte 'x'
---  break ((==) x) = breakChar x
---  break (==x) = breakChar x
---
-
-{-
-{-# INLINE breakOnPS #-}
-breakOnPS :: Char -> B.ByteString -> (B.ByteString, B.ByteString)
-breakOnPS c p = case BC.elemIndex c p of
-                Nothing -> (p, BC.empty)
-                Just n  -> (B.take n p, B.drop n p)
--}
-
 {-# INLINE hashPS #-}
 hashPS :: B.ByteString -> Int32
-hashPS ps =
-   case BI.toForeignPtr ps of
-   (x,s,l) ->
-    unsafePerformIO $ withForeignPtr x $ \p->
-    hash (p `plusPtr` s) l
+hashPS = B.foldl' hashByte 0
 
-hash :: Ptr Word8 -> Int -> IO Int32
-hash = f (0 :: Int32)
- where f h _ 0 = return h
-       f h p n = do x <- peek p
-                    let !h' =  fromIntegral x + rotateL h 8
-                    f h' (p `advancePtr` 1) (n-1)
+{-# INLINE hashByte #-}
+hashByte :: Int32 -> Word8 -> Int32
+hashByte h x = fromIntegral x + rotateL h 8
 
 {-# INLINE substrPS #-}
 substrPS :: B.ByteString -> B.ByteString -> Maybe Int
 substrPS tok str
     | B.null tok = Just 0
     | B.length tok > B.length str = Nothing
-    | otherwise = do n <- BC.elemIndex (BC.head tok) str
+    | otherwise = do n <- B.elemIndex (B.head tok) str
                      let ttok = B.tail tok
                          reststr = B.drop (n+1) str
                      if ttok == B.take (B.length ttok) reststr
@@ -238,49 +163,31 @@ breakLastPS c p = case BC.elemIndexEnd c p of
                   Nothing -> Nothing
                   Just n -> Just (B.take n p, B.drop (n+1) p)
 
--- TODO: rename
+------------------------------------------------------------------------
+-- linesPS and unlinesPS
+
 {-# INLINE linesPS #-}
 linesPS :: B.ByteString -> [B.ByteString]
 linesPS ps
      | B.null ps = [B.empty]
      | otherwise = BC.split '\n' ps
 
-{- QuickCheck property:
-
-import Test.QuickCheck
-import qualified Data.ByteString.Char8 as BC
-import Data.Char
-instance Arbitrary BC.ByteString where
-    arbitrary = fmap BC.pack arbitrary
-instance Arbitrary Char where
-  arbitrary = chr `fmap` choose (32,127)
-deepCheck = check (defaultConfig { configMaxTest = 10000})
-testLines =  deepCheck (\x -> (linesPS x == linesPSOld x))
-linesPSOld ps = case  BC.elemIndex '\n' ps of
-             Nothing -> [ps]
-             Just n -> B.take n ps : linesPS (B.drop (n+1) ps) -}
-
-{-| This function acts exactly like the "Prelude" unlines function, or like
-"Data.ByteString.Char8" 'unlines', but with one important difference: it will
-produce a string which may not end with a newline! That is:
-
-> unlinesPS ["foo", "bar"]
-
-evaluates to \"foo\\nbar\", not \"foo\\nbar\\n\"! This point should hold true for
-'linesPS' as well.
-
-TODO: rename this function. -}
-unlinesPS :: [B.ByteString] -> B.ByteString
-unlinesPS [] = BC.empty
-unlinesPS x  = BC.init $ BC.unlines x
 {-# INLINE unlinesPS #-}
-{- QuickCheck property:
+unlinesPS :: [B.ByteString] -> B.ByteString
+unlinesPS [] = B.empty
+unlinesPS x  = B.concat $ intersperse (BC.singleton '\n') x
 
-testUnlines = deepCheck (\x -> (unlinesPS x == unlinesPSOld x))
-unlinesPSOld ss = BC.concat $ intersperse_newlines ss
-    where intersperse_newlines (a:b:s) = a : newline : intersperse_newlines (b:s)
-          intersperse_newlines s = s
-          newline = BC.pack "\n" -}
+-- properties of linesPS and unlinesPS
+
+prop_unlinesPS_linesPS_left_inverse :: B.ByteString -> Bool
+prop_unlinesPS_linesPS_left_inverse x = unlinesPS (linesPS x) == x
+
+prop_linesPS_length :: B.ByteString -> Bool
+prop_linesPS_length x = length (linesPS x) == length (BC.elemIndices '\n' x) + 1
+
+prop_unlinesPS_length :: [B.ByteString] -> Bool
+prop_unlinesPS_length xs =
+  B.length (unlinesPS xs) == if null xs then 0 else sum (map B.length xs) + length xs - 1
 
 -- -----------------------------------------------------------------------------
 -- gzReadFilePS
@@ -293,17 +200,12 @@ gzDecompress :: Maybe Int -> BL.ByteString -> ([B.ByteString], Bool)
 gzDecompress mbufsize =
     -- This is what the code would be without the bad CRC recovery logic:
     -- return . BL.toChunks . GZ.decompressWith decompressParams
-#if MIN_VERSION_zlib(0,6,0)
     decompressWarn (ZI.decompressST ZI.gzipFormat decompressParams)
-#else
-    toListWarn . ZI.decompressWithErrors ZI.gzipFormat decompressParams
-#endif
   where
         decompressParams = case mbufsize of
                               Just bufsize -> GZ.defaultDecompressParams { GZ.decompressBufferSize = bufsize }
                               Nothing -> GZ.defaultDecompressParams
 
-#if MIN_VERSION_zlib(0,6,0)
         decompressWarn :: (forall s . ZI.DecompressStream (ST s)) -> BL.ByteString -> ([B.ByteString], Bool)
         decompressWarn = ZI.foldDecompressStreamWithInput
                            (\x ~(xs, b) -> (x:xs, b))
@@ -312,37 +214,18 @@ gzDecompress mbufsize =
                                       else error "trailing data at end of compressed stream"
                            )
                            handleBad
-#else
-        toListWarn :: ZI.DecompressStream -> ([B.ByteString], Bool)
-        toListWarn = foldDecompressStream (\x ~(xs, b) -> (x:xs, b)) ([], False) handleBad
-
-        -- cut and paste from Zlib since it's not currently exported (interface not yet certain)
-        foldDecompressStream :: (B.ByteString -> a -> a) -> a
-                             -> (ZI.DecompressError -> String -> a)
-                             -> ZI.DecompressStream -> a
-        foldDecompressStream chunk end err = fold
-                   where
-                       fold ZI.StreamEnd               = end
-                       fold (ZI.StreamChunk bs stream) = chunk bs (fold stream)
-                       fold (ZI.StreamError code msg)  = err code msg
-#endif
 
         -- For a while a bug in darcs caused gzip files with good data but bad CRCs to be
         -- produced. Trap bad CRC messages, run the specified action to report that it happened,
         -- but continue on the assumption that the data is valid.
-#if MIN_VERSION_zlib(0,6,0)
         handleBad (ZI.DataFormatError "incorrect data check") = ([], True)
         handleBad e = error (show e)
-#else
-        handleBad ZI.DataError "incorrect data check" = ([], True)
-        handleBad _ msg = error msg
-#endif
 
 isGZFile :: FilePath -> IO (Maybe Int)
 isGZFile f = do
     h <- openBinaryFile f ReadMode
     header <- B.hGet h 2
-    if header /= BC.pack "\31\139"
+    if header /= B.pack [31,139]
        then do hClose h
                return Nothing
        else do hSeek h SeekFromEnd (-4)
@@ -394,7 +277,7 @@ gzReadStdin = do
     rest   <- B.hGetContents stdin
     let allStdin = B.concat [header,rest]
     return $
-     if header /= BC.pack "\31\139"
+     if header /= B.pack [31,139]
       then allStdin
       else let decompress = fst . gzDecompress Nothing
                compressed = BL.fromChunks [allStdin]
@@ -409,12 +292,12 @@ type FileSegment = (FilePath, Maybe (Int64, Int))
 readSegment :: FileSegment -> IO BL.ByteString
 readSegment (f,range) = do
     bs <- tryToRead
-       `catch` (\(_::SomeException) -> do
-                     size <- Bundled.fileSize `fmap` Bundled.getFileStatus f
+       `catchIOError` (\_ -> do
+                     size <- fileSize `fmap` getSymbolicLinkStatus f
                      if size == 0
-                        then return BC.empty
+                        then return B.empty
                         else performGC >> tryToRead)
-    return $ BL8.fromChunks [bs]
+    return $ BL.fromChunks [bs]
   where
     tryToRead =
         case range of
@@ -435,8 +318,7 @@ readSegment (f,range) = do
 -- be written to swap.  If you read many small files, mmapFilePS will be
 -- less memory-efficient than readFilePS, since each mmapFilePS takes up a
 -- separate page of memory.  Also, you can run into bus errors if the file
--- is modified.  NOTE: as with 'readFilePS', the string representation in
--- the file is assumed to be ISO-8859-1.
+-- is modified.
 
 mmapFilePS :: FilePath -> IO B.ByteString
 #if mingw32_HOST_OS
@@ -444,7 +326,7 @@ mmapFilePS = B.readFile
 #else
 mmapFilePS f =
   mmapFileByteString f Nothing
-   `catch` (\(_ :: SomeException) -> do
+   `catchIOError` (\_ -> do
                      size <- fileSize `fmap` getSymbolicLinkStatus f
                      if size == 0
                         then return B.empty
@@ -454,97 +336,85 @@ mmapFilePS f =
 -- -------------------------------------------------------------------------
 -- fromPS2Hex
 
-foreign import ccall unsafe "static fpstring.h conv_to_hex" conv_to_hex
-    :: Ptr Word8 -> Ptr Word8 -> CInt -> IO ()
-
 fromPS2Hex :: B.ByteString -> B.ByteString
-fromPS2Hex ps = case BI.toForeignPtr ps of
-          (x,s,l) ->
-           BI.unsafeCreate (2*l) $ \p -> withForeignPtr x $ \f ->
-           conv_to_hex p (f `plusPtr` s) $ fromIntegral l
+fromPS2Hex = BC.map toLower . b16Enc
 
 -- -------------------------------------------------------------------------
 -- fromHex2PS
 
-foreign import ccall unsafe "static fpstring.h conv_from_hex" conv_from_hex
-    :: Ptr Word8 -> Ptr Word8 -> CInt -> IO ()
-
 fromHex2PS :: B.ByteString -> B.ByteString
-fromHex2PS ps = case BI.toForeignPtr ps of
-          (x,s,l) ->
-           BI.unsafeCreate (l `div` 2) $ \p -> withForeignPtr x $ \f ->
-           conv_from_hex p (f `plusPtr` s) (fromIntegral $ l `div` 2)
+fromHex2PS s =
+  case b16Dec $ BC.map toUpper s of
+    Right (result, remaining)
+      | B.null remaining -> result
+    _ -> error "fromHex2PS: input is not hex encoded"
+
+propHexConversion :: B.ByteString -> Bool
+propHexConversion x = fromHex2PS (fromPS2Hex x) == x
 
 -- -------------------------------------------------------------------------
 -- betweenLinesPS
 
--- | betweenLinesPS returns the B.ByteString between the two lines given,
+-- | Return the B.ByteString between the two lines given,
 -- or Nothing if they do not appear.
-
 betweenLinesPS :: B.ByteString -> B.ByteString -> B.ByteString
                -> Maybe B.ByteString
-betweenLinesPS start end ps
- = case break (start ==) (linesPS ps) of
-       (_, _:rest@(bs1:_)) ->
-           case BI.toForeignPtr bs1 of
-            (ps1,s1,_) ->
-             case break (end ==) rest of
-               (_, bs2:_) -> case BI.toForeignPtr bs2 of (_,s2,_) -> Just $ fromForeignPtr ps1 s1 (s2 - s1)
-               _ -> Nothing
-       _ -> Nothing
+betweenLinesPS start end ps =
+  case B.breakSubstring start_line ps of
+    (before_start, at_start)
+      | not (B.null at_start)
+      , B.null before_start || BC.last before_start == '\n' ->
+          case B.breakSubstring end_line (B.drop (B.length start_line) at_start) of
+            (before_end, at_end)
+              | not (B.null at_end)
+              , B.null before_end || BC.last before_end == '\n' -> Just before_end
+              | otherwise -> Nothing
+      | otherwise -> Nothing
+  where
+    start_line = BC.snoc start '\n'
+    end_line = BC.snoc end '\n'
 
--- -------------------------------------------------------------------------
--- breakAfterNthNewline
-
-breakAfterNthNewline :: Int -> B.ByteString
-                        -> Maybe (B.ByteString, B.ByteString)
-breakAfterNthNewline 0 the_ps | B.null the_ps = Just (B.empty, B.empty)
-                              | otherwise     = Just (B.empty, the_ps)
-breakAfterNthNewline n the_ps =
-  go n (B.elemIndices (BI.c2w '\n') the_ps)
-  where  go 0 []      = Just (the_ps, B.empty)
-         go _ []      = Nothing
-         go 1 (i:_)   = Just $ B.splitAt (i+1) the_ps
-         go !m (_:is) = go (m-1) is
-
--- -------------------------------------------------------------------------
--- breakBeforeNthNewline
-
-breakBeforeNthNewline :: Int -> B.ByteString -> (B.ByteString, B.ByteString)
-breakBeforeNthNewline 0 the_ps
- | B.null the_ps = (B.empty, B.empty)
-breakBeforeNthNewline n the_ps =
-  go n (B.elemIndices (BI.c2w '\n') the_ps)
-  where  go _ []      = (the_ps, B.empty)
-         go 0 (i:_)   = B.splitAt i the_ps
-         go !m (_:is) = go (m-1) is
+-- | Simpler but less efficient variant of 'betweenLinesPS'.
+spec_betweenLinesPS :: B.ByteString -> B.ByteString -> B.ByteString
+                    -> Maybe B.ByteString
+spec_betweenLinesPS start end ps =
+  case break (start ==) (linesPS ps) of
+    (_, _:after_start) ->
+      case break (end ==) after_start of
+        (before_end, _:_) ->
+          Just $ BC.unlines before_end
+        _ -> Nothing
+    _ -> Nothing
 
 -- | Test if a ByteString is made of ascii characters
 isAscii :: B.ByteString -> Bool
 isAscii = B.all (< 128)
 
--- | Decode a ByteString to a String according to the current locale
--- unsafePerformIO in the locale function is ratified by the fact that GHC 6.12
--- and above also supply locale conversion with functions with a pure type.
--- Unrecognized byte sequences in the input are skipped.
+-- * Encoding functions
+
+-- Use of 'unsafePerformIO' is ratified by the fact that these
+-- really are pure functions.
+
+-- | Decode a 'ByteString' containing UTF-8 to a 'String'. Decoding errors
+-- are flagged with the U+FFFD character.
+unpackPSFromUTF8 :: B.ByteString -> String
+unpackPSFromUTF8  = unsafePerformIO . decodeUtf8
+
+-- | Encode a 'String' to a 'ByteString' using UTF-8.
+packStringToUTF8 :: String -> B.ByteString
+packStringToUTF8 = unsafePerformIO . encodeUtf8
+
+-- | Decode a 'ByteString' to a 'String' according to the current locale,
+-- using lone surrogates for un-decodable bytes.
 decodeLocale :: B.ByteString -> String
 decodeLocale = unsafePerformIO . decode
 
--- | Encode a String to a ByteString with char8 encoding (i.e., the values of the
--- characters become the values of the bytes; if a character value is greater
--- than 255, its byte becomes the character value modulo 256)
-encodeChar8 :: String -> B.ByteString
-encodeChar8 = B.pack . map (fromIntegral . ord)
-
--- | Encode a String to a ByteString according to the current locale
+-- | Encode a 'String' to a 'ByteString' according to the current locale,
+-- converting lone surrogates back to the original byte. If that
+-- fails (because the locale does not support the full unicode range)
+-- then encode using utf-8, assuming that the un-ecodable characters
+-- come from patch meta data.
+--
+-- See also 'Darcs.UI.Commands.setEnvCautiously'.
 encodeLocale :: String -> B.ByteString
-encodeLocale = unsafePerformIO . encode
-
--- | Take a 'String' that represents byte values and re-decode it acording to
--- the current locale.
--- Note: we globally enforce char8 as the default encoding, see "Main" and
--- "Darcs.Utils". This means we get command line args and environment variables
--- as 'String's with char8 encoding, too. So we need this to convert such
--- strings back to the user's encoding.
-decodeString :: String -> IO String
-decodeString = decode . encodeChar8
+encodeLocale s = unsafePerformIO $ encode s `catchIOError` (\_ -> encodeUtf8 s)
