@@ -31,6 +31,9 @@ module Darcs.Util.DateMatcher
     -- for debugging only
     , DateMatcher(..)
     , getMatchers
+    -- for testing (GHCi, etc)
+    , testDate
+    , testDateAt
     ) where
 
 import Prelude ()
@@ -47,7 +50,7 @@ import Darcs.Util.IsoDate
     ( parseDate, englishDateTime, englishInterval, englishLast
     , iso8601Interval, resetCalendar, subtractFromMCal, getLocalTz
     , MCalendarTime(..), toMCalendarTime, unsafeToCalendarTime
-    , unsetTime
+    , unsetTime, readUTCDate
     )
 
 -- | 'withinDay' @x y@ is true if @x <= y < (x + one_day)@
@@ -179,6 +182,12 @@ getMatchers d = do
     tillEof p = do { x <- p; eof; return x }
     parseDateWith p = parse (tillEof p) "" d
 
+
+--- The following functions are for toying around in GHCi
+---
+--- > testDate   "2008/05/22 10:34"
+--- > testDateAt "2006-03-22 09:36" "2008/05/22 10:34"
+
 -- | 'tryMatchers' @ms@ returns the first successful match in @ms@
 --   It is an error if there are no matches
 tryMatchers :: [DateMatcher] -> CalendarTime -> Bool
@@ -190,3 +199,32 @@ tryMatchers [] = error "Can't support fancy dates."
 
 now :: IO CalendarTime
 now = getClockTime >>= toCalendarTime
+
+-- | 'testDate' @d@ shows the possible interpretations
+--   for the date string @d@ and how they match against
+--   the current date
+testDate :: String -> IO ()
+testDate d = do cnow <- now
+                testDateAtCal cnow d
+
+-- | 'testDate' @iso d@ shows the possible interpretations
+--   for the date string @d@ and how they match against
+--   the date represented by the ISO 8601 string @iso@
+testDateAt :: String -> String -> IO ()
+testDateAt iso = testDateAtCal (readUTCDate iso)
+
+-- | helper function for 'testDate' and 'testDateAt'
+testDateAtCal :: CalendarTime -> String -> IO ()
+testDateAtCal c d =
+ do ms <- getMatchers d
+    putStr . unlines . map (showMatcher c) $ ms
+
+-- | 'showMatcher' @c dm@ tells us if @dm@ applies to
+--   'CalendarTime' @c@; or if @dm@ just represents the
+--   failure to parse a date, in which case @c@ is moot.
+showMatcher :: CalendarTime -> DateMatcher -> String
+showMatcher cnow (DM n p m) =
+   "==== " ++ n ++ " ====\n" ++
+   (case p of
+     Left err -> shows err ""
+     Right x  -> show x ++ "\n" ++ show (m x cnow))
