@@ -3,6 +3,9 @@ module Darcs.Util.Diff
     , DiffAlgorithm(..)
     ) where
 
+import Prelude ()
+import Darcs.Prelude
+
 import qualified Darcs.Util.Diff.Myers as M ( getChanges )
 import qualified Darcs.Util.Diff.Patience as P ( getChanges )
 import qualified Data.ByteString as B ( ByteString )
