@@ -16,8 +16,6 @@
 -- the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 -- Boston, MA 02110-1301, USA.
 
-{-# LANGUAGE CPP #-}
-
 -- |
 -- Module      : Darcs.Util.Diff.Myers
 -- Copyright   : 2003 David Roundy
@@ -73,8 +71,6 @@ import Data.Array.Base
 import Data.Array.Unboxed
 import qualified Data.Map as Map ( lookup, empty, insertWith )
 
-#include "impossible.h"
-
 -- | create a list of changes between a and b, each change has the form
 --   (starta, lima, startb, limb) which means that a[starta, lima)
 --   has to be replaced by b[startb, limb)
