@@ -11,7 +11,6 @@ import Darcs.Util.Global ( debugFail )
 
 import Darcs.Util.Download.Request ( ConnectionError(..) )
 
-#ifdef HAVE_HTTP
 import Control.Exception ( catch, IOException )
 import Data.IORef ( newIORef, readIORef, writeIORef, IORef )
 import Network.HTTP
@@ -23,7 +22,6 @@ import Darcs.Util.Global ( debugMessage )
 import qualified Data.ByteString as B
 import qualified Data.ByteString.Char8 as BC
 import Version ( version )
-#endif
 
 fetchUrl :: String -> IO String
 postUrl
@@ -35,8 +33,6 @@ postUrl
 requestUrl :: String -> FilePath -> a -> IO String
 waitNextUrl :: IO (String, String, Maybe ConnectionError)
 
-#ifdef HAVE_HTTP
-
 headers :: [Header]
 headers =  [Header HdrUserAgent $ "darcs-HTTP/" ++ version]
 
@@ -102,13 +98,3 @@ waitNextUrl = do
                        _         -> Nothing
              return (u, e, ce)
     where h = return . ioeGetErrorString
-
-#else
-
-fetchUrl _ = debugFail "Network.HTTP does not exist"
-postUrl _ _ _ = debugFail "Cannot use http POST because darcs was not compiled with Network.HTTP."
-
-requestUrl _ _ _ = debugFail "Network.HTTP does not exist"
-waitNextUrl = debugFail "Network.HTTP does not exist"
-
-#endif
