@@ -23,27 +23,47 @@
 -- USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 {-# LANGUAGE CPP #-}
 module Darcs.Util.Encoding
-    ( encode,decode
+    ( encode, decode
+    , encodeUtf8, decodeUtf8
     ) where
 
 import Prelude ()
 import Darcs.Prelude
 
-import Data.ByteString ( ByteString )
+import qualified Data.ByteString as B
+import GHC.IO.Encoding ( TextEncoding, mkTextEncoding )
+import GHC.Foreign ( withCStringLen, peekCStringLen )
 
 #ifdef WIN32
-import qualified Darcs.Util.Encoding.Win32 as Backend ( encode, decode )
+import Darcs.Util.Encoding.Win32 ( encode, decode )
 #else
-import qualified Darcs.Util.Encoding.IConv as Backend ( encode, decode )
+
+import GHC.IO.Encoding ( getFileSystemEncoding )
+
+-- | Encode a 'String' into a 'B.ByteString' according to the user's locale
+-- with the ghc specific //ROUNDTRIP feature added. This means the argument
+-- is allowed to contain non-Unicode 'Char's as produced by 'decode'.
+encode :: String -> IO B.ByteString
+encode s = getFileSystemEncoding >>= textEncode s
+
+-- | Decode a 'B.ByteString' into a 'String' according to the user's locale
+-- with the ghc specific //ROUNDTRIP feature added. This means the result
+-- may contain 'Char's that are not valid Unicode in case decoding with the
+-- user's locale fails.
+decode :: B.ByteString -> IO String
+decode bs = getFileSystemEncoding >>= textDecode bs
+
 #endif
 
--- functions redefined to add haddock (there might well be a better way!)
+encodeUtf8 :: String -> IO B.ByteString
+encodeUtf8 s = mkTextEncoding "UTF-8//TRANSLIT" >>= textEncode s
 
--- | Encode a Unicode 'String' into a 'ByteString' suitable for the current
--- console.
-encode :: String -> IO ByteString
-encode = Backend.encode
+decodeUtf8 :: B.ByteString -> IO String
+decodeUtf8 bs = mkTextEncoding "UTF-8//TRANSLIT" >>= textDecode bs
+
+textEncode :: String -> TextEncoding -> IO B.ByteString
+textEncode s enc = withCStringLen enc s B.packCStringLen
+
+textDecode :: B.ByteString -> TextEncoding -> IO String
+textDecode bs enc = B.useAsCStringLen bs (peekCStringLen enc)
 
--- | Convert a 'ByteString' from the console's encoding into a Unicode 'String'.
-decode :: ByteString -> IO String
-decode = Backend.decode
