@@ -26,7 +26,7 @@
 -- Portability : portable
 
 module Darcs.Util.IsoDate
-    ( getIsoDateTime, readUTCDate
+    ( getIsoDateTime, readUTCDate, readUTCDateOldFashioned
     , parseDate, getLocalTz
     , englishDateTime, englishInterval, englishLast
     , iso8601Interval, iso8601Duration
@@ -34,6 +34,7 @@ module Darcs.Util.IsoDate
     , MCalendarTime(..), subtractFromMCal, addToMCal
     , toMCalendarTime, unsafeToCalendarTime
     , unsetTime, TimeInterval
+    , showIsoDateTime
     ) where
 
 import Prelude ( (^) )
@@ -45,7 +46,7 @@ import System.IO.Unsafe ( unsafePerformIO )
 import Data.Char ( toUpper, isDigit )
 import Data.Maybe ( fromMaybe )
 import Control.Monad ( liftM, liftM2 )
-import qualified Data.ByteString.Char8 as B
+import qualified Data.ByteString.Char8 as BC
 
 type TimeInterval = (Maybe CalendarTime, Maybe CalendarTime)
 
@@ -77,25 +78,36 @@ readDate tz d =
              Left e -> error $ "bad date: "++d++" - "++show e
              Right ct -> resetCalendar $ unsafeToCalendarTime ct
 
+-- | Similar to 'readUTCDate', except we /ignore/ timezone info
+-- in the input string. This is incorrect and ugly. The only reason
+-- it still exists is so we can generate file names for old-fashioned
+-- repositories in the same way that old darcs versions expected them.
+-- You should not use this function except for the above stated purpose.
+readUTCDateOldFashioned :: String -> CalendarTime
+readUTCDateOldFashioned d = 
+             case parseDate 0 d of
+             Left e -> error $ "bad date: "++d++" - "++show e
+             Right ct -> (unsafeToCalendarTime ct) { ctTZ = 0 }
+
 -- | Parse a date string, assuming a default timezone if
 --   the date string does not specify one.  The date formats
 --   understood are those of 'showIsoDateTime' and 'dateTime'
 parseDate :: Int -> String -> Either ParseError MCalendarTime
 parseDate tz d =
-              if length d >= 14 && B.all isDigit bd
+              if length d >= 14 && BC.all isDigit bd
               then Right $ toMCalendarTime $
-                   CalendarTime (readI $ B.take 4 bd)
-                                (toEnum $ (+ (-1)) $ readI $ B.take 2 $ B.drop 4 bd)
-                                (readI $ B.take 2 $ B.drop 6 bd) -- Day
-                                (readI $ B.take 2 $ B.drop 8 bd) -- Hour
-                                (readI $ B.take 2 $ B.drop 10 bd) -- Minute
-                                (readI $ B.take 2 $ B.drop 12 bd) -- Second
+                   CalendarTime (readI $ BC.take 4 bd)
+                                (toEnum $ (+ (-1)) $ readI $ BC.take 2 $ BC.drop 4 bd)
+                                (readI $ BC.take 2 $ BC.drop 6 bd) -- Day
+                                (readI $ BC.take 2 $ BC.drop 8 bd) -- Hour
+                                (readI $ BC.take 2 $ BC.drop 10 bd) -- Minute
+                                (readI $ BC.take 2 $ BC.drop 12 bd) -- Second
                                 0 Sunday 0 -- Picosecond, weekday and day of year unknown
                                 "GMT" 0 False
               else let dt = do { x <- dateTime tz; eof; return x }
                    in parse dt "" d
-  where bd = B.pack (take 14 d)
-        readI s = fst $ fromMaybe (error "parseDate: invalid date") (B.readInt s)
+  where bd = BC.pack (take 14 d)
+        readI s = fst $ fromMaybe (error "parseDate: invalid date") (BC.readInt s)
 
 -- | Display a 'CalendarTime' in the ISO 8601 format without any
 --   separators, e.g. 20080825142503
