@@ -54,7 +54,7 @@ import Prelude ()
 import Darcs.Prelude
 
 import Data.List ( inits )
-import Data.Maybe ( isJust, listToMaybe )
+import Data.Maybe ( fromJust, isJust, listToMaybe )
 import System.Exit ( exitWith, ExitCode(..) )
 import System.IO
     ( withFile, withBinaryFile, openBinaryTempFile
