@@ -106,7 +106,7 @@ import System.IO.Error ( isDoesNotExistError )
 import qualified Darcs.Util.Workaround as Workaround ( getCurrentDirectory )
 import qualified System.Directory ( setCurrentDirectory )
 import System.Directory ( doesDirectoryExist, doesFileExist )
-import qualified System.FilePath.Posix as FilePath ( normalise )
+import qualified System.FilePath.Posix as FilePath ( normalise, isRelative )
 import qualified System.FilePath as NativeFilePath ( takeFileName, takeDirectory )
 import System.FilePath( (</>), splitDirectories, normalise, dropTrailingPathSeparator )
 import System.Posix.Files ( isDirectory, getSymbolicLinkStatus )
@@ -498,6 +498,12 @@ setCurrentDirectory = System.Directory.setCurrentDirectory . toFilePath
   * Would it be worth adding a --semi-safe-paths option for allowing
     changes to certain preference files (_darcs\/prefs\/) in sub
     repositories'?
+
+  TODO:
+    Properly review the way we handle paths on Windows - it's not enough
+    to just use the OS native concept of path separator. Windows often
+    accepts both path separators, and repositories always use the UNIX
+    separator anyway.
 -}
 isMaliciousPath :: String -> Bool
 isMaliciousPath fp =
@@ -508,7 +514,7 @@ isMaliciousPath fp =
 --   don't start with ./
 isMaliciousSubPath :: String -> Bool
 isMaliciousSubPath fp =
-    not (isRelative fp) || isGenerallyMalicious fp
+    not (FilePath.isRelative fp) || isGenerallyMalicious fp
 
 isGenerallyMalicious :: String -> Bool
 isGenerallyMalicious fp =
