@@ -52,9 +52,10 @@ module Darcs.Util.Printer
     , insertBeforeLastline
     , prefixLines
     , invisiblePS, userchunkPS
-    -- * Rendering
-    , RenderMode(..)
-    , renderString, renderStringWith, renderPS, renderPSWith
+    -- * Rendering to 'String'
+    , renderString, renderStringWith
+    -- * Rendering to 'ByteString'
+    , renderPS, renderPSWith
     , renderPSs, renderPSsWith
     -- * Printers
     , Printers
@@ -70,14 +71,14 @@ module Darcs.Util.Printer
     , blueText, redText, greenText, magentaText, cyanText
     , colorText
     , lineColor
-    -- * IO
+    -- * IO, uses 'Data.ByteString.hPut' for output
     , hPutDoc,     hPutDocLn,     putDoc,     putDocLn
     , hPutDocWith, hPutDocLnWith, putDocWith, putDocLnWith
     , hPutDocCompr
     , debugDocLn
     , ePutDocLn
     , errorDoc
-    -- * Unsafe constructors
+    -- * TODO: It is unclear what is unsafe about these constructors
     , unsafeText, unsafeBoth, unsafeBothText, unsafeChar
     , unsafePackedString
     ) where
@@ -85,17 +86,12 @@ module Darcs.Util.Printer
 import Prelude ()
 import Darcs.Prelude
 
-import Control.Exception ( throwIO, ErrorCall(..) )
 import Data.String ( IsString(..) )
-import Data.List (intersperse)
-import Data.Monoid ( (<>) )
-import GHC.Stack ( currentCallStack )
-import System.IO (Handle, stdout, stderr, hPutStr)
-import System.IO.Unsafe ( unsafePerformIO )
-import qualified Data.ByteString as B (ByteString, hPut, concat)
-import qualified Data.ByteString.Char8 as BC (unpack, pack, singleton)
+import System.IO ( Handle, stdout, stderr )
+import qualified Data.ByteString as B ( ByteString, hPut, concat )
+import qualified Data.ByteString.Char8 as BC ( singleton )
 
-import Darcs.Util.ByteString ( linesPS, encodeLocale, gzWriteHandle )
+import Darcs.Util.ByteString ( linesPS, decodeLocale, encodeLocale, gzWriteHandle )
 import Darcs.Util.Global ( debugMessage )
 
 -- | A 'Printable' is either a String, a packed string, or a chunk of
@@ -146,74 +142,71 @@ parens d = lparen <> d <> rparen
 
 -- | Fail with a stack trace and the given 'Doc' as error message.
 errorDoc :: Doc -> a
-errorDoc x = unsafePerformIO $ do
-   stack <- currentCallStack
-   throwIO $ ErrorCall $ renderString Encode $ x $$ vcat (map text stack)
+errorDoc x = error $ renderString x
 
--- | 'putDocWith' puts a doc on stdout using the given printer.
+-- | 'putDocWith' puts a 'Doc' on stdout using the given printer.
 putDocWith :: Printers -> Doc -> IO ()
-putDocWith prs = hPutDocWith prs Encode stdout
+putDocWith prs = hPutDocWith prs stdout
 
--- | 'putDocLnWith' puts a doc, followed by a newline on stdout using
+-- | 'putDocLnWith' puts a 'Doc', followed by a newline on stdout using
 -- the given printer.
 putDocLnWith :: Printers -> Doc -> IO ()
-putDocLnWith prs = hPutDocLnWith prs Encode stdout
+putDocLnWith prs = hPutDocLnWith prs stdout
 
--- | 'putDoc' puts a doc on stdout using the simple printer 'simplePrinters'.
+-- | 'putDoc' puts a 'Doc' on stdout using the simple printer 'simplePrinters'.
 putDoc :: Doc -> IO ()
-putDoc = hPutDoc Encode stdout
+putDoc = hPutDoc stdout
 
--- | 'putDocLn' puts a doc, followed by a newline on stdout using
+-- | 'putDocLn' puts a 'Doc', followed by a newline on stdout using
 -- 'simplePrinters'
 putDocLn :: Doc -> IO ()
-putDocLn = hPutDocLn Encode stdout
+putDocLn = hPutDocLn stdout
 
--- | 'eputDocLn' puts a doc, followed by a newline to stderr using
+-- | 'eputDocLn' puts a 'Doc', followed by a newline to stderr using
 -- 'simplePrinters'. Like putDocLn, it encodes with the user's locale.
 -- This function is the recommended way to output messages that should
 -- be visible to users on the console, but cannot (or should not) be
 -- silenced even when --quiet is in effect.
 ePutDocLn :: Doc -> IO ()
-ePutDocLn = hPutDocLn Encode stderr
+ePutDocLn = hPutDocLn stderr
 
--- | 'hputDocWith' puts a doc on the given handle using the given printer.
-hPutDocWith :: Printers -> RenderMode -> Handle -> Doc -> IO ()
-hPutDocWith prs target h d = hPrintPrintables target h (renderWith (prs h) d)
+-- | 'hputDocWith' puts a 'Doc' on the given handle using the given printer.
+hPutDocWith :: Printers -> Handle -> Doc -> IO ()
+hPutDocWith prs h d = hPrintPrintables h (renderWith (prs h) d)
 
--- | 'hputDocLnWith' puts a doc, followed by a newline on the given
+-- | 'hputDocLnWith' puts a 'Doc', followed by a newline on the given
 -- handle using the given printer.
-hPutDocLnWith :: Printers -> RenderMode -> Handle -> Doc -> IO ()
-hPutDocLnWith prs target h d = hPutDocWith prs target h (d <?> newline)
+hPutDocLnWith :: Printers -> Handle -> Doc -> IO ()
+hPutDocLnWith prs h d = hPutDocWith prs h (d <?> newline)
 
--- |'hputDoc' puts a doc on the given handle using 'simplePrinters'
-hPutDoc :: RenderMode -> Handle -> Doc -> IO ()
+-- |'hputDoc' puts a 'Doc' on the given handle using 'simplePrinters'
+hPutDoc :: Handle -> Doc -> IO ()
 hPutDoc = hPutDocWith simplePrinters
 
--- | 'hputDocLn' puts a doc, followed by a newline on the given handle using
+-- | 'hputDocLn' puts a 'Doc', followed by a newline on the given handle using
 -- 'simplePrinters'.
-hPutDocLn :: RenderMode -> Handle -> Doc -> IO ()
+hPutDocLn :: Handle -> Doc -> IO ()
 hPutDocLn = hPutDocLnWith simplePrinters
 
 -- | like 'hPutDoc' but with compress data before writing
-hPutDocCompr :: RenderMode -> Handle -> Doc -> IO ()
-hPutDocCompr target h = gzWriteHandle h . renderPSs target
+hPutDocCompr :: Handle -> Doc -> IO ()
+hPutDocCompr h = gzWriteHandle h . renderPSs
 
 -- | Write a 'Doc' to stderr if debugging is turned on.
 debugDocLn :: Doc -> IO ()
-debugDocLn = debugMessage . renderString Standard
+debugDocLn = debugMessage . renderString
 
 -- | @'hPrintPrintables' h@ prints a list of 'Printable's to the handle @h@
-hPrintPrintables :: RenderMode -> Handle -> [Printable] -> IO ()
-hPrintPrintables target h = mapM_ (hPrintPrintable target h)
+-- It uses binary output of 'ByteString's. If these not available,
+-- converts according to locale.
+hPrintPrintables :: Handle -> [Printable] -> IO ()
+hPrintPrintables h = mapM_ (hPrintPrintable h)
 
 -- | @'hPrintPrintable' h@ prints a 'Printable' to the handle @h@.
-hPrintPrintable :: RenderMode -> Handle -> Printable -> IO ()
-hPrintPrintable Standard h (S ps) = hPutStr h ps
-hPrintPrintable Encode  h (S ps) = B.hPut h (encodeLocale ps)
-hPrintPrintable Standard h (PS ps) = B.hPut h ps
-hPrintPrintable Encode  h (PS ps) = B.hPut h ps
-hPrintPrintable Standard h (Both _ ps) = B.hPut h ps
-hPrintPrintable Encode  h (Both _ ps) = B.hPut h ps
+hPrintPrintable :: Handle -> Printable -> IO ()
+hPrintPrintable h (S ps) = B.hPut h (encodeLocale ps)
+hPrintPrintable h (PS ps) = B.hPut h ps
+hPrintPrintable h (Both _ ps) = B.hPut h ps
 
 -- | A 'Doc' is a bit of enriched text. 'Doc's are concatenated using
 -- '<>' from class 'Monoid', which is right-associative.
@@ -224,22 +217,7 @@ newtype Doc = Doc { unDoc :: St -> Document }
 instance IsString Doc where
    fromString = text
 
--- TODO this is a rather ad-hoc hack that further complicates
--- some already confusing code. We should find a more general
--- solution. See the discussion on issue1639.
--- | Used when rendering a 'Doc' to indicate if the result
--- should be encoded to the current locale or left alone.
--- In practice this only affects output when a relevant
--- DARCS_DONT_ESCAPE_XXX option is set (see Darcs.Util.Printer.Color)
--- If in doubt, choose 'Standard'.
-data RenderMode =
-      Encode       -- ^Encode Strings with the current locale.
-                   -- At present ByteStrings are assumed to be in
-                   -- UTF8 and are left alone, so will be mis-encoded
-                   -- in non-UTF8 locales.
-    | Standard     -- ^Don't encode.
-
--- | The State associated with a doc. Contains a set of printers for each
+-- | The State associated with a 'Doc'. Contains a set of printers for each
 -- hanlde, and the current prefix of the document.
 data St = St { printers :: !Printers',
                currentPrefix :: !([Printable] -> [Printable]) }
@@ -259,54 +237,50 @@ type Printer = Printable -> St -> Document
 data Color = Blue | Red | Green | Cyan | Magenta
 
 -- | 'Document' is a wrapper around '[Printable] -> [Printable]' which allows
--- for empty Documents. The simplest 'Documents' are built from 'String's
--- using 'text'.
+-- to handle the special case of an empty 'Document' in a non-uniform manner.
+-- The simplest 'Documents' are built from 'String's using 'text'.
 data Document = Document ([Printable] -> [Printable])
               | Empty
 
 -- | renders a 'Doc' into a 'String' with control codes for the
--- special features of the doc.
-renderString :: RenderMode -> Doc -> String
+-- special features of the 'Doc'.
+renderString :: Doc -> String
 renderString = renderStringWith simplePrinters'
 
 -- | renders a 'Doc' into a 'String' using a given set of printers.
-renderStringWith :: Printers' -> RenderMode -> Doc -> String
-renderStringWith prs target d = concatMap (toString target) $ renderWith prs d
-    where toString Standard (S s) = s
-          toString Encode  (S s) = BC.unpack . encodeLocale $ s
-          toString Standard (PS ps) = BC.unpack ps
-          toString Encode  (PS ps) = BC.unpack ps
-          toString Standard (Both s _) = s
-          toString Encode  (Both s _) = BC.unpack . encodeLocale $ s
+-- If content is only available as 'ByteString', decode according to
+-- the current locale.
+renderStringWith :: Printers' -> Doc -> String
+renderStringWith prs d = concatMap (toString) $ renderWith prs d
+    where toString (S s) = s
+          toString (PS ps) = decodeLocale ps
+          toString (Both s _) = s
 
 -- | renders a 'Doc' into 'B.ByteString' with control codes for the
 -- special features of the Doc. See also 'readerString'.
-renderPS :: RenderMode -> Doc -> B.ByteString
+renderPS :: Doc -> B.ByteString
 renderPS = renderPSWith simplePrinters'
 
 -- | renders a 'Doc' into a list of 'PackedStrings', one for each line.
-renderPSs :: RenderMode -> Doc -> [B.ByteString]
+renderPSs :: Doc -> [B.ByteString]
 renderPSs = renderPSsWith simplePrinters'
 
--- | renders a doc into a 'B.ByteString' using a given set of printers.
-renderPSWith :: Printers' -> RenderMode -> Doc -> B.ByteString
-renderPSWith prs target d = B.concat $ renderPSsWith prs target d
+-- | renders a 'Doc' into a 'B.ByteString' using a given set of printers.
+renderPSWith :: Printers' -> Doc -> B.ByteString
+renderPSWith prs d = B.concat $ renderPSsWith prs d
 
 -- | renders a 'Doc' into a list of 'PackedStrings', one for each
--- chunk of text that was added to the doc, using the given set of
+-- chunk of text that was added to the 'Doc', using the given set of
 -- printers.
-renderPSsWith :: Printers' -> RenderMode -> Doc -> [B.ByteString]
-renderPSsWith prs target d = map (toPS target) $ renderWith prs d
-    where toPS Standard (S s)        = BC.pack s
-          toPS Encode  (S s)        = encodeLocale s
-          toPS Standard (PS ps)      = ps
-          toPS Encode  (PS ps)      = ps
-          toPS Standard (Both _ ps)  = ps
-          toPS Encode  (Both _ ps)  = ps
+renderPSsWith :: Printers' -> Doc -> [B.ByteString]
+renderPSsWith prs d = map toPS $ renderWith prs d
+    where toPS (S s)        = encodeLocale s
+          toPS (PS ps)      = ps
+          toPS (Both _ ps)  = ps
 
 -- | renders a 'Doc' into a list of 'Printables' using a set of
 -- printers. Each item of the list corresponds to a string that was
--- added to the doc.
+-- added to the 'Doc'.
 renderWith :: Printers' -> Doc -> [Printable]
 renderWith ps (Doc d) = case d (initState ps) of
                         Empty -> []
@@ -327,21 +301,16 @@ prefix s (Doc d) = Doc $ \st ->
 -- of the Doc abstraction
 prefixLines :: Doc -> Doc -> Doc
 prefixLines prefixer prefixee =
-   vcat $ map (prefixer <+>) $ map packedString $ linesPS $
-   -- this will just get round-tripped back into a Doc,
-   renderPS Standard prefixee
+  vcat $ map (prefixer <+>) $ map packedString $ linesPS $ renderPS prefixee
 
 -- TODO try to find another way to do this, it's rather a violation
 -- of the Doc abstraction
 insertBeforeLastline :: Doc -> Doc -> Doc
 insertBeforeLastline a b =
-   -- as this will just get round-tripped back into a Doc,
-   -- we use 'Standard' as the Target type so the encoding
-   -- is left alone
-   case reverse $ map packedString $ linesPS $ renderPS Standard a of
-   (ll:ls) -> vcat (reverse ls) $$ b $$ ll
-   [] -> error "empty Doc given as first argument of Printer.insert_before_last_line"
-
+  case reverse $ map packedString $ linesPS $ renderPS a of
+    (ll:ls) -> vcat (reverse ls) $$ b $$ ll
+    [] ->
+      error "empty Doc given as first argument of Printer.insert_before_last_line"
 
 lineColor :: Color -> Doc -> Doc
 lineColor c d = Doc $ \st -> case lineColorT (printers st) c d of
@@ -350,7 +319,7 @@ lineColor c d = Doc $ \st -> case lineColorT (printers st) c d of
 hiddenPrefix :: String -> Doc -> Doc
 hiddenPrefix s (Doc d) =
     Doc $ \st -> let pr = printers st
-                     p = S (renderStringWith pr Standard $ hiddenText s)
+                     p = S (renderStringWith pr $ hiddenText s)
                      st' = st { currentPrefix = currentPrefix st . (p:) }
                  in case d st' of
                       Document d'' -> Document $ (p:) . d''
@@ -364,7 +333,7 @@ unsafeBoth s ps = Doc $ simplePrinter (Both s ps)
 -- | 'unsafeBothText' builds a 'Doc' from a 'String'. The string is stored in the
 -- Doc as both a String and a 'B.ByteString'.
 unsafeBothText :: String -> Doc
-unsafeBothText s = Doc $ simplePrinter (Both s (BC.pack s))
+unsafeBothText s = Doc $ simplePrinter (Both s (encodeLocale s))
 
 -- | 'packedString' builds a 'Doc' from a 'B.ByteString' using 'printable'
 packedString :: B.ByteString -> Doc
@@ -478,6 +447,7 @@ invisiblePrinter _ = unDoc empty
 
 infixr 6 `append`
 infixr 6 <+>
+infixr 5 `vplus`
 infixr 5 $$
 
 -- | The empty 'Doc'
@@ -487,6 +457,9 @@ empty = Doc $ const Empty
 doc :: ([Printable] -> [Printable]) -> Doc
 doc f = Doc $ const $ Document f
 
+instance Semigroup Doc where
+  (<>) = append
+
 -- | 'mappend' ('<>') is concatenation, 'mempty' is the 'empty' 'Doc'
 instance Monoid Doc where
   mempty = empty
@@ -511,7 +484,7 @@ Doc a <?> Doc b =
                                                      Empty -> s
                                                      Document bf -> bf s)
 
--- | @a '<+>' b@ is @a@ followed by a space, then @b@
+-- | @a '<+>' b@ is @a@ followed by @b@ with a space in between if both are non-empty
 (<+>) :: Doc -> Doc -> Doc
 Doc a <+> Doc b =
     Doc $ \st -> case a st of
@@ -533,15 +506,25 @@ Doc a $$ Doc b =
                         where pf = currentPrefix st
                               sf = lineColorS $ printers st
 
+-- | @vplus a b@ is @a@ above @b@ with an empty line in between if both are non-empty
+vplus :: Doc -> Doc -> Doc
+Doc a `vplus` Doc b =
+   Doc $ \st -> case a st of
+                Empty -> b st
+                Document af ->
+                    Document (\s -> af $ case b st of
+                                         Empty -> s
+                                         Document bf -> sf (newlineP:newlineP:pf (bf s)))
+                        where pf = currentPrefix st
+                              sf = lineColorS $ printers st
+
 -- | Pile 'Doc's vertically
 vcat :: [Doc] -> Doc
-vcat [] = empty
-vcat ds = foldr1 ($$) ds
+vcat = foldr ($$) empty
 
 -- | Pile 'Doc's vertically, with a blank line in between
 vsep :: [Doc] -> Doc
-vsep [] = empty
-vsep ds = foldr1 ($$) $ intersperse (text "") ds
+vsep = foldr vplus empty
 
 -- | Concatenate 'Doc's horizontally
 hcat :: [Doc] -> Doc
