@@ -1,6 +1,3 @@
-{-# LANGUAGE CPP #-}
-
-
 -- |
 -- Module      : Darcs.Util.Progress
 -- Copyright   : 2008 David Roundy
@@ -30,7 +27,7 @@ module Darcs.Util.Progress
 
 
 import Prelude ()
-import Darcs.Prelude hiding ( lookup )
+import Darcs.Prelude
 
 import Prelude hiding (lookup)
 
