@@ -27,6 +27,7 @@ import Darcs.Util.Path
 import Darcs.Util.Tree
 
 import Control.Applicative( (<$>) )
+import Control.Exception ( throw )
 
 import Data.List( sortBy )
 import Data.Int( Int64 )
@@ -216,7 +217,7 @@ instance (Monad m) => TreeRO (TreeMonad m) where
            t <- gets tree
            let f = findFile t p'
            case f of
-             Nothing -> fail $ "No such file " ++ show p'
+             Nothing -> throw $ userError $ "No such file " ++ show p'
              Just x -> lift (readBlob x)
 
     currentDirectory = ask
@@ -251,7 +252,7 @@ instance (Monad m) => TreeRW (TreeMonad m) where
            let item = find tr from'
                found_to = find tr to'
            unless (isNothing found_to) $
-                  fail $ "Error renaming: destination " ++ show to ++ " exists."
+                  throw $ userError $ "Error renaming: destination " ++ show to ++ " exists."
            unless (isNothing item) $ do
                   modifyItem from Nothing
                   modifyItem to item
