@@ -1,5 +1,3 @@
-{-# LANGUAGE CPP #-}
-
 {-
 Copyright (C) 2004 David Roundy
 
@@ -64,8 +62,6 @@ import Data.Char ( isSpace )
 import qualified System.FilePath as FP ( isRelative, isAbsolute, isValid )
 import System.FilePath ( (</>) )
 
-#include "impossible.h"
-
 isRelative :: String -> Bool
 isRelative "" = bug "Empty filename in isRelative"
 isRelative f  = FP.isRelative f
