@@ -2,7 +2,8 @@
 
 const char *curl_request_url(const char *url,
                              const char *filename,
-                             int cache_time);
+                             int cache_time,
+                             int *errorCode);
 
 const char *curl_wait_next_url(int *errorCode, long* httpErrorCode);
 
