@@ -3,10 +3,9 @@
 
 
 # issue701
-rm -rf temp0 temp1 temp2
 
 # step 1
-darcs init temp0 --darcs-2
+darcs init temp0
 cd temp0
 echo m1 > foo
 darcs record -lam m1
@@ -59,5 +58,3 @@ darcs pull -a
 echo m2-a1-m4 > foo
 echo y | darcs mark-conflicts
 cd ..
-
-rm -rf temp0 temp1 temp2
