@@ -6,7 +6,7 @@
 . lib                  # Load some portability helpers.
 
 check_patches_order () {
-    darcs changes | tr -d "\n" | grep $1.*$2.*$3
+    darcs log | tr -d "\n" | grep $1.*$2.*$3
 }
 
 test_init () {
@@ -16,8 +16,7 @@ test_init () {
     cd R1
 
     touch "r1_0"
-    darcs add "r1_0"
-    darcs record -a --author=me -m "Adding r1_0" "r1_0"
+    darcs record -lam "Adding r1_0"
 
     cd ..
 
@@ -26,17 +25,15 @@ test_init () {
     cd R2
 
     touch "r2_0"
-    darcs add "r2_0"
-    darcs record -a --author=me -m "Adding r2_0" "r2_0"
-    darcs send --no-minimize --author=me -a --no-edit-description -o ../R1/P 
+    darcs record -lam "Adding r2_0"
+    darcs send --no-minimize -a --no-edit-description -o ../R1/P 
 
     cd ..
 
     cd R1
 
     touch "r1_1"
-    darcs add "r1_1"
-    darcs record -a --author=me -m "Adding r1_1" "r1_1"
+    darcs record -lam "Adding r1_1"
     
     cd ..
 }
