@@ -13,7 +13,7 @@ for cmd in $all_commands; do
   grep disable log
   rm log
   # --disable works from defaults
-  sub_commands="$(darcs $cmd --list-options | grep -v -- -- || true)"
+  sub_commands="$(darcs $cmd --list-options | grep -v -- -- | cut -f ';' -f 1 || true)"
   # disabling super commands in the defaults file is broken
   if test -z "$sub_commands"; then
     echo "$cmd --disable" > _darcs/prefs/defaults
