@@ -1,8 +1,7 @@
 #!/usr/bin/env bash
 
-. lib
+. ./lib
 
-set -ev
 switch_to_latin9_locale
 rm -rf temp1
 rm -rf temp2
@@ -25,7 +24,8 @@ darcs init
 cd ../temp1
 
 # long email adress: check that email adresses of <= 77 chars don't get split up
-darcs send --from="Kjlt berstrm $seventysevenaddy" \
+darcs send --mail\
+           --from="Kjlt berstrm $seventysevenaddy" \
            --subject "Un patch pour le rpositoire" \
            --to="Un garon franais <garcon@francais.fr>" \
            --sendmail-command='tee mail_as_file %<' \
