@@ -1,5 +1,5 @@
 #!/usr/bin/env bash
-set -ev
+. ./lib
 
 # See issue1327.
 # results in the error:
@@ -24,4 +24,4 @@ darcs unrecord -p "Add fileA and fileB" --all
 darcs record --author foo@bar --ignore-times --all fileA -m "Add just fileA"
 cd ../temp1
 darcs pull --all ../temp2
-echo y | darcs obliterate --dont-prompt-for-dependencies -p "Add fileA and fileB"
+echo yy | darcs obliterate --dont-prompt-for-dependencies -p "Add fileA and fileB"
