@@ -1,7 +1,5 @@
 #!/usr/bin/env bash
-set -ev
-
-. lib
+. ./lib
 
 rm -rf temp1 temp1a temp1b
 
