@@ -25,14 +25,8 @@
 ## CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 ## SOFTWARE.
 
-#pragma repo-format darcs-2
-
 . lib
 
-## This bug only affects darcs-2 repositories.
-fgrep darcs-2 ~/.darcs/defaults &>/dev/null || exit 200
-
-rm -rf d e                      # Another script may have left a mess.
 darcs initialize --repodir d/
 darcs initialize --repodir e/
 touch d/f d/g e/f
@@ -40,7 +34,6 @@ darcs record     --repodir d/ -lam 'Add f and g'
 darcs record     --repodir e/ -lam 'Add f'
 echo >d/f
 darcs record     --repodir d/ -am 'Change f'
+darcs pull       --repodir e/ -a d/ --allow-conflicts #no conflict mark-up
+echo y | darcs obliterate --repodir e/ -ap 'Add f and g'
 darcs pull       --repodir e/ -a d/
-darcs obliterate --repodir e/ -ap 'Add f and g'
-darcs pull       --repodir e/ -a d/
-rm -rf d/ e/                    # Clean up after ourselves.
