@@ -37,9 +37,7 @@ darcs push      --repo R S -a   # Try to push patches between repos.
 darcs push      --repo S R
 rm -rf R/ S/                    # Clean up after ourselves.
 #!/bin/sh
-set -ev
-
-. lib
+. ./lib
 
 rm -rf temp1
 darcs init --repodir temp1
@@ -53,7 +51,7 @@ echo "b" >> a
 echo "y" | not darcs amend-record --all --patch=p1
 
 # There should be one patch in the repo
-test 1 -eq `darcs changes --count` || exit 1
+test 1 -eq `darcs changes -a --count` || exit 1
 
 # Another check: there should be nothing new after a is restored
 echo "a" > a
@@ -62,24 +60,3 @@ not darcs whatsnew -l
 cd ..
 rm -rf test1
 
-# the following section of the test does not create a new repo,
-# it oprates on an existing old-fashioned repo; no need to run it 3 times
-if grep 'old-fashioned' .darcs/defaults; then
-
-    # check that we do the operations on checkpoints tentatively
-    rm -rf old-with-checkpoint
-    gunzip -c $TESTDATA/old-with-checkpoint.tgz | tar xf -
-
-    echo "test exit 1" > old-with-checkpoint/_darcs/prefs/prefs
-
-    # this amend will fail
-    echo 'y' | not darcs amend --repo old-with-checkpoint -m 'no longer a checkpoint :P' -p "checkpoint here"
-
-    # check that the patch has not been removed from
-    # the checkpoints inventory
-    grep "checkpoint here" old-with-checkpoint/_darcs/checkpoints/inventory
-    #NOTE: checking the file is a rather white box way of testing it, but
-    #      I could not think of any other way at the time, please improve
-
-    rm -rf old-with-checkpint
-fi
