@@ -2,10 +2,12 @@
 
 . ./lib
 
-rm -rf temp1 temp2
+# this test fails for darcs-1 repos
+skip-formats darcs-1
+
 mkdir temp1
 cd temp1
-darcs init --darcs-2
+darcs init
 echo first > a
 darcs add a
 darcs record -am 'first'
@@ -28,5 +30,3 @@ cd temp1
 darcs pull -a ../temp2 | grep conflict
 grep third a
 cd ..
-
-rm -rf temp1 temp2
