@@ -60,9 +60,9 @@ for i in 1 2 3 4 5 6 7 8 9; do
     darcs record -a -m "change b $i"
 done
 
-echo no | darcs send --all --to=random@random --sendmail-command=false ../temp1
-echo yes | not darcs send --all --to=random@random --sendmail-command=false ../temp1
-not darcs send --all --to=random@random --sendmail-command=false --allow-unrelated-repos ../temp1
+echo no | darcs send --mail --all --to=random@random --sendmail-command=false ../temp1
+echo yes | not darcs send --mail --all --to=random@random --sendmail-command=false ../temp1
+not darcs send --mail --all --to=random@random --sendmail-command=false --allow-unrelated-repos ../temp1
 cd ..
 
 rm -rf temp1 temp2
