@@ -23,9 +23,7 @@
 ## CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 ## SOFTWARE.
 
-set -ev
-
-. lib
+. ./lib
 
 cacheDir=$HOME/.darcs/cache
 
@@ -33,4 +31,4 @@ rm -rf R S
 darcs init  --repo R
 darcs get R S
 not grep "$cacheDir" S/_darcs/prefs/sources
-not grep "cache:" S/_darcs/prefs/sources
\ No newline at end of file
+not grep "cache:" S/_darcs/prefs/sources
