@@ -59,8 +59,8 @@ ln -s ./non-recorded-file2 ./Non-Recorded-File2
 ln -s "`pwd`"/non-recorded-file2 ./Non-ReCoRdEd-File2
 darcs w -l >log 2>&1                                             # should report only "non-recorded-file"
 darcs rec -alm "added ./non-recorded-file2" >>log 2>&1            # should add only file, not symlink
-darcs changes -s --patch="added ./non-recorded-file2" >>log 2>&1  # should report only file, not symlink
-not grep -vE "(^ *$|^\+|[0-9]:[0-9][0-9]:[0-9]|./non-recorded-file2)" log
+darcs changes -s --patches="added ./non-recorded-file2" >>log 2>&1  # should report only file, not symlink
+not grep -vE "(^patch|^Author|^ *$|^\+|[0-9]:[0-9][0-9]:[0-9]|./non-recorded-file2)" log
 rm Non-Recorded-File2 ./Non-ReCoRdEd-File2
 
 # Case 16: case-folding link to recorded file
