@@ -34,5 +34,5 @@ darcs apply foo.dpatch
 cd ..
 
 cd R
-not darcs pull -a ../S > log 2>&1
+not darcs pull -a ../S 2>&1 | tee log
 cd ..
