@@ -1,7 +1,5 @@
 #!/usr/bin/env bash
 
-#pragma repo-format darcs-2
-
 . lib
 
 mkdir future
@@ -17,10 +15,6 @@ EOF
 cat _darcs/format
 cd ..
 
-if grep darcs-1 .darcs/defaults; then
-    exit 200
-fi
-
 # get future repo: should be ok
 darcs get future temp1
 cd temp1
@@ -29,5 +23,3 @@ touch toto
 darcs add toto
 darcs record -am 'blah'
 cd ..
-rm -rf temp1 future
-
