@@ -1,5 +1,5 @@
 #!/usr/bin/env bash
-set -ev
+. lib
 
 rm -rf tempc
 mkdir tempc
