@@ -5,10 +5,12 @@
 
 . lib
 
-rm -rf temp1 temp_a temp_b temp_c temp_d
+# this test fails for darcs-1 repos
+skip-formats darcs-1
+
 mkdir temp1
 cd temp1
-darcs init --darcs-2
+darcs init
 echo 0 > f
 darcs add f
 darcs record -am 00
@@ -38,5 +40,3 @@ darcs pull -a ../temp_c > log
 not grep -i "no remote" log
 not grep -i get_extra log
 cd ..
-
-rm -rf temp1 temp_a temp_b temp_c temp_d log
