@@ -1,5 +1,5 @@
 #!/usr/bin/env bash
-set -ev
+. ./lib
 
 # for issue381: "darcs send -o message --edit-description doesn't work"
 
