@@ -1,14 +1,13 @@
 #!/usr/bin/env bash
 
-#pragma repo-format darcs-2
-
 . ./lib
 
-rm -rf temp1 temp2
+# this test fails in the darcs 1 format
+skip-formats darcs-1
+
 mkdir temp1
 cd temp1
-# this test fails in the darcs 1 format
-darcs init --darcs-2
+darcs init
 echo A > f
 darcs add f
 darcs record --ignore-times -a -m A
@@ -29,5 +28,3 @@ darcs record --ignore-times -a -m B-A
 (darcs push -a || :) 2> push-result
 grep "Refusing to apply" push-result
 cd ..
-
-rm -rf temp1 temp2
