@@ -1,6 +1,6 @@
 #!/bin/sh
 
-set -ev
+. ./lib
 
 ## I would use the builtin !, but that has the wrong semantics.
 not () { "$@" && exit 1 || :; }
