@@ -1,7 +1,5 @@
 #!/bin/sh
-set -ve
-
-. lib
+. ./lib
 
 rm -rf temp1
 mkdir temp1
