@@ -1,5 +1,5 @@
 #!/usr/bin/env bash
-set -ev
+. ./lib
 rm -rf temp1 temp2
 mkdir temp1
 cd temp1
@@ -13,7 +13,7 @@ mkdir temp2
 cd temp2
 darcs init
 darcs pull ../temp1 -a
-darcs optimize --compress
+darcs optimize compress
 for f in _darcs/patches/*-*; do
 	gzip -t < "$f"
 done
