@@ -21,12 +21,11 @@
 ## CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 ## SOFTWARE.
 
-#pragma repo-format darcs-2
-
 . lib
-grep darcs-2 $HOME/.darcs/defaults || exit 200
 
-rm -rf R S
+# this test fails for darcs-1 repos
+skip-formats darcs-1
+
 darcs init --repo R
 darcs init --repo S
 
