@@ -1,5 +1,5 @@
 #!/usr/bin/env bash
-set -ev
+. ./lib
 
 rm -rf fooOld tempA tempB
 mkdir fooOld tempA tempB
@@ -53,5 +53,3 @@ cd ..
 # At this point, tempB and tempA should agree since we have pulled both ways.
 cmp tempB/foo tempA/foo
 
-rm -rf fooOld tempA tempB
-
