@@ -4,21 +4,13 @@
 # Tests that darcs clone --verbose reports getting a pack when there is one,
 # and does not report when there is none or when --no-packs is passed.
 
-#pragma repo-format darcs-1,darcs-2
-
 . lib
 
-rm -rf S
+skip-formats darcs-1 # compressed repo is darcs-2
 
-if grep darcs-1 .darcs/defaults; then
-format=hashed
-elif grep darcs-2 .darcs/defaults; then
-format=darcs-2
-else format=ERROR; fi
+gunzip -c $TESTDATA/laziness-complete.tgz | tar xf -
 
-gunzip -c $TESTDATA/many-files--${format}.tgz | tar xf -
-
-cd many*
+cd repo
 
 darcs optimize http
 test -e _darcs/packs/basic.tar.gz
@@ -28,13 +20,13 @@ cd ..
 serve_http # sets baseurl
 
 # check that default behaviour is to get packs
-darcs clone $baseurl/many-files--${format} S --verbose |grep "Cloning packed basic repository"
+darcs clone $baseurl/laziness-complete S --verbose |grep "Cloning packed basic repository"
 
 # check that it does really not get packs when --no-packs is passed
 rm -rf S
-darcs clone $baseurl/many-files--${format} S --no-packs --verbose  |not grep "Cloning packed basic repository"
+darcs clone $baseurl/laziness-complete S --no-packs --verbose  |not grep "Cloning packed basic repository"
 
 # check that it does not clam getting packs when there are not
 rm -rf S
-rm -rf many-files--${format}/_darcs/packs/
-darcs clone $baseurl/many-files--${format} S --verbose |not grep "Cloning packed basic repository"
+rm -rf laziness-complete/_darcs/packs/
+darcs clone $baseurl/laziness-complete S --verbose |not grep "Cloning packed basic repository"
