@@ -14,3 +14,11 @@ darcs optimize reorder| grep -i "done"
 cd ..
 
 rm -rf temp1
+
+## issue2388 - optimize fails if no patches have been recorded
+
+darcs init temp1
+cd temp1
+darcs optimize clean
+cd ..
+rm -rf temp1
