@@ -1,5 +1,5 @@
 #!/usr/bin/env bash
-set -ev
+. ./lib
 
 rm -rf temp1 temp2 temp3
 mkdir temp2
@@ -13,24 +13,24 @@ darcs init
 touch foo bar
 darcs add foo bar
 darcs record -a -m foobar -A author
-darcs send -a --dont-edit-description -o ../temp3 ../temp2
+darcs send --no-minimize -a --dont-edit-description -o ../temp3 ../temp2
 test -f ../temp3
 
-darcs send -a --dont-edit-description -o ../temp2/patchfile ../temp2
+darcs send --no-minimize -a --dont-edit-description -o ../temp2/patchfile ../temp2
 test -f ../temp2/patchfile
 
 mkdir subdir
-darcs send -a --dont-edit-description -o subdir/../../temp2/patchfile1 ../temp2
+darcs send --no-minimize -a --dont-edit-description -o subdir/../../temp2/patchfile1 ../temp2
 test -f ../temp2/patchfile1
 
 cp ../temp3 correct
 
 cd subdir
-darcs send -a --dont-edit-description -o ../patchfile ../../temp2
+darcs send --no-minimize -a --dont-edit-description -o ../patchfile ../../temp2
 diff -u ../patchfile ../correct
 
 rm ../patchfile
-darcs send -a --set-default --dont-edit-description -o - ../../temp2 > ../patchfile
+darcs send --no-minimize -a --set-default --dont-edit-description -o - ../../temp2 > ../patchfile
 grep -v Creating ../patchfile | diff -u ../correct -
 
 darcs apply --repodir=../../temp2 --dry-run ../patchfile > out
