@@ -32,32 +32,29 @@
 ## SOFTWARE.
 
 . lib
-rm -rf R S
 
+rm -rf R
 darcs init --repo R
 cd R
 darcs show patch-index | grep 'Patch Index is not yet created.' # init
-
 cd ..
-rm -rf R
 
+rm -rf R
 darcs init --repo R --with-patch-index
 cd R
 darcs show patch-index | grep 'Patch Index is in sync with repo.' # init --patch-index
-
 cd ..
 
+rm -rf S
 darcs clone R S
 cd S
 darcs show patch-index | grep 'Patch Index is not yet created.' # clone
-
 cd ..
-rm -rf S
 
+rm -rf S
 darcs clone R S --with-patch-index
 cd S
 darcs show patch-index | grep 'Patch Index is in sync with repo.' # clone --patch-index
-
 cd ..
 
 rm -rf repo repo2
@@ -65,10 +62,9 @@ gunzip -c $TESTDATA/simple-v1.tgz | tar xf -
 echo 'I understand the consequences of my action' | darcs convert darcs-2 repo repo2 
 cd repo2
 darcs show patch-index | grep 'Patch Index is not yet created.' # convert
-
 cd ..
-rm -rf repo repo2
 
+rm -rf repo repo2
 gunzip -c $TESTDATA/simple-v1.tgz | tar xf -
 echo 'I understand the consequences of my action' | darcs convert darcs-2 repo repo2 --with-patch-index
 cd repo2
@@ -96,4 +92,3 @@ darcs log -a d/f --no-patch-index
 darcs show patch-index | grep 'Patch Index is not yet created.' # log -a file --no-patch-index
 
 cd ..
-rm -rf R S repo repo2
