@@ -1,8 +1,6 @@
 #!/usr/bin/env bash
 
-set -ev
-
-. lib
+. ./lib
 
 rm -rf temp1
 mkdir temp1
