@@ -12,11 +12,11 @@ touch d/t.t
 darcs add d/t.t
 darcs record -am "initial record"
 echo 'some change' >> d/t.t
-echo n | darcs record --interactive > log
+echo ny | darcs record --interactive > log
 # pre-test: plain text files are not binary
 not grep binary log
 echo 'd/t' >>  _darcs/prefs/binaries
-echo n | darcs record --interactive > log
+echo ny | darcs record --interactive > log
 grep binary log
 cd ..
 
