@@ -1,6 +1,6 @@
 #!/usr/bin/env bash
 
-set -ev
+. ./lib
 
 # Check that the right patches get pulled using --dont-prompt-for-dependencies
 
@@ -22,12 +22,12 @@ darcs record -Ax -alm foo2
 echo bar2 > b
 darcs record -Ax -alm bar2
 cd ../temp2
-echo y | darcs pull ../temp1 -i --dont-prompt-for-dependencies -p foo2 --dry-run > toto
+echo yy | darcs pull ../temp1 -i --dont-prompt-for-dependencies -p foo2 --dry-run > toto
 #on the previous line, we get asked about foo2, and we take it
 grep foo2 toto | wc -l | grep 2
 #we don't get asked about foo1, but we take it anyway, so 
 grep foo1 toto | wc -l | grep 1
 #and we don't send bar
-grep bar toto | wc -l | grep 0
+not grep bar toto
 cd ..
 rm -rf temp1 temp2
