@@ -4,16 +4,11 @@
 # For issue855: wish: avoid taking lock if using --dry-run
 chmod -R u+w temp2 || :
 rm -rf temp1 temp2
-mkdir temp1
-cd temp1
-darcs init
-cd ..
-mkdir temp2
+darcs init temp1
+darcs init temp2
 cd temp2
-darcs init
 touch x
-darcs add x
-darcs record -am "test"
+darcs record -lam test
 cd ..
 chmod -R u-w temp2
 cd temp2
