@@ -26,4 +26,4 @@
 
 rm -rf R && darcs init --repo R
 cd R
-runhaskell $TESTBIN/renameHelper.hs
+runghc $TESTBIN/renameHelper.hs
