@@ -1,5 +1,5 @@
 #!/usr/bin/env bash
-set -ev
+. ./lib
 
 DARCS_EDITOR=echo
 export DARCS_EDITOR
