@@ -29,7 +29,7 @@ cat > ref <<EOF
 1.0
 EOF
 
-cmp my ref
+diff --strip-trailing-cr my ref
 cd ..
 
 rm -rf temp1
